FUNCTION dispArray
   /* 
     ------------------------------------------------------------------
     Programmer..: Roland Wingerter
     Date........: 13 March 2003
     Notes.......: Display array elements in result pane.
                   Can be used to display nested arrays. 
     Written for.: dBase Plus
     Rev. History: None
     Calls.......: dispArray_aux()
     Usage.......: dispArray( oArray, [lShowType] ) 
     Example.....: a = {1,2,{31,32,{331}},"X",date()}
                   dispArray( a ) or dispArray ( a, true )
     Returns.....: true if successful
     Parameter...: oArray - array to display 
                   lShowType - true/false. If true, the data type 
                               of each element is displayed as well. 
     ------------------------------------------------------------------
   */
   parameters oArray, lShowType

   if argcount() = 1
      lShowType = false
   endif

   // First argument must be a one-dimensional array. 
   if type("oArray") # 'A'
      msgbox( "No array passed!", "Error", 16 )
      return -1
   elseif oArray.dimensions > 1
      msgbox( "Can't process multidimensional array!", "Error", 16 )
      return-1
   endif

   clear

   // Call auxiliary function
   dispArray_aux ( oArray, 0 )

return true


FUNCTION dispArray_aux 
   /*
   -----------------------------------------------------------------------
   Programmer..: Roland Wingerter
   Date........: 13 Mar 2003
   Notes.......: Auxiliary function for dispArr() which does the 
                 actual work. 
   Calls.......: showit()
                 For nested arrays, the function calls itself recursively. 
   Written for.: dBASE PLUS
   Rev. History: None
   ---------------------------------------------------------------------
   */
   parameters xValue, nIndex

   // xValue   = data element (any data type or array)
   // nIndex   = index of data element

   local i

   if type("xValue") # 'A'
     // We have a simple data element
      showit( xValue, nIndex, aIndex, lShowType )
   else
      // We have an array 

      // Check if aIndex exists
      // aIndex is used for storing numbers which represent 
      // the current array index, 
      // e.g. arr[1][2] is stored as aIndex={1,2}
      if type("aIndex") = 'A'
         // Update contents of aIndex 
         aIndex.add(nIndex)
         // Show it on screen
         showit( xValue, nIndex, aIndex, lShowType )
      else
         // create empty array
         aIndex = new array()
      endif

      for i=1 to xValue.Size
         //** Recursive call for each element in xValue! **
         dispArray_aux ( xValue[i], i )
      endfor

      // Remove last element from aIndex
      if aIndex.Size >= 2
         aIndex.resize(aIndex.size-1)
      else
         aIndex = new array()
      endif
   endif

return 


FUNCTION showit
   /*
   -----------------------------------------------------------------------
   Programmer..: Roland Wingerter
   Date........: 13 Mar 2003
   Notes.......: Auxiliary function for displaying array 
                 index and data.   
   Calls.......: None
   Called from.: arrDisp_aux()
   Written for.: dBASE PLUS
   Rev. History: None
   ---------------------------------------------------------------------
   */
   parameters xValue, nIndex, aIndex, lShowType

   local cIndex, n, xData

   // Build string representing the position of the 
   // current data element in the array
   // e.g. if aIndex={1,2} then cIndex = "a[1][2]"
   cIndex = "a"
   for n=1 to aIndex.size
      cIndex += "[" + ltrim(str(aIndex[n])) + "]"
   endfor

   // If not an array, add Index of current element.
   if type("xValue") # 'A'
      cIndex += "[" + ltrim(str(nIndex)) + "]"
   endif
          
   // Put character data between delimiters
   xData = iif( type("xValue")='C', [']+xValue+['], xValue )

   // Now display it
   if lShowType
      // Display index, data element and data type
      ? cIndex, "=", xData, "("+type("xValue")+")"
   else
      // Display index and data element
      ? cIndex, "=", xData
   endif
return



