/*
    GetTableLevel
    Returns the level set in the BDE registry setting

    Author: Vic McClung
    Date..: August, 2000
    
    Updated: August, 2015 -- Based on information provided by
             Ivar B. Jessen, this now looks for Registry.prg
             in the appropriate directory location based on
             the version of dBASE Plus being used. Remember
             that if using this in multiple versions of dBASE
             you will need to recompile it or the version
             information will be wrong ...
             
             This code MAY require that dBASE be opened with
             dBASE being run "As Administrator" ... OR check
             the permissions using Regedit on your computer
             (Windows program) for the keys to see if permissions
             allow access -- for some reason, upgrading Windows
             may not automatically update the permissions in the
             Registry:
             
             HKEY_LOCAL_MACHINE
                SOFTWARE
                   Borland
                      Database Engine
                         Settings
                            DRIVERS
                               DBASE
                                  TABLE CREATE
                                  
             What I (KM) found was selecting each level, right-
             clicking (select "Permissions", and setting "Full" 
             permissions for each set of users allows this to 
             work ... strange.

    Usage:

       ? GetTableLevel()

    Dependencies:

       registry.prg in the samples directory
       winreg.h     in the include directory

*/
#INCLUDE <winreg.h>

function getTableLevel()
   local reg, cLevel
   // use registry.prg in samples directory, based on
   // version of dBASE:
   if __version__ < 2.8
      set procedure to _dbwinhome+"samples\registry.prg" additive
   elseif __version__ < 10
      set procedure to ( left(_app.currentUserPath, len( _app.currentUserPath)-3)+;
                       "samples\registry.prg" )
   else // dBASE Plus 10 and later
      set procedure to (_app.userDocumentsPath+"\samples\registry.prg")
   endif
   
   // create registry object
   reg = new Registry(HKEY_LOCAL_MACHINE, ;
         "SOFTWARE\Borland\Database Engine\Settings" + ;
         "\DRIVERS\DBASE\TABLE CREATE" )
   cLevel = reg.queryValue("LEVEL")
   
   reg.close()
   
   // try to get this info using Windows Scripting Host
   // (thanks to Berard Mouille) -- if the registry code
   // didn't work:
   if type( "cLevel" ) == "U"
      oWsh = new OleAutoClient( "WScript.Shell" )
      cKey = "HKEY_LOCAL_MACHINE\SOFTWARE\Borland\"+;
             "Database Engine\Settings\DRIVERS\DBASE\TABLE CREATE\LEVEL"
      cLevel = oWsh.RegRead( cKey )
   endif
return cLevel
// done
