* Funclib.prg
* Functions that are employed in a variety of applications.
* By Peter Rorlick et al

CLASS MBSString(cString) of String

	function CharIsDigit(cChar)
   	if cChar = null or empty(cChar)
      	return false
      endif
 	return (cChar $ "0123456789")

	function StringIsNumber(cString)
   	if cString = null or empty(cString)
      	return false
      endif
   	local lStringIsNumber, i
      lStringIsNumber = true
		for i = 1 to len(cString)
      	if not this.charIsDigit(subs(cString, i, 1))
         	lStringIsNumber := false
            exit
         endif
      next
  	return lStringIsNumber

ENDCLASS
**************************************************************
* class junk  // uncomment this and the ENDCLASS to get the left pane in alphabetical order.
**************************************************************
Proc BrowseModal( cFields )   // param is optional
do BrowseMod.wfm with .t., cFields
retu
***************************************************************
FUNC DateToString(dDate)   // Returns 11 chars, e.g. "1998 Jan 31"
IF EMPTY(dDate)
  RETU SPACE(11)
ENDIF
*RETU YEAR(dDate)+' '+LEFT(translate(CMONTH(dDate)),3)+' '+DAY(dDate) + IIF(Day(dDate)<10,' ','')
RETU YEAR(dDate)+' '+LEFT(CMONTH(dDate),3)+' '+DAY(dDate) + IIF(Day(dDate)<10,' ','')
************************************************************
FUNC eDate(P_DATE)   // returns a variable-length charstring containing an
                     // English date, e.g. "August 30, 1998"
RETU CMONTH(P_DATE)+' '+DAY(P_DATE)+', '+YEAR(P_DATE)
************************************************************
FUNC eDateLong(P_Date)  // Returns a variable-length charstring containing an
                        // English date, e.g. "Tuesday, August 30, 1998"
RETU CDOW(P_Date)+', '+cMonth(P_DATE)+' '+DAY(P_DATE)+', '+YEAR(P_DATE)
***************************************************************
FUNC eDate11(dDate)   // Returns 11 chars, e.g. "1998 Jan 31"
Return DateToString(dDate)
***************************************************************
FUNC EDate14(dDate)   // Returns a 14-char English date, e.g. "Aug. 25, 1998 " or "April 25, 1998"
                      // Nice for grids and browses.
If empty(dDate)
  RETU SPACE(14)
Endif
local cMonth
If Month(dDate) > 2 and Month(dDate) < 8
  * Don't abbreviate the month:
  cMonth = cMonth(dDate)
Else
  * Abbreviate the month to 3 chars:
  cMonth = Left(cMonth(dDate),3) + '.'
Endif
RETURN Left(cMonth +' '+ DAY(dDate) +', '+ YEAR(dDate)+'   ',14)
*********************************************************************
FUNC AllTrim(cString)
RETU LTRIM(RTRIM(cString))
*********************************************************************
/*
* Saveli's version:
Function StrTran( cString, cSearch, cRepl)     && Like Clipper's built-in function.

LOCAL nAt, cString1, nHandle  // Hanlde - Numeric value that indicate number of character in string that already parsed...
Priv cReplace

local cReturnString

cReturnString = cString

cReplace=cRepl

IF TYPE('cReplace') <> 'C'  && If this optional parameter was omitted...
  cReplace = ""  && Omiting the 3rd parameter means "remove all instances of the searh string".
ENDIF

nHandle = 0
cString1 = cReturnString

nAt = AT(cSearch,cString1)
DO WHILE nAt > 0
  cString1 = STUFF(cString1, nAt, LEN(cSearch), cReplace)
  cReturnString =  Left(cReturnString, nHandle) + cString1
  nHandle = nHandle + nAt + Len(cReplace)-1
  cString1 = SubStr(cReturnString, nHandle+1,Len(cReturnString) - nHandle)
ENDTRY
  nAt = AT(cSearch,cString1)
ENDDO
RETURN cReturnString
*/
*-----------------------------------------------------------------
* Pete's version:
Function StrTran( cTargetString, cSearch, cReplace)     && Like Clipper's built-in function.

if argCount() < 3
  cReplace = ""  && Omiting the 3rd parameter means "remove all instances of the searh string".
endif

local cString, nAt
cString = cTargetString
nAt = AT( cSearch, cString )
DO WHILE nAt > 0
  cString = STUFF(cString, nAt, LEN(cSearch), cReplace)
  nAt = AT( cSearch, cString )
ENDDO
RETURN cString
***********************************************************
FUNC RandomFileName    && Example of cExt: "DBF" or ".DBF"
PARAM cExt
IF TYPE('cExt') # 'C'
  cExt = '.'
ENDIF
IF not '.'$cExt
  cExt = '.' + cExt
ENDIF
/*
LOCAL cFileName
cFileName = 'T_'+RIGHT(STR(RANDOM(),8,6),6) + cExt
DO WHILE FILE(cFileName)
  cFileName = 'T_'+RIGHT(STR(RANDOM(),8,6),6) + cExt
ENDDO
RETU cFileName
*/
return funique( "T_??????" + upper(cExt) )
*****************************************************************************
FUNC Expandmac( cStringToexpand )
private Toexpand
Toexpand = cStringToexpand  // Make a copy of it, so we can leave the parameter intact.
*
* This routine expands a memo or long character string, substituting macro-
* expanded values for any |macro|s in the text.
* A "Macro" is defined here as a substring delimited on both sides by vertical
* bars, and the macro MUST evaluate to a string. Very little error-checking
* is performed here, so if a macro is not expandable, the program will crash.
* Hard and soft returns within macros are ignored.
*
*  Example :     "Today is |DateToString(DATE())|."
*
Txtwitxpma = ''  && Text with expanded macro: nil to begin with
DO WHILE .NOT.EMPTY( Toexpand )
 IF AT( "|", Toexpand ) > 0
  DO WHILE AT( "|", Toexpand ) > 0
   Macbeg = AT( "|" , Toexpand )
   Macend = AT( "|" , SUBS(Toexpand , Macbeg + 1) )
   Macval = SUBS( Toexpand, Macbeg + 1, Macend - 1)
   Txtwitxpma = Txtwitxpma + SUBS( Toexpand, 1, Macbeg - 1 )
   * Strip hard and soft returns from MacVal:
   MacVal = RemoveCharsFromString( MacVal, CHR(141)+CHR(13)+CHR(10) )
   IF TYPE(Macval) # 'C'
     Txtwitxpma = Txtwitxpma + '**MACRO-ERROR**'   && This should never happen, but just to be safe...
   ELSE
     Txtwitxpma = Txtwitxpma + &Macval
   ENDI
   Toexpand = SUBS( Toexpand, Macbeg + Macend + 1)
  ENDD
 ELSE    && We have a text without macro, or expanded text + end of text normal.
   IF EMPTY(Txtwitxpma)
     Txtwitxpma = Toexpand
   ELSE
     Txtwitxpma = Txtwitxpma +Toexpand
   ENDI
   Toexpand = ''
 ENDI
ENDD               && .not.EMPTY(Toexpand)
RETU Txtwitxpma
*************************************************
FUNC RemChars( cChars2Rem, cFromList )  //S new function  7-08-93  1:01:30 pm
/*
Returns a string freed from every specific character in the string cChars2Rem
*/
LOCAL i
IF EMPTY( cChars2Rem ) .OR. EMPTY(  cFromList )
 RETU cFromList
ENDI
FOR i = 1 TO LEN( cChars2Rem )
 cFromList = STRTRAN( cFromList, SUBS( cChars2Rem, i, 1 ), '' )
NEXT
RETU cFromList
*************************************************************
FUNC CreatePublicVariable(cVarName,gValue,cExpToReturn)    // Callable from code blocks!
PRIV cName
cName = cVarName    // Must be private for macro-expansion
PUBLIC &cName
&cName = gValue
RETU ''
**************************************************************
FUNC TypeOfExpression( cExpression )
/*
  Evaluates cExpression as a macro; if it crashes during the evaluation,
  this func returns 'U', else it returns the type of the evaluated expression.
  This allows us to test expressions containing UDFs or functions from EXTEND.LI

  Note that TYPE() is crash-proof, but returns 'U' if the expression contains
  a UDF or an Extend.LIB function, even if the expression is valid!

  Also note that VALTYPE() properly returns the type of any valid expression,
  but it will cause a crash if that expression is not valid!

  TypeOfExpression() is the best of both worlds, being crash-proof and
  returning the correct type of any expression.

   Use this func instead of TYPE() or VALTYPE() to check if the expression
   passed as parameter is macro expandable.
*/
PRIV sEval  // we can not macro expand a local variable!
PRIV sResult

sEval = cExpression

Try
 sResult=&sEval
 Return TYPE("sResult")// was VALTYPE(&sEval)  // This is a clipper function
Catch (Exception e)
 * ExMsgBox(sEval,"There is an error in the expression",16)
 Return 'U'
EndTry
************************************************
Function LocalType   // Call this without the quotes to Type() a local var.
Param gValue
Return Type('gValue')
***********************************************************
PROC RefreshBrowse(oBrowseControl,oForm)

if argCount() > 1   // if form param was passed...
  Freeze(oForm)
endif

oBrowseControl.Visible = .F.
local cAlias
cAlias = oBrowseControl.Alias
oBrowseControl.Alias = cAlias   && Resetting this property refreshes the browse display.
* Watch out: The pointers in any child tables may move as a result!
* Also, resetting the .Alias is quite slow (one second on a Pentium over a network).

* The "visibility" method will repair a messed-up header but will not refresh the data lines:
oBrowseControl.Visible = .T.

if argCount() > 1   // if form param was passed...
  UnFreeze()
endif

return
***********************************************************
Proc SwapThisRowWithNext(oRowset)
SwapRows(oRowset,1)
return
***********************************************************
Proc SwapThisRowWithPrevious(oRowset)
SwapRows(oRowset,-1)
return
***********************************************************
Proc SwapRows(oRowset,nSkip)

  /*
    This does not work with BLOB fields (pictures, sounds, etc).
    For those, we'd need something like:
       COPY BINARY ... TO &cFileName
       .....
       REPLACE BINARY ..... FROM &cFileName
  */

* Swaps the values of the current row with the next (1) or previous (-1) row.
* Usually this proc is called when you want to "move" a row up or down
* in a grid.  For example:
*   Form.DownButton.OnClick={;SwapThisRowWithNext(Form.Rowset)}
* The row pointer ends up row that looks as though it was moved.
* Note: only works when there is no active index.

local aValues, aValues2, numFields, i, b1, b2

if oRowset.EndOfSet
  return   // do nothing
endif

b1 = oRowset.BookMark()
if not oRowset.Next(nSkip)  // if end of set (or beginning of set) is reached
  oRowset.Goto(b1)
  return   // do nothing
endif
b2 = oRowset.BookMark()

numFields = oRowset.Fields.Size

oRowset.Goto(b1)
aValues = new Array()
for i = 1 to numFields
  aValues.Add(oRowset.Fields[i].Value)
next

oRowset.Goto(b2)
aValues2 = new Array()
for i = 1 to numFields
  aValues2.Add(oRowset.Fields[i].Value)
next

for i = 1 to numFields
  oRowset.Fields[i].Value = aValues[i]
next
oRowset.Goto(b1)
for i = 1 to numFields
  oRowset.Fields[i].Value = aValues2[i]
next
oRowset.Goto(b2)   // End up on the row that looks like it has been "moved".

oRowset.RefreshControls()
return
*******************************************************
FUNCTION MDXbyte
 *----------------------------------------
 *-- Programmer..: Bowen Moursund (CIS: 72662,436)
 *-- Date........: 05/21/1992
 *-- Notes.......: Sets the MDX byte in a DBF header ON or OFF.
 *--               The DBF must not be open when the function is called.
 *-- Written for.: dBASE IV v1.5
 *-- Rev. History: 05/21/1992 -- Original
 *-- Calls.......: dBASE low level file functions
 *-- Called by...: Any
 *-- Usage.......: MDXbyte(<cDBFpath>,<cOnOff>)
 *-- Example.....: lByteSet = MDXbyte("mydbf.dbf","OFF")
 *-- Returns.....: .T. if successful
 *-- Parameters..: cDBFpath = the <path>filename.ext of the DBF
 *--               cOnOff   = "ON" or "OFF"
 *----------------------------------------------------------------------

   parameters cDBFpath,cOnOff
   private all

   cOnOff = upper(m->cOnOff)
   * check the validity of the parameters
   lSuccess = ( pcount() = 2 .AND. m->cOnOff $ "ON|OFF" .AND. ;
                file(m->cDBFpath) )
   if lSuccess
      nHandle = fopen(m->cDBFpath,"RW")
      if m->nHandle > 0
         if fseek(m->nHandle, 28) = 28
            lSuccess = ( fwrite(m->nHandle, iif(m->cOnOff="OFF",;
                                                  chr(0),chr(1))) = 1 )
         else
            lSuccess = .F.
         endif
         lClosed = fclose(m->nHandle)
      else
         lSuccess = .F.
      endif
   endif

 RETURN m->lSuccess
**************************************************************
Func Translate(cString)
*
if type('_app.Language')#'C' or _app.Language=='English' or empty(cString)
  return cString  // Can't translate or no need to translate.
endif
*
* Return the English cString after it has been translated to French or whatever:
local q, r

q = AppQuery('Translat','English')
r = q.rowset
if r.findkey(padr(cString,100))  // Search is case-sensitive.
  if not empty(r.fields[_app.Language].Value)
    return Trim(r.fields[_app.Language].Value)
  endif
else      // not found...
  if type('_app.BuildTranslationTable')=='L' and _app.BuildTranslationTable
    * Append a row to Translat.dbf, and fill in Translat->English:
    * (Somebody will have to type in the French (and other) fields later.)
    r.BeginAppend()
    r.fields['English'].Value = cString
    r.Save()
  endif
endif
return cString   // Not found in translation table, or the target language string is blank.
*
* To turn the switches on/off and set _app.language, you can type: Lang()
*
**************************************************************
Func TranslateToEnglish(cString)
* For now, we'll assume that cString is a French or English string.
* (Might have to redo the logic when we deal with other languages.)
* What we're about to do here is attempt to locate the string in the
* Translat->French column, and if found then return Translat->English,
* else return the original cString.
*
if type('_app.Language')#'C' or _app.Language=='English' or empty(cString)
  return cString  // Can't translate or no need to translate. Assume cString is already English.
endif
*
local q, r

q = AppQuery('Translat','SecondLanguage')
r = q.rowset
if r.findkey(padr(cString,100))  // Search is case-sensitive.
  * French (or other 2nd language) string was found.
  return Trim(r.fields['English'].Value)
else      // not found...
  return cString   // Not desirable, but better than nothing.
endif
******************************************************************
Func TranslateArray
param a
  if Type('a') # 'A'
    return a   // It's not an array!
  endif
  local aResult
  private i
  aResult = new Array()
  for i = 1 to a.size
    do case
    case type('a[i]') == 'C'
      aResult.add( translate(a[i]) )
    case type('a[i]') == 'A'
      aResult.add( TranslateArray(a[i]) )
    otherwise
      aResult.add( a[i] )
    endcase
  next
return aResult
******************************************************************
Func Tmsgbox(cMsg,cTitle,n1,n2)   // multilingual message box
retu ExMsgBox( Translate(cMsg), Translate(cTitle), n1, n2 )
*****************************************************************
Func msgboxFromFile(cFilename,cTitle,n1)   // message box whose message is in a text file
retu MsgBox( MemoRead(cFilename), iif(argcount()>2,cTitle,null), iif(argcount()>2,n1,null) )
*******************************************************************
Func FileInCurrentDirectory(cFile)   // Check for existence of a file in the current dir.
local retVal
private savepath
savepath = set('PATH')
set path to
retVal = File(cFile)
set path to &savepath
Return RetVal
******************************************************************
Func Beep
param nFreq,nDur,cWAVFilename
* Params are optional. First 2 are ignored if cWAVFilename is passed.
if type('cWAVFilename')=='C'   // if this param was passed...
  play sound filename &cWAVFilename   // instead of chr(7)
  return
endif
private cSaveSettings, cNewSettings
cSaveSettings = setTo('BELL')
if type('nFreq')=='N'   // if this param was passed...
  cNewSettings = ltrim(str(nFreq)) + ',' + ltrim(str(nDur))
else
  cNewSettings = '800,20'   // Default settings (applied when no params are passed).
endif
set bell to &cNewSettings
?? chr(7)
set bell to &cSaveSettings
return
**********************************************************************
FUNC StringToArray(cStringArray)
*
*Converts a string array into an array.
*
*Assumptions:
*   cStringArray is valid
*
*FOR example a string array contains an array of report columns definitions
*that is "{{'1','999'},{'Invoices->amt','9,999,999.99'}}".
*
*If an array in a memo field (that starts with a "{") is too long to macro-
*expand: The user could rewrite the memo field so that it is code block
*format, for example:
*  s:={}
*  AADD(s,{a,b,c}),;
*  AADD(s,{a,b,c}),;
*  AADD(s,{a,b,c}),;
*    etc...
*
LOCAL v
IF EMPTY(cStringArray)
   local a
   a = new array()
   RETU a   && return an empty array
ELSEIF LEFT(cStringArray,1) = '{'    && If it's in array format
   * Make an array from the character string skimmed out of soft returns,
   * carriage returns and line feeds:
   PRIV c
   c = RemoveCharsFromString( cStringArray, CHR(141)+CHR(13)+CHR(10) )
   RETU &c
ELSE   && Assume it's a in code block format
   PRIVATE aArray
   aArray = new Array()
   * The existence of aArray allows the code block to refer to this var
   * without having to initialize it - thus the code block could just be
   * a series of aArray.add(...)'s. Alernatively, the code block can return an
   * actual array, without utilizing aArray.

   * Run the code block that is stored in the (memo field) cStringArray:
   cTempFileName = RandomFileName('.TMP')
   SET ALTE TO (cTempFileName)
   SET ALTE ON
   SET MEMOWIDTH TO 255
   ? cStringArray
   CLOSE ALTE
   DO RunCbFil WITH cTempFileName
   ERASE (cTempFileName)

   RETU aArray
ENDIF
******************************************************************************
FUNC RemoveCharsFromString(cString,cCharsToRemove)
LOCAL i, cReturnString
cReturnString = ''
FOR i = 1 TO LEN(cString)
  IF .NOT. SUBS(cString,i,1) $ cCharsToRemove
    cReturnString = cReturnString + SUBS(cString,i,1)
  ENDIF
NEXT
RETURN cReturnString
******************************************************************************
FUNC ArrayToString(aArray) // Can parse one-dimentional arrays & nested but not Multi-Dimentional...
Return AddArrayToString(aArray)
******************************************************************************
FUNC AddArrayToString(aArray)  // It's a sub-function of ArrayToString function..
  local i, cStringToReturn
  priv ii, aaArray
  aaArray = aArray
  cStringToReturn = "{"
  For i=1 to aLen(aArray)
    ii = i

    If i<>1
     cStringToReturn += ","
    EndIf

    If TYPE('aaArray[ii]')=='U'
      cStringToReturn += "NULL"
    ELSEIf TYPE('aaArray[ii]')=='A'
      cStringToReturn += AddArrayToString(aArray[i])
    Else
      cStringToReturn += ValueToString(aArray[i])
    EndIf
  Next
  cStringToReturn += "}"
return cStringToReturn
******************************************************************************
Function ValueToString(oValue) // a="string" -> ValueToString(a) -> "'string'"
  priv ooValue
  ooValue = oValue // For ignoring the problems with TYPE() function...
  DO CASE
    CASE TYPE('ooValue')=='C'
       If AT("'",ooValue)==0
         ooValue = "'" + ooValue +"'"
       ElseIf AT('"',ooValue)==0
         ooValue = '"' + ooValue +'"'
       ElseIf AT(']',ooValue)==0
         ooValue = '[' + ooValue +']'
       Else
         tMsgBox("Can't save process string cause it contains both type of quotes and square brackets.","Error",64)
         MsgBox(ooValue,"Error - Can't handle this string")
       EndIf
     CASE TYPE('ooValue')=='D'
       ooValue = "{" + ooValue +"}"
     OTHERWISE
       ooValue = "" + ooValue
  ENDCASE
Return ooValue
*******************************************************
PROC KillMdxAndOpenExcl
PARAM cDbfPrefix

cDbfPrefix = TRIM(cDbfPrefix)

* Close the table if it's already open, and make sure we're in an unused work area:
IF SELE(cDbfPrefix) > 0
  SELE &cDbfPrefix
ELSE
  SELECT SELECT()
ENDIF
USE

* Find out whether the table is in use in another session or by another user:
* i.e., can we open the file with FOPEN() ?
LOCAL nHandle
DO WHIL .T.
  _app.OpenError = .F.
  ON ERROR _app.OpenError=.T.
  nHandle = FOPEN(cDbfPrefix+'.DBF')    && Under the default error handler, this will fail with "access denied" Ferror()=5 if currently open by another user.
  ON ERROR    && restore default error handler.
  IF _app.OpenError
    *-IF MyMsgBox('Could not open file '+cDbfPrefix+'.DBF exclusively. This file may be in use on other stations. Please ask other users to exit before you RETRY.', 'File open error',{'Retry','Cancel'}) = 2
      IF MyMsgBox('Could not open file '+cDbfPrefix+'.DBF exclusively. This file may be in use on other stations. Please ask other users to exit before you RETRY.', 'File open error',{'Retry','Cancel'},.f.) = 2
      if "RUNTIME" $ upper(version(0))
        quit
      else
        return
      endif
    ENDIF
    LOOP    && Retry
  ELSE
    FCLOSE(nHandle)
    EXIT    && no problem
  ENDIF
ENDDO

FixHeader( (cDbfPrefix) )            && fix record count in DBF header

MDXbyte(cDbfPrefix+".dbf","OFF")    && delete MDX reference in DBF Header
IF file(cDbfPrefix+".mdx")
  erase &cDbfPrefix..mdx        && erase existing MDX file before Indexing
ENDIF

USE &cDbfPrefix EXCLUSIVE

RETU
*****************************************************************
PROC FixHeader

* Based on a FoxPro program written by Thomas D. Bellmer, published in "Best of
* FoxTalk 1994-1995"
* Purpose: To write the correct RECCOUNT() value to the header of a DBF.
* This fixes the problem when FoxPro cannot open a table, and gives the
* message "Not a table/dbf". This problem can be caused by users turning
* off the computer while FoxPro tables are still open.

* Important: This routine will crash with an "Access denied" message
*            if the table is currently in use by anybody.


PARA lcfname     && we assume .DBF is extension is not supplied.
IF .NOT. '.'$lcfname
  lcfname = lcfname + '.DBF'
ENDIF

PRIV ALL LIKE l*

* Open the file with unbuffered read/write access:
lnhandle = FOPEN(lcfname,'RW')   && This gives "access denied" error if the table is currently open by any user.

* Test for possible file opening error:
IF lnhandle = -1
  tMsgBox( "Error: "+STR(FERROR(),2)+"  File "+lcfname )
  RETU
ENDIF

* Read first 12 chars of header:
lchdrinfo = FREAD(lnhandle,12)

* Calculate the header length:
lnhdrsize  = INT(ASC(SUBS(lchdrinfo,09,01)) ;
               + ASC(SUBS(lchdrinfo,10,01)) * 256)

* Calculate the record length:
lnreclngth = INT(ASC(SUBS(lchdrinfo,11,01)) ;
               + ASC(SUBS(lchdrinfo,12,01)) * 256)

** afsize[2] contains the file size in bytes
* DECLARE afsize[1]
* ADIR(afsize,lcfname)
local nSizeInBytes
nSizeInBytes = new File().size( lcfName )

* Calculate the # of records:
lnreccnt = INT( (nSizeInBytes - lnhdrsize) / lnreclngth )
lcfirst8 = LEFT(lchdrinfo,4) + SPACE(4)
FOR lni = 3 TO 0 STEP -1
  * Replace chars from left to right starting at 8 and going to 5:
  lcfirst8 = STUFF(lcfirst8, lni+5, 1, CHR(INT(lnreccnt/ 256^lni)))
  * Perform modulus division and use remainder as the remaining record count
  * in the next pass of the FOR loop:
  lnreccnt = MOD(lnreccnt,256^lni)
NEXT

* Go to top of file:
FSEEK(lnhandle,0,0)
* Write first 8 positions back to the header:
FWRITE(lnhandle, lcfirst8)
FCLOSE(lnhandle)
RETU
*******************************************************
FUNCTION MDXbyte
 *----------------------------------------
 *-- Programmer..: Bowen Moursund (CIS: 72662,436)
 *-- Date........: 05/21/1992
 *-- Notes.......: Sets the MDX byte in a DBF header ON or OFF.
 *--               The DBF must not be open when the function is called.
 *-- Written for.: dBASE IV v1.5
 *-- Rev. History: 05/21/1992 -- Original
 *-- Calls.......: dBASE low level file functions
 *-- Called by...: Any
 *-- Usage.......: MDXbyte(<cDBFpath>,<cOnOff>)
 *-- Example.....: lByteSet = MDXbyte("mydbf.dbf","OFF")
 *-- Returns.....: .T. if successful
 *-- Parameters..: cDBFpath = the <path>filename.ext of the DBF
 *--               cOnOff   = "ON" or "OFF"
 *----------------------------------------------------------------------

   parameters cDBFpath,cOnOff
   private all

   cOnOff = upper(m->cOnOff)
   * check the validity of the parameters
   lSuccess = ( pcount() = 2 .AND. m->cOnOff $ "ON|OFF" .AND. ;
                file(m->cDBFpath) )
   if lSuccess
      nHandle = fopen(m->cDBFpath,"RW")
      if m->nHandle > 0
         if fseek(m->nHandle, 28) = 28
            lSuccess = ( fwrite(m->nHandle, iif(m->cOnOff="OFF",;
                                                  chr(0),chr(1))) = 1 )
         else
            lSuccess = .F.
         endif
         lClosed = fclose(m->nHandle)
      else
         lSuccess = .F.
      endif
   endif

 RETURN m->lSuccess
***************************************************************
* Beginning of functions related to Time fields:              *
***************************************************************
Func DisplayTime(cHHMM)
/*
 cHHMM must be 4 or 5 characters, in military format. The ":" is optional.
 Returns a string in one of two formats, e.g. "11:59 p.m." or "23:59 p.m."
 The latter (military format) is used only when _app.TimeFormat is a string but
 not "12-hour".
 This function is ideal for displaying a time field in a grid.
 Example:
    function times1_onOpen
      local f
      f = new field()
      f.fieldname = 'Start time'
      f.beforeGetValue = {||DisplayTime(this.parent['StartTime'].value)}
      this.rowset.fields.add(f)
      return

 (For entryfields datalinked to time fields, use the TimeField class in Time.cc)

 Author: Peter Rorlick
*/

do case
case empty(CHHMM)
	return space(10)
case type('_app.TimeFormat')#'C' or Upper(_app.TimeFormat)=='12-HOUR'
  return padr(MilitaryTo12hour( left(cHHMM,2) + ':' + right(cHHMM,2) ), 10)
otherwise
  return padr(MilitaryDisplay( left(cHHMM,2) + ':' + right(cHHMM,2) ), 10)
endcase
/*if empty(cHHMM)
  return space(10)
elseif type('_app.TimeFormat')#'C' or Upper(_app.TimeFormat)=='12-HOUR'
  // 12-hour display:
  return padr(MilitaryTo12hour( left(cHHMM,2) + ':' + right(cHHMM,2) ), 10)
else
  // 24-hour display:
  return padr(MilitaryDisplay( left(cHHMM,2) + ':' + right(cHHMM,2) ), 10)
endif
*/

***************************************************************
FUNC MilitaryDisplay(cMilitaryTime)   && Returns time string in military format + am/pm.
/*
Assumptions:
	-cMilitaryTime MUST BE A VALID 24-hour HH:MM format
*/
LOCAL nHH, cFirstChar

IF EMPTY(cMilitaryTime)
  RETU ''
ENDIF

IF LEN(cMilitaryTime) <> 5 or AT(':', cMilitaryTime) <> 3
  RETU 'INVALID'
ENDIF

nHH = VAL(LEFT(cMilitaryTime,2))

IF LEFT(cMilitaryTime,1) = '0'
  cMilitaryTime = SUBS(cMilitaryTime,2)
ENDIF

RETU cMilitaryTime + IIF(nHH>11,'',' a.m.')
**************************************************************
FUNC MilitaryTo12hour(cMilitaryTime)   && Returns time string in 12-hour format
/*
Assumptions:
	-cMilitaryTime MUST BE either HH:MM or HHMM or empty
   -cMilitaryTime is in VALID 24-hour format
*/
IF EMPTY(cMilitaryTime)
   RETU ''
ENDIF
IF LEN(cMilitaryTime) <> 5 or AT(':', cMilitaryTime) <> 3
  RETU 'INVALID'
ENDIF

LOCAL nHH, cRetVal
nHH = VAL(LEFT(cMilitaryTime,2))

IF AT(':', cMilitaryTime) = 0
  cMilitaryTime = LEFT(cMilitaryTime,2) + ':' + RIGHT(cMilitaryTime,2)
ENDIF

IF nHH > 12 &&add pm indicator here rather than at end
  cRetVal = LTRIM(STR(nHH-12)) + RIGHT(cMilitaryTime,3) + ' p.m.'
ELSEIF nHH = 12 &&additional check for noon
  cRetVal = cMilitaryTime + ' p.m.'
ELSEIF nHH = 0 &&additional check for midnight
  cRetVal = '12' + RIGHT(cMilitaryTime,3) + ' a.m.'
ELSE
  cRetVal = LTRIM(STR(nHH)) + RIGHT(cMilitaryTime,3) + ' a.m.'
ENDIF
RETU cRetVal
*************************************************************
* From Gary White:
Func ValidTime(cTimeStr)
        && checks validity of user entered times
      local nHour,nMinute,lValid
      lValid=.t.                       && start out optimistic
      cTimeStr=atrim(cTimeStr)
      if len(cTimeStr)<>5        && valid time has to be five chars
         lValid=.f.
      endif
      nHour=val(left(cTimeStr,2))      && get the hours
      nMinute=val(right(cTimeStr,2))   && ... and the minutes
      if nHour<0 .or. nHour>23         && check the hours
         lValid=.f.
      endif
      if nMinute<0 .or. nMinute>59     && ... and the minutes
         lValid=.f.
      endif
      return lValid                    && tell 'em how they did

*************************************************************
Func NormalizeTimeString(cTimeString)
/*
Author: Serge Plourde
Created on 98.04.20

'Normalize' a given time string into a format such as 11:00 p.m. or 23:00.

cTimeString may look like:
 ?             Current time clock value
 23:59		   23:59
 2359			   23:59
 234			   2:34
 234p			   14:34
 11:30pm		   23:30
 11:30p		   23:30
 11:30		   11:30
 11pm			   23:00
 11			   11:00
 931p			   21:31
 8765xyz		   error: 8765 bad time value

*/

local cNormalizedTimeString
local lResumeNormalization
local nAt  //Position of ':' in time string
local cHours
local cMinutes
local cCharacter
local cAmOrPm
private cHoursAndMinutes

if empty(cTimeString)
   return ''
endif

lResumeNormalization = true

//First remove any blank from the time string:
cNormalizedTimeString = allTrim(cTimeString)
cNormalizedTimeString = StrTran(cNormalizedTimeString, ' ', '' )
cNormalizedTimeString = StrTran(cNormalizedTimeString, '.', '' )
if cNormalizedTimeString = '?' or upper(left(cNormalizedTimeString,3)) == 'NOW'
   //Replace '?' with current system time clock value:
   if type('_app.TimeFormat')#'C' or Upper(_app.TimeFormat)=='12-HOUR'
	   return militaryTo12Hour(left(time(),5))
   else
      return militaryDisplay(left(time(),5))
   endif
else
   if not (left(cNormalizedTimeString,1)$'1234567890')
      return 'INVALID'  //invalid: it must start with a digit.
   endif
endif

//Parse the time string for the hours and minutes
nAt = At(':', cNormalizedTimeString)
cHours = ''
if nAt <> 0
   //Parse hours and minutes from both sides of ':'
   if nAt = 1
      cHours = '00'
   else
   	cHours = left(cNormalizedTimeString, nAt - 1)
      if len(cHours) = 1
         cHours = '0' + cHours
      endif
   endif

   //Parse the minutes
   if nAt < len(cNormalizedTimeString)
    	cNormalizedTimeString = subs(cNormalizedTimeString, nAt + 1)
   else
    	cNormalizedTimeString = ''
   endif

	cMinutes = ''
   do while true and not empty(cNormalizedTimeString)
      //will exit as soon as no minutes value from string or empty string
      cCharacter = left(cNormalizedTimeString,1)
      if cCharacter $ '1234567890'
         cMinutes = cMinutes + cCharacter
         if len(cNormalizedTimeString) > 1
            cNormalizedTimeString = subs(cNormalizedTimeString,2)
         else
            cNormalizedTimeString = ''
            exit
         endif
      else
         exit
      endif
   enddo

   if empty(cMinutes)
      cMinutes = '00'
   else
      if len(cMinutes) = 1
         cMinutes = '0' + cMinutes
      endif
   endif

endif //':' in time string: parsed hours and minutes

if empty(cHours)
   //means that ':' was not part of the string
   cHoursAndMinutes = ""
   do while len(cHoursAndMinutes) < 5 and not empty(cNormalizedTimeString)
      //will exit as soon as no minutes value from string
      cCharacter = left(cNormalizedTimeString,1)
      if cCharacter $ '1234567890'
         cHoursAndMinutes = cHoursAndMinutes + cCharacter
         if len(cNormalizedTimeString) > 1
            cNormalizedTimeString = subs(cNormalizedTimeString,2)
         else
            cNormalizedTimeString = ''
            exit
         endif
      else
         exit
      endif
   enddo


   do case
   case len(cHoursAndMinutes) = 1
      cHours = '0' + cHoursAndMinutes
      cMinutes = '00'

   case len(cHoursAndMinutes) = 2
      cHours = cHoursAndMinutes
      cMinutes = '00'
   case len(cHoursAndMinutes) = 3
      cHours = '0' + left(cHoursAndMinutes, 1)
      cMinutes = right(cHoursAndMinutes, 2)
   case len(cHoursAndMinutes) = 4
      cHours = left(cHoursAndMinutes, 2)
      cMinutes = right(cHoursAndMinutes, 2)
   otherwise
      return 'INVALID'
   endcase
endif

/*
At this point we have cHours and cMinutes in HHMM with leading zero.
cNormalizedTimeString contains the remaining of the given string after the hours
and minutes have been parsed.
*/
cAmOrPm = upper(left(ltrim(cNormalizedTimeString),1))
if cHours = '12' and cAmOrPm <> 'A'
   cAmOrPm = 'P'
endif
if empty(cAmOrPm)
   cAmOrPm = 'A'
endif

if cAmOrPm = 'P'   //time labeled as pm
   if val(cHours) < 12
	   cHours = str(val(cHours) + 12, 2)
   endif
*->Test source code:
else
	*-AM
	if cHours = '12'
   	cHours = '00'
	endif
*<-End of test source code
endif

if Val(cHours) > 24 or val(cMinutes) > 60 or;
   (cHours + cMinutes >= '2400')
   return 'INVALID'  //invalid time
endif

if type('_app.TimeFormat')#'C' or Upper(_app.TimeFormat)=='12-HOUR'
 	//return 12-hour format
   cNormalizedTimeString = militaryTo12Hour( cHours + ':' + cMinutes)
else
 	//24-hour format
   cNormalizedTimeString = militaryDisplay(cHours + ':' + cMinutes)
endif


return cNormalizedTimeString
***************************************************************
Func NormalTimeStrToMilitary(cTimeString)
/*
Author: Serge Plourde
Created on 98.04.22

Take a 'Normalize(d)TimeString() and returns a military time.
*/
local cHours, cMinutes

if empty(cTimeString)
   return spac(4)
endif

if type('_app.TimeFormat')#'C' or Upper(_app.TimeFormat)=='12-HOUR'
   /*
   cTimeString is in 12-hour mode: 12:00 a.m. @ 12:59 a.m. are
   actually 00:00 @ 00:59
   */
   do case
   case '12:' $ cTimeString and 'a.m.' $ cTimeString
   	cHours = '00'
   case 'p.m.' $ cTimeString and not '12:' $ cTimeString
	   cHours = left(cTimeString,at(':', cTimeString) - 1)
      cHours = str(val(cHours) + 12, 2)
   case 'p.m.' $ cTimeString and left(cTimeString,3) == '12:'
      cHours = '12'
   otherwise
	   cHours = left(cTimeString,at(':', cTimeString) - 1)
   endcase
else
   *-cTimeString is already in 24-hour mode
	cHours = left(cTimeString,at(':', cTimeString) - 1)
endif
cMinutes = subs(cTimeString, at(':', cTimeString) + 1, 2)

if len(cHours) = 1
   cHours = '0' + cHours
endif
return cHours + cMinutes
***************************************************************
* End of functions related to Time fields:                    *
***************************************************************

***************************************************************
* Read all data from table & write it in other table...You    *
* need just pass table name of table which you want to repair *
* Example : 'pa.dbf' -> 'paRepaired.dbf'                      *
***************************************************************
function RepairTable(TableName)
  priv c,n,tName

  tName=RemoveExtensionFromTheFileName(TableName)
  sele sele()
  c="use '"+tName+"'"
  &c
  c="copy to '"+tName+"Structure' stru exte"
  &c
  c="create '"+tName+"Repaired' from '"+tName+"Structure'"
  c="use '"+tName+"'"
  &c
  c="copy to '"+tName+"Repaired'"
  &c
  c="delete table "+tName+"Structure"
  &c
  use
return
***************************************************************
* Removes extention from the filename,                        *
* Example : 'mytable.dbf' -> 'mytable'                        *
***************************************************************
function RemoveExtensionFromTheFileName(FileName)
  priv f,n

  f=FileName
  n=At('.',f)

  If n <> 0
    f = STUFF(f,n,Len(f)-n+1,'')
  EndIf

return f
***************************************************************
function cShortMonth(MonthNumber)
  if TYPE('_app.language') <> 'C'
    _app.language = 'English'
  EndIf

  if (UPPER(_app.language) == 'FRENCH')
   DO CASE
     CASE MonthNumber=0
        return "jan"
     CASE MonthNumber=1
        return "fv"
     CASE MonthNumber=2
        return "mars"
     CASE MonthNumber=3
        return "avr"
     CASE MonthNumber=4
        return "mai"
     CASE MonthNumber=5
        return "juin"
     CASE MonthNumber=6
        return "juil"
     CASE MonthNumber=7
        return "ao"
     CASE MonthNumber=8
        return "sep"
     CASE MonthNumber=9
        return "oct"
     CASE MonthNumber=10
        return "nov"
     CASE MonthNumber=11
        return "dc"
     otherwise
        return "???"
   EndCase
  Else
   DO CASE
     CASE MonthNumber=0
        return "Jan"
     CASE MonthNumber=1
        return "Feb"
     CASE MonthNumber=2
        return "Mar"
     CASE MonthNumber=3
        return "Apr"
     CASE MonthNumber=4
        return "May"
     CASE MonthNumber=5
        return "June"
     CASE MonthNumber=6
        return "July"
     CASE MonthNumber=7
        return "Aug"
     CASE MonthNumber=8
        return "Sep"
     CASE MonthNumber=9
        return "Oct"
     CASE MonthNumber=10
        return "Nov"
     CASE MonthNumber=11
        return "Dec"
     otherwise
        return "???"
   EndCase
  EndIf
 return
***************************************************************
function cShortDayOfWeek(DayOfTheWeek)
  if TYPE('_app.language') <> 'C'
    _app.language = 'English'
  EndIf

  if (UPPER(_app.language) == 'FRENCH')
   DO CASE

     CASE DayOfTheWeek=0
        return "dim"
     CASE DayOfTheWeek=1
        return "lun"
     CASE DayOfTheWeek=2
        return "mar"
     CASE DayOfTheWeek=3
        return "mer"
     CASE DayOfTheWeek=4
        return "jeu"
     CASE DayOfTheWeek=5
        return "ven"
     CASE DayOfTheWeek=6
        return "sam"
   EndCase
  Else
   DO CASE

     CASE DayOfTheWeek=0
        return "Sun"
     CASE DayOfTheWeek=1
        return "Mon"
     CASE DayOfTheWeek=2
        return "Tue"
     CASE DayOfTheWeek=3
        return "Wed"
     CASE DayOfTheWeek=4
        return "Thu"
     CASE DayOfTheWeek=5
        return "Fri"
     CASE DayOfTheWeek=6
        return "Sat"
   EndCase
  EndIf
 return
***************************************************************
function DateToStringEx(dDate)

*   Examples of how to display a date:
*     Wed 12 Mar 1998
*     Fri 1 Jan 2000
*     Sat 29 July 1990
*     29 July 1990      (if _app.HideDOW exists and is TRUE)
   if type('_app.HideDOW')='U'
     _app.HideDOW=.f.
   endif

   Priv ddDate
   ddDate=dDate
   if (TYPE('ddDate')<>'D') AND (TYPE('ddDate')<>'O')
     return ''
   EndIf

   s=new String()

   If(_app.HideDOW)
     s=""
   Else
     s=cShortDayOfWeek(dDate.getDay()) + " "
   EndIf

   s = s + LTRIM(TRIM(str(dDate.getDate()))) + " " + cShortMonth(dDate.getMonth(),2,0) + ;
     " " + LTRIM(TRIM(str(YEAR(dDate),4,0)))

//   tMsgBox(s)

 return s
***************************************************************
Func NormalizeDateString(ccDateString)
// returns the date object..
*     Fri 1 May 1998
*     1 May 1998
*     today
*     tomorrow
*     yesterday
*     tod       (today)
*     tom       (tomorrow)
*     y         (yesterday)
*     12Mar     (Currnent year is assumed if year is omitted)
*     12Mar98
*     12Mar1998
*     1Jul99
*     010799       (see note below, order of DMY or MDY or YMD is configurable)
*     01071999
*     01/07/99
*     01/07/1999   (allow separator to be / - . or space)
*     a number (range -999 to 999)  (date()+num)
*     a 4 digit number       (intrepet as MMDD or DDMM, according to _app.dateFormat)
priv cDateString
//TRY

cDateString=ccDateString

If TYPE('_app.dateFormat')<>'C'
  _app.dateFormat = 'DMY'
EndIf

if TYPE('cDateString')<>'C'
//  tMsgBox('Somehow cDateString is not a string')
  return date()
endif

If LEN(ALLTRIM(cDateString))<1
  return NULL
EndIf

 dDate = New Date() // date to return..

 aDate = DivideString(cDateString)

/* aview(aDate)
 crashme()*/

  if (aDate[1]=='+')
    if Type('aDate[2]')='N'
      return (dDate+aDate[2])
    else // It's character +Monday
      nd=DayOfTheWeekToNumber(aDate[2])

      if (nd<>-1)
        if (dDate.GetDay()<nd)
          return (dDate+nd-dDate.GetDay())
        else
          return (dDate+7+nd-dDate.GetDay())
        EndIf
      Else
        ShowError() // Will throw the exception
      EndIf
    endIf
  EndIf

  if (aDate[1]=='-')
    if Type('aDate[2]')='N'
      return (dDate-aDate[2])
    else // It's character -Tuesday
      nd=DayOfTheWeekToNumber(aDate[2])

      if (nd<>-1)
        if (dDate.GetDay()>nd)
          return (dDate+nd-dDate.GetDay())
        else
          return (dDate-7+nd-dDate.GetDay())
        EndIf
      Else
        ShowError() // Will throw the exception
      EndIf
    endIf
  EndIf

  If (aLen(aDate)==6) and (TYPE('aDate[3]') == 'N')
//    ?arraytostring(aDate)
    return (RestoreDateFromTheString({aDate[3],aDate[4],aDate[6]}))
//    return (new Date(cDateString))
  EndIf

  If (aLen(aDate)==4) and (TYPE('aDate[1]') == 'N')
    ?arraytostring(aDate)

    if (_app.dateFormat <> 'YMD')
    return (RestoreDateFromTheString({aDate[1],aDate[2],aDate[4]}))
    Else
      cString = aDate[4] + ' ' + aDate[2] + ' ' + aDate[1]
      return (RestoreDateFromTheString({aDate[4],aDate[2],aDate[1]}))
    EndIf
  EndIf

  if (aLen(aDate)==1)

    If (TYPE('aDate[1]')=='C')

     cTry1 = UPPER(ALLTRIM(cDateString))

     DO CASE

      CASE (AT(cTry1,"AUJOURD'HUI")==1)   // Try for aujourd'hui
        return (dDate)

      CASE (LEN(cTRY1)>=3) And (AT(cTry1,"TODAY")==1)   // Try for TODAY
        return (dDate)

      CASE (LEN(cTRY1)>=3) And (AT(cTry1,"TOMORROW")==1)   // Try for Tomorrow
        return (dDate+1)

      CASE (AT(cTry1,"DEMAIN")==1)   // Try for demain
        return (dDate+1)

      CASE (AT(cTry1,"HIER")==1)   // Try for hier
        return (dDate-1)

      CASE (AT(cTry1,"YESTERDAY")==1)  // Try for Yesterday
        return (dDate-1)

      CASE (AT(cTry1,"BLANK")==1)  // Try for blank
        return NULL

      CASE (AT(cTry1,"ESPACE")==1)  // Try for espace
        return NULL

      CASE (AT(cTry1,"AIDE")==1) OR (AT(cTry1,"HELP")==1)  // Try for Help
        return -3

      CASE (AT(cTry1,"FORMAT")==1)   // Try for Format
        return -2

      CASE (AT(cTry1,"CALENDAR")==1)  // Try for Calendar
        return -1

      CASE (AT(cTry1,"CALENDRER")==1)  // Try for Calendrer
        return -1

      OTHERWISE
        ShowError() // Will throw the exception

     EndCase

    Else // It's something like 010797

      nDay=   .f.
      nMonth= .f.
      nYear=  .f.

      If _app.dateFormat == 'DMY'
        c=SUBSTR(cDateString,1,2)
        nDay=&c

        if (LEN(cDateString)>2)
          c=SUBSTR(cDateString,3,2)
          nMonth=&c
        EndIf
        if (LEN(cDateString)>4)
          c=SUBSTR(cDateString,5)
          nYear=&c
        EndIf
      ElseIf _app.dateFormat = 'MDY'
        c=SUBSTR(cDateString,1,2)
        nMonth=&c
        c=SUBSTR(cDateString,3,2)
        nDay=&c
        If (LEN(cDateString)>4)
          c=SUBSTR(cDateString,5)
          nYear=&c
        EndIf
      Else
        nn=IIF (LEN(cDateString)==6,2,4)
        c=SUBSTR(cDateString,1,nn)
        nYear=&c
        c=SUBSTR(cDateString,nn+1,2)
        nMonth=&c
        c=SUBSTR(cDateString,nn+3,2)
        nDay=&c
      EndIf

      If TYPE('nDay') == 'N'
          If (nDay<=31) and (nDay>0)
            dDate.SetDate(nDay)
          Else
            ShowError()
          EndIf
      EndIf

      If TYPE('nYear') == 'N'
          if (nYear>=0) and (nYear<=79)
            nYear=2000 + nYear
          EndIf

        dDate.SetYear(nYear)
      EndIf

      If TYPE('nMonth') == 'N'
          If (nMonth<=12) and (nMonth>0)
            dDate.SetMonth(nMonth-1)
          Else
            ShowError()
          EndIf
      EndIf

    EndIf
  Else // It is something like 12Mar97 or 01/01/07 or error

    If (TYPE('aDate[1]')=='C')
        if (TYPE('aDate[3]')=='N')  // Mar 12 97
          dDate = RestoreDateFromTheString({aDate[1],aDate[3],aDate[5]})
        Elseif (type('aDate[4]')=='N')
          dDate = RestoreDateFromTheString({aDate[2],aDate[3],aDate[4]})
        Else
          dDate = RestoreDateFromTheString({aDate[2],aDate[3]})
        EndIf
    Else
      // aDate[2] must be character, even there is no need to check
      If ((aDate[2]=='/') OR (aDate[2]=='-') OR (aDate[2]=='.') OR (aDate[2]==' '))
        If TYPE('_app.dateFormat')<>'C'
          _app.dateFormat = 'DMY'
        EndIf

        nDay=   .f.
        nMonth= .f.
        nYear=  .f.

        If _app.dateFormat == 'DMY'
          nDay=aDate[1]
          nMonth= IIF ((TYPE('aDate[3]')=='N'),aDate[3],.f.)
          nYear= IIF ((TYPE('aDate[5]')=='N'),aDate[5],.f.)
          If (TYPE('nMonth')=='C') // If month is character..
            d = RestoreDateFromTheString(nDay,nMonth,nYear)
            nMonth = d.Month
          EndIf

        ElseIf _app.dateFormat = 'MDY'
          nMonth=aDate[1]
          nDay= aDate[3]
          nYear= IIF ((TYPE('aDate[5]')=='N'),aDate[5],.f.)

          If (TYPE('nMonth')=='C') // If month is character..
            d = RestoreDateFromTheString(nDay,nMonth,nYear)
            nMonth = d.Month
          EndIf
        Else
          nYear=aDate[1]
          nMonth= aDate[3]
          nDay= aDate[5]

          If (TYPE('nMonth')=='C') // If month is character..
            d = RestoreDateFromTheString(nDay,nMonth,nYear)
            nMonth = d.Month
          EndIf
        EndIf

        If TYPE('nDay') == 'N'
          If (nDay<=31) and (nDay>0)
            dDate.SetDate(nDay)
          Else
            ShowError()
          EndIf
        EndIf

        If TYPE('nYear') == 'N'
          dDate.SetYear(nYear)
        EndIf

        If TYPE('nMonth') == 'N'
          If (nMonth<=12) and (nMonth>0)
            dDate.SetMonth(nMonth-1)
          Else
            ShowError()
          EndIf
        EndIf

      Else
        dDate = RestoreDateFromTheString(aDate)
      EndIf
    EndIf
  EndIf
/*catch ( Exception e )
  Beep()
  tMsgBox('There is an error in your expression','Error',16)
  return -4
endtry*/

  return dDate
***************************************************************
Func RestoreDateFromTheString(aDDate) // Works only with '01 Mar 1997' format
local dDate
priv aDate

        aDate = aDDate

        If (TYPE('aDate[1]')=='C') AND (TYPE('aDate[2]')=='N')
          nTmp = aDate[2]
          aDate[2] = aDate[1]
          aDate[1] = nTmp
        EndIf

        dDate = new Date()
        dDate.SetDate(aDate[1])
        c1=Character(LTRIM(aDate[2]),0)
        c2=Character(LTRIM(aDate[2]),1)
        c3=Character(LTRIM(aDate[2]),2)
        c4=Character(LTRIM(aDate[2]),3)
//        ?c1+c2+c3+c4

        DO CASE

          CASE (c1 == 'A') // Apr, Aug, Avr, Auot
            If (c2 == 'U')
              // August
              dDate.SetMonth(7)
            ElseIf c2=='O'
              // Aout
              dDate.SetMonth(7)
            Else
              // April, Avril
              dDate.SetMonth(3)
            EndIf

          CASE (c1 == 'D')  // Dec
            // December
              dDate.SetMonth(11)

          CASE (c1 == 'F')  // Feb
            // Feb
              dDate.SetMonth(1)

          CASE (c1 == 'J')  // Jan, June, July, Juin, Juillet
            if (c2 == 'A')
              // January
              dDate.SetMonth(0)
            ElseIf (c2 == 'U')
              If (c3 == 'I') // Juin, Juillet
                If (c4 == 'N')
                  dDate.SetMonth(5) // Juin
                Else
                  dDate.SetMonth(6) // Juillet
                EndIf
              ElseIf (c3 == 'L')
                // July
                dDate.SetMonth(6)
              Else
              // June
                dDate.SetMonth(5)
              EndIf
            Else
              ShowError()  // Will throw the exception
            EndIf

          CASE (c1=='M')
            if (c3=='R')
              // March, Mars
              dDate.SetMonth(2)
            Else
              // May, Mai
              dDate.SetMonth(4)
            EndIf

          CASE (c1 == 'N')
            // November, Novembre
              dDate.SetMonth(10)

          CASE (c1 == 'O')
            // October, Octobre
              dDate.SetMonth(9)

          CASE (c1 == 'S')
            // September, Septembre
              dDate.SetMonth(8)
        ENDCASE

        If (TYPE('aDate[3]') == 'N')
          if (aDate[3]>=0) and (aDate[3]<=79)
            aDate[3]=2000 + aDate[3]
          EndIf

          dDate.SetYear(aDate[3])
        EndIf

   return dDate
***************************************************************
Func Character(cString,nNumber)
return UPPER(cString.CharAT(nNumber))
***************************************************************
Func DivideString(cSString)
// Returns array of substrings of this string..
// Example : '12Mar97' -> {12,'Mar',97}
//				 '11/23/98' -> {12,'/',23,'/',98}
//           '+Monday'  -> {'+','Monday'}
  priv cString
  cString = ALLTRIM(cSString)
  aContents=new Array()
  c = Character(cString,0)
  If (ASC(c)>=48) and (ASC(c)<=57)
    flNumber=.t.
  Else
    flNumber=.f.
  EndIf
  nN=0
  cS=''
  for i=0 to Len(cString)-1
    c = Character(cString,i)
    If (ASC(c)>=48) and (ASC(c)<=57)
      If (flNumber)
        nN = nN*10 + &c
      Else
        flNumber = .t.
        If (LEN(cs)>0)
          aContents.Add(cS)
          cS = ''
        EndIf
        nN = &c
      EndIf
    Else
      If (flNumber)
        aContents.Add(nN)
        nN = 0
        flNumber = .f.
        cS = c
      Else
        If (c == '+') or (c == '-')  or (c == '/') or (c == ' ') or (c == '.') or (c == '-')
          If (LEN(cs)>0)
            aContents.Add(cS)
          EndIf
          cS=c
          aContents.Add(cS)
          cS=''
         Else
          cS = cS + c
        EndIf
      EndIf
    EndIf
  Next

  If (flNumber)
    aContents.Add(nN)
  Else
    aContents.Add(cS)
  EndIf

 return aContents
***************************************************************
Func DayOfTheWeekToNumber(cString)
 cDay = SubStr(UPPER(ALLTRIM(cString)),1,3)
 DO CASE
   CASE (cDay=='SUN') or (cDay=='DIM')
    return 0
   CASE (cDay=='MON') or (cDay=='LUN')
    return 1
   CASE (cDay=='TUE') or (cDay=='MAR')
    return 2
   CASE (cDay=='WED') or (cDay=='MER')
    return 3
   CASE (cDay=='THU') or (cDay=='JEU')
    return 4
   CASE (cDay=='FRI') or (cDay=='VEN')
    return 5
   CASE (cDay=='SAT') or (cDay=='SAM')
    return 6
   OTHERWISE
    return -1
 ENDCASE
return // Useless but it's easier to understand that function has been ended here
***************************************************************
FUNCTION Age( dBirth )
   *-------------------------------------------------------------------
   *-- Programmer..: Martin Leon
   *-- Date........: 10/23/1991
   *-- Notes.......: Returns age of thing as of date() given its
   *--               birthdate.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 10/23/1991 -- Original
   *--               08/10/1993 -- Ken Mayer, "stole" algorithm from
   *--                the routine AGE2. It's more efficient.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.Age(<dBirth>)
   *-- Example.....: ? "Joe is "+ltrim(str(_app.Date.Age(dBirth)))+;
   *--                 " today ..."
   *-- Returns.....: Numeric value in years
   *-- Parameters..: dBirth = birthdate of thing to return age of.
   *-------------------------------------------------------------------

RETURN ( floor( ( val( dtos( date() ) ) ;
                   - val( dtos( dBirth ) ) ) / 10000 ) )
***************************************************************
func MacroExpand   // For use in code blocks - equivalent of &c
param c
return &c
********************************************************************
Proc UpdateModStamp   // For VdB7.01  (For data synchronization modules)
param oRowset, lDontSave   // param is optional, but Must be passed if the record was changed by OODML methods.
if type('oRowset')=='U'
  * XBASE DML
  IF TYPE('ModStamp')='C'
    REPL ModStamp WITH DTOS(DATE()) + ' ' + TIME() + ' ' + ;
         IIF( TYPE('_App.CurrentUserCode')='C', _App.CurrentUserCode, ' ' )
    IF EMPTY(AddStamp)
      REPL AddStamp WITH ModStamp
    ENDIF
  ENDIF
else
  * OODML
  if type("oRowset.fields['ModStamp']") == 'O'
    * This table has a ModStamp field.
    oRowset.fields['ModStamp'].Value = DTOS(DATE()) + ' ' + TIME() + ' ' + ;
         IIF( TYPE('_App.CurrentUserCode')='C', _App.CurrentUserCode, ' ' )
    if empty(oRowset.fields['AddStamp'].Value)
      oRowset.fields['AddStamp'].Value = oRowset.fields['ModStamp'].Value
    endif
    if lDontSave
      * We will NOT save() here if the 2nd param was TRUE.
    else
      oRowset.save()
    endif
  endif
endif
return
************************************************************************
Func MemoRead(cFilename)   // For clipper compatibility
return FileContents(cFileName)
************************************************************************
Func FileContents(cFileName)
local f, cRetVal
f = new file()
f.open(cFileName,'R')
cRetVal = f.read(f.size(cFileName))
f.close()
return cRetVal
******************************************************************************
PROCEDURE WriteArrayToFile
  PARAMETER oArray, cFileName
  * Write the content of a text array to a file.
  LOCAL nFileHandle
  LOCAL nLine, nLines
  LOCAL cCRLF
  cCRLF = CHR(13)+CHR(10)
  nFileHandle = FCREATE(cFileName)
  nLines = ALen(oArray)
  FOR nLine = 1 TO nLines
     FWRITE(nFileHandle, oArray[nLine] + cCRLF)
  NEXT
  FClose(nFileHandle)
return
******************************************************************************
Proc SwitchLanguage
  if _app.Language == 'English'
    _app.Language = 'French'
  else
    _app.Language = 'English'
  endif
  *
  if type('_app.framewin.app.rootmenu.TranslateTexts') $ 'CB/FP'
    _app.framewin.app.rootmenu.TranslateTexts()
  endif
  *
  // If there is a toolbar on the frameWin, reset it to ensure that it's 
  // appropriate to the new language:
  _app.framewin.app.ResetToolBar()
  *
  TMsgBox('Language has been switched. Subsequent forms will open in the new language.')
return
******************************************************************************
Proc CloseAllQueries
if type('_app.Queries') == 'A'
  for i = 1 to _app.Queries.size
    if type('_app.Queries[i].sql') == 'C'  // if this array element is still a Query object...
      * Kill it:
      _app.Queries[i].active = .f.
      _app.Queries[i] = null
    endif
  next
endif
return
******************************************************************************
function ArrayScan2d( aSearch, aTarget )  // Returns numeric element #, or zero if not found.
  * aSearch is 1-d array to search for.
  * aTarget is a 2-dimensional array, in which we will search for aSearch.
  local i
  for i = 1 to aTarget.size
    if ArrayCompare( aSearch, aTarget[i] )
      return i
    endif
  next
return 0  // not found
******************************************************************************
function ArrayCompare( a1, a2 )  // Returns TRUE iff their contents are equal.
  local j
  if a1.size # a2.Size
    return false  // Sizes of the arrays are different.
  else
    for j = 1 to a1.Size
      if not a1[j] == a2[j]
        return false   // not a match.
      endif
    next
  endif
return true
******************************************************************************
function AScan2d( gSearch, aTarget, nSubElementPosition, lMatchCase )
  // Performs an ASCAN, comparing the gSearch value against the nth element of each subarray.
  // Returns numeric element #, or zero if not found.
  * gSearch is non-array (e.g., string or numeric or date) value to search for.
  * aTarget is a 2-dimensional array, in which we will search for gSearch.
  * nSubElementPosition is the index into the subarrays where we will seek.
  * lMatchCase (this param is optional, assumed TRUE if omitted) determines case-sensitivity of the comparisons.
  * Example:
  *  ? AScan2d( 'hello', {{1,2},{'aa',{99.12.31}},{TRUE,'hello'}}, 2 )  // returns 3.
  if argCount() < 4
    lMatchCase = true   // Case sensitive, if this param was omitted.
  elseif not lMatchCase
    priv c
    c = gSearch
    if type('c') == 'C'
      gSearch = upper(gSearch)  // Comparisons will ignore case.
    endif
  endif
  local i
  private a, gValue
  for i = 1 to aTarget.size
    a = aTarget[i]
    if type('a')=='A' and a.Size>=nSubElementPosition
      if lMatchCase
        if a[nSubElementPosition] == gSearch
          return i
        endif
      else
        * Ignore case:
        gValue = a[nSubElementPosition]
        if type('gValue') == 'C'
          if upper(gValue) == gSearch
            return i
          endif
        else
          if gValue == gSearch
            return i
          endif
        endif
      endif
    endif
  next
return 0  // not found
***************************************
function UseExclusive( cTableName, lInNewArea )  // Works for VdB 7.01
  * Returns TRUE if the table was opened exclusively.
  * Returns FALSE if the table was opened in SHARED mode.
  local lSuccess
  if lInNewArea
    select select()
  endif
  use (cTableName) exclusive
  try
    delete tag DummyXYZ   // Nonexistent tag name
    * The above line will throw one of 3 errors:
    *  Error 110 "Operation requires exclusive use of table"   OR:
    *  Error 53  "Tag not found: DUMMYXYZ"       OR: 
    *  Error ??  "Table is not indexed"
  catch( exception e )
    if e.code = 53  // message = 'Error:  Tag not found'
      * We have succeeded in opening the table EXCLUSIVEly.
      lSuccess = true
    else
      lSuccess = false   // Error was probably "Operation requires exclusive..."
    endif
  endtry
return lSuccess
********************************************************
Function nToDow(nDow)
local aDay

   aDay = {'Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'}
   if nDow <= aDay.Size
      return aDay[nDow]
   endif
return ""         
******************************************************
Function QQuery( cTableName, cIndexName, lViewRowset )  // Returns a query object
local q
q = new ZQuery()
q.sql = 'select * from ' + cTableName
q.active = true
if argCount() = 2    // 2nd param is optional.
  private c
  c = cIndexName
  if type('c') == 'C'
    q.rowset.IndexName = cIndexName
  endif
endif
if lViewRowset      // 3rd param is optional.
  RView(q.rowset)
endif
return q
******************************************************
Proc SetProcTo( cFileName )  // SETs PROC TO (cFileName) ADDI  -  but only if the proc file is not already open.
local cFile  // Don't want to make changes to the param itself.
cFile = upper(cFileName)
if not '.'$cFileName
  cFile += '.PRG'
endif
local i, ciProcFile
for i = 1 to 9999
  ciProcFile =  set('PROC',i)
  if empty( ciProcFile )
    * We've hit the end of the procedure list.
    set proc to (cFile) addi
    exit
  else
    ciProcFile = subs( ciProcFile, RAT('\',ciProcFile)+1 )  // Strip out the path so that we're left with just the filename.
    if upper(ciProcFile) == cFile
      * The desired procedure file is already open.
      exit
    endif
  endif
next
return
***************************************************
Proc ListOpenProcedureFiles
local i, aProcs
aProcs = new array()
for i = 1 to 9999
  ciProcFile =  set('PROC',i)
  if empty( ciProcFile )
    * We've hit the end of the procedure list.
    exit
  endif
  ? ciProcFile
  aProcs.Add(ciProcFile)
next
exmsgbox(aProcs.size,'Number of open procedures')
aView(aProcs)
return
****************************************************
Proc Freeze( oForm )
* Freezes the form, or the entire desktop if no parameter is passed.
if type("LockWindowUpdate") # "FP"
  extern CLOGICAL LockWindowUpdate( CHANDLE ) user32
endif
if argCount() > 0
  LockWindowUpdate( oForm.hWnd )            // Freeze the form.
  if type( 'oForm.freeze' ) # 'N'
    oForm.freeze = 0  // Create custom property, to manage "nested freezes".
  endif
  oForm.freeze++
else
  if type("GetDesktopWindow") # "FP"
    extern CHANDLE GetDesktopWindow( CVOID ) user32
  endif
  LockWindowUpdate( GetDesktopWindow () )   // Freze the entire desktop.
endif
return
****************************************************
Proc UnFreeze       // Unlocks a frozen desktop or form.
param oForm  // optional, but highly recommended when unfreezing a form.
if argCount() > 0
  if type( 'oForm.freeze' ) == 'N'
    oForm.freeze--
    if oForm.freeze <= 0
      oForm.freeze := 0
      LockWindowUpdate(0)
    else
      * We're in a "nested freeze". So do NOT unfreeze the form.
    endif
  else
    LockWindowUpdate(0)
  endif
else   // no param was passed.
  LockWindowUpdate(0)
endif
return
* We will assume that for every Freeze(form), there will be a
* corresponing UnFreeze(form).
*************************************************************

// Time should be military format
// 23:00 or 2300
// ?roundtime("2303",15) --> 2315
// ?roundtime("2308",7)  --> 2314
// ?roundtime("2308",60) --> 2400
// Invalid times --> ""
// if a 3rd param is passed then we round down
function RoundTime(c,n,xDown)
local cTime,nHours,nMinutes

   cTime = alltrim(c)
  
   nHours   = val(left(cTime,2))
   nMinutes = val(right(cTime,2))

   if nHours > 24 or nMinutes > 60
      return ""
   endif

   do while mod(nMinutes,n) > 0
      if pcount() = 3
         nMinutes--
      else
         nMinutes++
      endif
   enddo      
   if nMinutes = 60
      nMinutes = 0
      nHours++
   endif
   if len(cTime) = 4
      cTime = str(nHours,2,0,"0")+str(nMinutes,2,0,"0")
   else
      cTime = str(nHours,2,0,"0")+":"+str(nMinutes,2,0,"0")
   endif

return cTime

****************************************************************
// Increments time
// cTime has to be "10:00" or "1000"  Military
// nIncrement is in minutes
// this can return > 2400
function IncTime(cTime,nIncrement)
return AdjustTime(cTime,nIncrement)
/*
local nHours,nMinutes
local cRet

   cRet = alltrim(cTime)            
   nHours = val(left(cTime,2))
   nMinutes = val(right(cTime,2))

   nMinutes += nIncrement
   if nMinutes >= 60
      nMinutes = nMinutes - 60
      nHours++
   endif

   if len(cRet) > 4
      cRet = str(nHours,2,0,"0")+":"+str(nMinutes,2,0,"0")
   else
      cRet = str(nHours,2,0,"0")+str(nMinutes,2,0,"0")
   endif
return (cRet)               
*/
****************************************************************
// Decrements time
// cTime has to be "10:00" or "1000"  Military
// nIncrement is in minutes
function DecTime(cTime,nIncrement)
return AdjustTime(cTime,-nIncrement)

local nHours,nMinutes
local cRet

   cRet = alltrim(cTime)            
   nHours = val(left(cTime,2))
   nMinutes = val(right(cTime,2))

   nMinutes -= nIncrement
   if nMinutes < 0
      nMinutes = nMinutes + 60
      nHours--
   endif

   if len(cRet) > 4
      cRet = str(nHours,2,0,"0")+":"+str(nMinutes,2,0,"0")
   else
      cRet = str(nHours,2,0,"0")+str(nMinutes,2,0,"0")
   endif
return (cRet)               


*******************************************************************
FUNC ElapsedMinutes(cStartTime,cEndTime,lAlertOnError)
* Returns numeric elapsed minutes
* Parameters are expected in the form "HH:MM" or "HHMM"
* Range is 00:00 to 23:59
*
IF argCount() < 3
  lAlertOnError := .T.    // Set to TRUE if parameter was omitted.
ENDIF
IF ' ' $ cStartTime+cEndTime .OR. LEN(cStartTime+cEndTime)<8 .OR. LEN(cStartTime+cEndTime)>10
  IF lAlertOnError
    MsgBox('Unable to calculate elapsed time. Returning ZERO minutes.','Error',48)
  ENDIF
  RETU 0   // Blank (or bad) params were received.
ENDIF
IF cEndTime < cStartTime
  * We assume that cEndTime is on the following day after cStartTime,
  * i.e. cStartTime is before midnight and cEndTime is after midnight.
  * To facilitate calculation of elapsed time, we will add 24 hours to
  * cEndTime, and then calculate "normally". (For example, 01:23 becomes 25:23)
  cEndTime := str(VAL(LEFT(cEndTime,2))+24,2) +;
              IIF(LEN(cEndTime)=5,':','') +;   // Add ':' if format is HH:MM
              RIGHT(cEndTime,2)
ENDIF
RETU ( 60*VAL(LEFT(cEndTime,2)) +;
       VAL(RIGHT(cEndTime,2)) ) -;  // EndTime minutes past 00:00
     ( 60*VAL(LEFT(cStartTime,2)) +;
       VAL(RIGHT(cStartTime,2) ) )   // minus (StartTime minutes past 00:00)
*****************************************************************
Func PadR(cString,nLength,cPadChar)   && For clipper/Foxpro compatibility
return left( cString+REPL(IIF(PCOUNT()<3,' ',cPadChar),nLength), nLength )
*****************************************************************
Func PadL(cString,nLength,cPadChar)   && For clipper/Foxpro compatibility
if nLength <= len(cString)
  return left(cString,nLength)  // This might even trim it.
endif
* else pad it to the left:
return replicate(IIF(PCOUNT()<3,' ',cPadChar),nLength-len(cString)) + cString
*******************************************************
func AppQuery(cTableName,cTagName)
* Returns a zQuery object.
* Creates (if not already existing) a new element
* in _app.aQueries array.  Also creates the array if needed.
* Executes the query only once.
  if type('_app.aQueries') # 'O'
    * Create the empty array:
    _app.aQueries = new AssocArray()
  endif
  if not _app.aQueries.IsKey(cTableName)
    * Add a new element:
    _app.aQueries[cTableName] = QQuery(cTableName)
  endif
  if argCount() > 1
    _app.aQueries[cTableName].rowset.IndexName := cTagName
  endif
return _app.aQueries[cTableName]
*******************************************************
proc KillAppQuery(cTableName)  // Kill the specified one.
  if type('_app.aQueries') == 'O'
    _app.aQueries.RemoveKey(cTableName)
  endif
return
*******************************************************
proc KillAppQueries           // Kill them all.
  if type('_app.aQueries') == 'O'
    _app.aQueries.RemoveAll()
  endif
return
*******************************************************
func AppForm
  Param cFormClassName
  * Returns a form object.
  * Instantiates (if not already instantiated) the form object.
  * Creates (if not already existing) a new element in
  * _app.aForms array.  Also creates the array if needed.
  if type('_app.aForms') # 'O'
    * Create the empty array:
    _app.aForms = new AssocArray()
  endif
  if not _app.aForms.IsKey(cFormClassName)
    * Add a new element:
    private c
    c = '_app.aForms["' + cFormClassName + '"] = new ' + cFormClassName + '()'
    &c
  else
    * The form object already exists.  If it's in a foreign (non-English) language
    * and it needs to be translated back to English, we'll need to re-instatiate it.
    if type('_app.Language')=='C' and _app.Language == 'English'
      if type('_app.aForms[cFormClassName].Language')=='C' and not _app.aForms[cFormClassName].Language=='English'
        * Form.Language is not "English". So we reinstantiate:
        private c
        c = '_app.aForms["' + cFormClassName + '"] = new ' + cFormClassName + '()'
        &c
      endif
    endif
  endif
return _app.aForms[cFormClassName]
*******************************************************
proc KillAppForm(cFormClassName)  // Kill reference to the specified form.
  if type('_app.aForms') == 'O'
    _app.aForms.RemoveKey(cFormClassName)
  endif
return
*******************************************************
proc KillAppForms                // Kill them all.
  if type('_app.aForms') == 'O'
    _app.aForms.RemoveAll()
  endif
return
***********************************************
FUNC AdjustTime( PTIME, ADJUST )
* returns the parameter after the specified adjustment.
* e.g. ADJUSTTIME("2359",1)  returns "2400"
*      ADJUSTTIME("23:59",1) returns "24:00"
LOCAL MINPAST0000,HR,MIN
MINPAST0000 = VAL(LEFT(PTIME,2))*60 + VAL(RIGHT(PTIME,2)) + ADJUST
* MINPAST000 is minutes past midnite
HR = INT(MINPAST0000/60)
MIN = MINPAST0000-(HR*60)
RETU SUBS(STR(100+HR,3),2,2) + IIF(LEN(PTIME)=5,':','') + SUBS(STR(100+MIN,3),2,2)
**************************************************
* Create table in memory
* How do we extern the DbiCreateInMemTable() BDE function?
* Does it return a rowset?
* C function example follows
/*
DBIResult fDbiCreateInMemTable(hDBIDb hTmpDb, phDBICur phTmpCur)
{
   DBIResult    rslt;
   UINT16       NumFields;

   FLDDesc fldDesc[] = {
         { // Field 1 - ALPHA
            1, "MyAlpha", fldZSTRING, fldUNKNOWN, 10, 0,
            0, 0, 0, fldvNOCHECKS, fldrREADWRITE
         },
         {  // FIELD 2 - NUMERIC
            2, "MyNumber", fldFLOAT, fldUNKNOWN, 0, 0, 0,
            0, 0, fldvNOCHECKS, fldrREADWRITE
         }
   };
   NumFields = sizeof(fldDesc) / sizeof(FLDDesc);

   rslt = Chk(DbiCreateInMemTable(hTmpDb, "InMemTbl", NumFields, fldDesc,
                                   phTmpCur));
   return rslt;
}   
*/
****************************************************
Func NewTempFileName( cSkeleton )
  if empty(cSkeleton)
    cSkeleton := 'T_??????.DBF'  // Defaults to this if param is omitted.
  endif
  local cFileName
  cFileName = FUnique(cSkeleton)
  if type('_app.TempTables') == 'U'
    _app.TempTables = new array()
  endif
  _app.TempTables.Add(cFileName)  // Add it to an array of filenames that are to be deleted when the app is closed.
return cFileName
*****************************************************
func RealValue( oField )
* Calling example:  c = RealValue(r.fields['ExamType_No'])
  if oField.ClassName == 'FIELD'
    * This is a calculated field.
    return oField.value
  endif
  local saveBeforeGetValue, retVal
  saveBeforeGetValue = oField.BeforeGetValue
  oField.BeforeGetValue := null  // Temporarily disable any morphing
  retVal = oField.Value
  oField.BeforeGetValue := saveBeforeGetValue  // Turn morphing back on, if any.
return retVal
****************************************************
Func ListIndexInfo(cTableName)
?
? 'Indexes for '+cTableName+':'
local t, i
t = new tableDef()
t.tableName = cTableName
t.load()
for i = 1 to t.indexes.size
  ?
  with (t.indexes[i])
    ? '  ' + Proper(indexName)
    ? '    ' + expression
    if not empty(ForExpression)
      ? '    For ' + ForExpression
    endif
    if Descending
      ? '    Descending'
    endif
    if Unique
      ? '    Unique'
    endif
    if Type # 0
      ? '    Type: '+Type
    endif
  endWith
next
return
***************************************************
function Alert(cTitle,cMessage)
   local f
   set procedure to Alert.Wfm Additive
   f = New AlertForm()
   f.Mdi = false
   f.Title.Text = cTitle
   f.Message.Text = cMessage
   f.ReadModal()
return
****************************************************
// fill in later
// this function is refered to in several places but never 
// declared.  Pete forgot? I'm leaving it blank for now 
// but the user will be able to enter any wierd thing they 
// want which may result in is strange data!
function ShowError
return
*****************************************************
function MakeOneStringDelimited(cDelimiter, aOfStrings)
local cAllStrings
cAllStrings = ''
if cDelimiter = null
   cDelimiter = ''
endif
if aOfStrings.size = 0
   return ''
endif
for i = 1 to aOfStrings.size
	if aOfStrings[i] = null
      cThisString = ''
   else
      cThisString = trim(aOfStrings[i])
   endif
   if not empty(cAllStrings) and not empty(cThisString)
   	cAllStrings += cDelimiter
   endif
   cAllStrings += cThisString
next
return cAllStrings
*****************************************************
function copyOutOfEXE( myfile )
  local lCopied
  lCopied = false
  // if file is not on disk, but is in the executable
  if ( not file( myfile ) and file( myfile, true ) )
    copy file (myFile) to (myfile) // extract from .exe
    lCopied := true
  endif
return lCopied
*****************************************************
func FirstNameLastName( cLastFirst )
// E.g. FirstNameLastName('Smith, John') returns "John Smith"
  if localType(cLastFirst) # 'C'
    return ''
  endif
  local nCommaPosition
  nCommaPosition = at(',',cLastFirst)
  if nCommaPosition == 0
    return trim(cLastFirst)
  endif
return subs( trim(cLastFirst), nCommaPosition+2 ) + ' ' + left(cLastFirst, nCommaPosition-1 )
*****************************************************
func TranslateFrameWinMenu

  _app.framewin.app.rootmenu.release()    

  local cMenuClassName
  cMenuClassName = _app.frameWin.app.menuClassName
  private c
  c = '_app.framewin.app.rootmenu = new ' + cMenuClassName + '( _app.framewin, "Root")'
  &c

  if type('_app.framewin.app.rootmenu.onInitMenu') $ 'CB/FP'
    _app.framewin.app.rootmenu.onInitMenu()  // OnInitMenu is not firing automatically - why?
  endif

  _app.framewin.app.rootmenu.TranslateTexts()
*  shell( true, true )

return
*****************************************************
Func ExtractTableNameFromSQL(cSQL)  
  // Returns a string containing the name of the table, 
  // extracted from a SELECT...FROM... string.
  local c, nAt
  c = subs( cSQL, AT('from ',lower(cSQL))+5 )
  if left(c,1) == [']
    c = subs(c,2)
    nAt = AT(['],c)
    return left(c,nAt-1)
  elseif left(c,1) == ["]
    c = subs(c,2)
    nAt = AT(["],c)
    return left(c,nAt-1)
  elseif left(c,1) == "["
    c = subs(c,2)
    nAt = AT("]",c)
    return left(c,nAt-1)
  else
    * c does not begin with a string delimiter.
    nAt = AT(" ",c)
    if nAt > 0
      return left(c,nAt-1)
    else
      return c
    endif
  endif
*****************************************************
Func TypeOfObject
  param o  // An object
  //
  // Returns an upper case string describing the name
  // of the base class (the one at the top of the 
  // inheritance tree) of object o.
  // By Peter Rorlick
  //
  do case  // The order of the cases is vital.
  case type('o.BaseClassName') == 'C'
    * If this custom property exists, use it.
    * (Esp. to distinguish between a subclassed 
    * RadioButton and a subclassed Checkbox.)
    return upper(o.BaseClassName)
  case type('o.ReportGroup') == 'O'
    return 'REPORT'
  case type('o.InDesign') == 'L'
    return 'FORM'
  case type('o.rotate') == 'N'
    return 'TEXT'
  case Type('o.First')=='O' and type('o.UseTablePopUp')=='U'
    if type('o.datasource') == 'C'
      return 'NOTEBOOK'
    else
      return 'CONTAINER'
    endif
  case type('o.columns') == 'O'
    return 'GRID'
  case type('o.SpeedBar') == 'L'
    return 'PUSHBUTTON'
  case type('o.PatternStyle') == 'N'
    return 'RECTANGLE'
  case type('o.EvalTags') == 'L'
    return 'EDITOR'
  case type('o.Multiple') == 'L'
    return 'LISTBOX'
  case type('o.DropDownHeight') == 'N'
    return 'COMBOBOX'
  case type('o.alias') == 'C'
    return 'BROWSE'
  otherwise
    return o.className
  endcase
*****************************************************
//// This doesn't seem to work very well, at least under NT.
//Func RunDOScommand( cCommand )  // Like RUN but suppresses the DOS Window.
//  if type("WinExec") # "FP"
//      extern cINT WinExec(CSTRING, CINT) kernel32
//  Endif
//  WinExec(cCommand,0)
//return
*-----------
//// This version works, but doesn't suppress the DOS window:
//Func RunDOScommand   // Like RUN but suppresses the DOS Window.
//  param cCommand
//  freeze()
//  Run &cCommand
//  unfreeze()
//return
*****************************************************
Func ProperName( cName )
  local s
  set proc to exString.cc addi
  s = New ExString(cName)
return s.ProperName()
****************************************************
Func FileDateAndTime(cFilename)
  // Returs a string: YYYYMMDD hh:mm:ss
  // Param can include the path.
  local a
  a = new array()
  a.dir(cFileName)
  if a.size == 0
    return ''
  endif
return dtos(a[3]) +' '+ a[4]
*****************************************************
proc Dbug( nMode )
  * The param is optional.  
  * If you pass a param, then we invoke the debugger only if _app.debug==true.
  * Note that you can also turn on the debugger by pressing Alt-D (See setprocs.prg).
  if not "runtime" $ LOWER( VERSION(0) )
    if argCount() == 0
      set step on
    elseif type('_app.debug')=='L' and _app.debug
      set step on
    endif
  endif
return
****************************************************
* endclass  // uncomment this to get the left pane in alphabetical order.

