* Picker Classes      Author: Peter Rohrlick      June 1996
* VdB7 version

* Important: These controls MUST be used on forms that have zForm somewhere in
* their inheritance tree.  Otherwise, their Init methods will not automatically
* be run, and lots of errors will ensue.  The benefit: your forms will open
* MUCH faster.

*************************************************************************************
SET PROC TO zControl.CC ADDITIVE   // Programs using these classes must do this.

CLASS ENTRYPicker(FormObj) OF zENTRYFIELD(FormObj) Custom
  with (this)
*  Key = CLASS::Key
   *OnOpen         = CLASS::DefinePickerButton
   OnLeftDblClick = CLASS::PushTheButton
   OnGotFocus     = CLASS::EntryPicker_OnGotFocus
   OnLostFocus    = CLASS::EntryPicker_OnLostFocus
   Left = 13.332
*  FontBold = .T.
   Top = 10.9395
   Width = 24.5
   Value = This.name
   SelectAll = false   // So that selected strings are immediately visible even if they have lots of trailing spaces.
 endwith

   Procedure Init
   * Instead of the OnOpen event, which is slow cause it occurs AFTER the form is open.
   * Remember to call EntryPicker::Init() if you over-ride this method.
   CLASS::DefinePickerButton()
   return

   Procedure ExtractAlias
   IF TYPE('This.PkrAlias') # 'C'
     * .PkrAlias does not have to be supplied by the programmer when
     * using the picker classes, as long as the .PkrReturnExpression
     * begins with an alias, e.g. "Patients->LastName"
     IF TYPE('This.PkrReturnExpression') # 'C'
       msgBox('Warning: PkrReturnExpression property is missing for '+this.name)
       * For quick testing. Form will crash if user clicks on picker buttons.
     else
       This.PkrAlias = LEFT(This.PkrReturnExpression,AT('->',This.PkrReturnExpression)-1)
     endif
   Endif
   *-----------------------------------------------------------


   Procedure DefinePickerButton
     *
     EntryPicker::ExtractAlias()
     *
  *   zEditor::DefineEditorPopup()    // *? This lines causes instability!
     IF TYPE('This.PkrMultiSelector')='L' .AND. This.PkrMultiSelector = .T.
       * Background of RG+ indicates an editable MultiPicker:
       This.ColorHighlight = "N/RG+"
  **    This.Key = NULL
     ELSE
       * Background of G+ indicates a non-editable SinglePicker:
       This.ColorHighlight = "N/G+"
     ENDIF
  *   LOCAL lEnabled  
  *   lEnabled = This.Enabled //Uncommented by SP...
     This.Enabled = .T.    && No longer any such thing as a disabled picker. 
     PRIV cBtnName, c
     cBtnName = 'Pkr'+TRIM(This.Name)
     IF TYPE('this.parent.'+cBtnName) = 'O'
       RETU   && Object of this name is already defined
     ENDIF
     c = 'this.parent.'+cBtnName
     PRIV cn
     cn = c+' = new PUSHBUTTON(this.parent)'
     &cn
     this.oPlusButton = &c
     with (&c)
       group = true
       Top = This.Top + .1
       Left = This.Left + This.Width - (2*2.835) - .3
       Height = This.Height - .15
       Width = 2.835
       Text = ""
       OnClick = CLASS::Pkr_OnClick
       PageNo = This.PageNo
       UpBitmap = "FILENAME Plus.BMP"
       TabStop = false
       SpeedBar = true
  *     enabled = lEnabled  //New by SP...
     endwith
  
     priv oButton
     oButton = &c
     oButton.oOwner = this  // Create a reference to the Entryfield that owns the new button.
     oButton.onFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-(2*2.835)-0.3, this.oOwner.Top+0.1) }
     oButton.cOwner = This.Name   && Name of editfield that owns this button
                      && This.cOwner is a string;
                      && &("Form."+This.cOwner) is an object.
  
  
     * Now establish the button for the DropDown list:
     cBtnName = 'PkrD'+TRIM(This.Name)
     c = 'this.parent.'+cBtnName
     PRIV cn
     cn = c+' = new PUSHBUTTON(this.parent)'
     &cn
     this.oDropDownButton = &c
     with (&c)
       group = true
       Top = This.Top + .1
       Left = This.Left+This.Width-2.835 - .3
       Height = This.Height - .15
       Width = 2.835
       Text = ""
       OnClick = CLASS::DropDownButton_OnClick
       PageNo = This.PageNo
       UpBitmap = "FILENAME pickdrop.BMP"
       TabStop = false
       SpeedBar = true
     endwith
  
     oButton = &c
     oButton.oOwner = this  // Create a reference to the Entryfield that owns the new button.
     oButton.onFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-2.835-0.3, this.oOwner.Top+0.1) }
     oButton.cOwner = This.Name   && Name of editfield that owns this button
  
     this.SetVisibility( this.visible )  // To set visibility of pkr buttons.

   return
   *-----------------------------------------------------


   Procedure SetVisibility( lVisible )
     // Sets visibility of the entryfield and its buttons.
     // This method should be used instead of directly setting the Visible property.
     this.visible := lVisible
     This.oPlusButton.visible := lVisible and ( type('_app.PermitPowerTools')=='U' or _app.PermitPowerTools )
     This.oDropDownButton.visible := lVisible
   return


   Proc Enable   // Enables the entryfield plus its buttons
     This.Enabled = true
     This.oPlusButton.Enabled = true
     This.oDropDownButton.Enabled = true
   return


   Proc Disable  // Disables the entryfield and its buttons
     This.Enabled = false
     This.oPlusButton.Enabled = false
     This.oDropDownButton.Enabled = false
   return


   PROC DropDownButton_OnClick
     PRIV c
     c = 'this.parent.CurrentPicker = This.parent.'+this.cOwner
     &c      && For easy reference to the target entryfield in the select and abort methods.
     c = "this.parent."+This.cOwner+".SetFocus()"
     &c
     c = "this.parent."+This.cOwner+".DropDownList()"
     &c
     *  KEYB "|"    && This will force the dropdown via the Key() proc.
   return


   PROC Pkr_OnClick
   if this.visible = false
     * This occurs for users who are not permitted access to the "power tools".
     return  // Deny access to this feature.
   endif
   PRIV c, cMulti, cClassName
   *
   c = "this.parent."+This.cOwner+".SetFocus()"
   &c    && set focus to the object that owns this button.
   *
   c = "this.parent."+This.cOwner
   cMulti = c + '.PkrMultiSelector'
   cClassName = c + '.ClassName'
   IF TYPE(cMulti)='L' .AND. &cMulti = .T.
     * MultiPicker:
     DO PickM.WFM WITH &c   && Pass the object that owns the button
   ELSE
     * Single picker:
     DO Picker.WFM WITH &c   && Pass the object that owns the button
   ENDIF
   *
   * Re-establish behaviour of ENTER to invoke the picker:
   STATIC oPickerButton
   c = 'this.parent.'+TRIM(This.Name)
   oPickerButton = &c
   ON KEY LABEL Shift+F2 oPickerButton.OnClick()
   IF UPPER(&cClassName.)=='ENTRYPICKER'   && If for EntryPicker (i.e. not for an EditorPicker)
     ON KEY LABEL Enter oPickerButton.OnClick()
   ENDIF
   *
   RETU


   PROCEDURE PushTheButton
   PRIV c
   c = 'this.parent.Pkr'+TRIM(This.Name)+'.OnClick()'
   &c


   PROCEDURE EntryPicker_OnGotFocus
   This.WhatUserTyped = ''    && For the Key() procedure.
   * Arrange it so the user can hit ENTER or Shift+F2 to invoke the picker:
   STATIC oPickerButton, oDropDownButton
   PRIV c
   * c = 'this.parent.Pkr'+TRIM(This.Name)
   oPickerButton = this.oPlusButton
   ON KEY LABEL Shift+F2 oPickerButton.OnClick()
   IF UPPER(This.ClassName)=='ENTRYPICKER'    && If for EntryPicker (i.e. not for EditorPicker)
     ON KEY LABEL Enter oPickerButton.OnClick()
   ENDIF
   * Explanation: In theory we should be able to accomplish this the
   * "normal" way, i.e.:
   *   PROCEDURE Key( nChar, nPosition )
   *   IF nChar = 10  .OR. nChar=13     && Ctrl+ENTER   OR   Enter
   *     * Activate the associated button:
   *     CLASS::PushTheButton()
   *   ENDIF
   *   RETU .T.
   * However, the SEEKER's Key method gets disabled when we do it this way.
   * Maybe you just can't nest Key routines.
   *
*  ON KEY LABEL Shift+DnArrow KEYB '|'   && Drop-down list
   * c = 'this.parent.PkrD'+TRIM(This.Name)
*  oDropDownButton = &c
   oDropDownButton = this.oDropDownButton
*  ON KEY LABEL Shift+DnArrow oDropDownButton.OnClick()   && Drop-down list
   ON KEY LABEL F2 oDropDownButton.OnClick()   && Drop-down list
   *
   * Arrange behaviour of up-arrow, down-arrow:
   IF TYPE('This.PkrMultiSelector')#'L' .OR. This.PkrMultiSelector = .F.
     * This is NOT a MultiPicker:
     ON KEY LABEL UpArrow KEYB '{'     && Fill field with next entry from the pick-list
     ON KEY LABEL DnArrow KEYB '}'      && Fill field with previous entry from the pick-list
     * Behaviour of these keys is handled in PROC Key() below.
   ENDIF
   *
*   if type('this.AssignValue') == 'C'
*     * tMsgBox(this.value,'',48)
*     * tMsgBox(this.Assignvalue,'',48)  // Nothing wrong according to these 2 msgboxes, but the next line sometimes crashes: "Expecting object".
*     this.value = this.AssignValue    // AssignValue gets set in Picker.wfm.
*     this.AssignValue =  0     && So that OnChange() methods can check the TYPE() of this property to determine whether to refer to .Value or .AssignValue.
*   endif
   *
   return

   PROCEDURE EntryPicker_OnLostFocus
   ON KEY LABEL Enter       && Remove the association
   ON KEY LABEL UpArrow 
   ON KEY LABEL DnArrow 
   *ON KEY LABEL Shift+DnArrow
   ON KEY LABEL F2
   ON KEY LABEL Shift+F2


  PROCEDURE Key( nChar, nPosition )     
  * AutoFill the entryfield with incrementally searched values.
  * This.WhatUserTyped = '' is initialised in PROCEDURE EntryPicker_OnGotFocus
  LOCAL nSkip
  nSkip = 0
  DO CASE
* CASE nChar = ASC('|')       && Shift + Down-arrow
*   CLASS::DropDownList()
*   RETU .F.
  CASE TYPE('This.PkrMultiSelector')='L' .AND. This.PkrMultiSelector = .T.
    * Trap Alt-D to invoke date-inserter tool.
    If nChar == 4  // Alt-D
      zEntryField::InsertDate()
      * return false
    EndIf
    RETU .T.
  CASE nChar = 9 .OR. nChar = -400       && Tab or Shift-Tab
    RETU .T.
  CASE nChar = 8       && 8 is backspace
    This.WhatUserTyped = LEFT(This.WhatUserTyped,LEN(This.WhatUserTyped)-1)
    IF LEN(This.WhatUserTyped) = 0
      IF .NOT.EMPTY(This.OnChange)
        This.OnChange()
      ENDIF
      IF TYPE('This.PkrCodeBlockOnSelect')='CB'
        This.PkrCodeBlockOnSelect()
      ENDIF
      KEYB "{Home}"     && Unhighlight the first char in the field.
      RETU .F.
    ENDIF
  CASE nChar = 127     && 127 is Ctrl-backspace
    This.WhatUserTyped = ''
    This.Value = ''               && Clear the entryfield
    IF .NOT.EMPTY(This.OnChange)
      This.OnChange()
    ENDIF
    IF TYPE('This.PkrCodeBlockOnSelect')='CB'
      This.PkrCodeBlockOnSelect()
    ENDIF
    RETU .F.
  CASE nChar = ASC('{')      &&  Previous picklist item
    nSkip = -1
    This.WhatUserTyped = TRIM(This.Value)   && Prepare for the SEEK
    * WARNING: This will only REALLY work properly when the .PkrReturnExpression 
    * corresponds to the index key (or to a LEFT() portion of it).
  CASE nChar = ASC('}')      &&  Next picklist item
    nSkip = 1
    This.WhatUserTyped = TRIM(This.Value)   && Prepare for the SEEK
  CASE nChar >= 32    && if the user typed a printable char...
    This.WhatUserTyped += CHR(nChar)
  OTHE        &&  Other chars, such as navigation keys, will just beep:
    Beep()
    RETU .F.
  ENDCASE
/*
****** OODML version: ***************
  ** seek This.WhatUserTyped :
  if type('this.Query')=='O'
    * The lookup query for this entrypicker is already open.
  else
    this.query = new zQuery()
    this.query.SQL = 'select * from '+ This.PkrAlias
    this.query.active = true
    if type('this.PkrTag')=='C'
      this.query.rowset.IndexName = this.PkrTag
    else
      this.query.rowset.IndexName = ????  // The first tag in the index order bag - the OODML equivalent of TAG(1).
    endif
    *
    if not "ROWSET" $ upper(PkrReturnExpression)
      * Convert PkrReturnExpression to OODML:
      * Remove aliasing (State->Code becomes Code)
      *  Add rowset references (Code becomes This.query.rowset.fields['Code'].value)
      *  Must parse the expression to isolate fieldnames:
      *    padr(trim(LASTNAME)+', '+FIRSTNAME,100)+PA_ID  // field names in CAPS must be converted to: This.query.rowset.fields['WHATEVER'].value
      ?????????
    endif
  endif
  if type('This.PkrSetKeyTo')#'U'
    this.query.rowset.SetRange( This.PkrSetKeyTo )
    This.SeekPrefix = This.PkrSetKeyTo
  else
    This.SeekPrefix = ''
  endif
  if type('This.PkrFilter')=='C'
    * Watch out: The filter must now (as of June 23 1998) be a SQL expression!
    * Not backward compatible to xbase filters!
    this.query.rowset.filter = This.PkrFilter
  endif
  IF this.query.rowset.findKey( This.SeekPrefix + UPPER( TRIM( This.WhatUserTyped ) ) ) .OR.;
        nSkip<>0 .AND. EMPTY(This.Value)
    .........etc.........
  ** Remember to use bookmark instead of nSaveRec. Also no need to restore workarea.
**********************************
*/
  * seek This.WhatUserTyped :
  Form.SaveWorkArea = WorkArea()
  * Warning: If the current alias = the lookup alias, this won't work!
  IF SELECT(This.PkrAlias) = 0     && Is the lookup table is already open?
    SELE Select()
    USE (This.PkrAlias)
  ELSE
    SELE (This.PkrAlias)
  ENDIF
  LOCAL cSaveTag
  cSaveTag = Tag()
  IF TYPE('This.PkrTag')='C'
    SET ORDER TO TAG (This.PkrTag)
  ELSE
    SET ORDER TO Tag(1)
  ENDIF
  IF TYPE('This.PkrSetKeyTo')#'U'
    SET KEY TO This.PkrSetKeyTo
    This.SeekPrefix = This.PkrSetKeyTo
  ELSE
    This.SeekPrefix = ''
  ENDIF
  IF TYPE('This.PkrFilter')='C'
    PRIV c
    c = This.PkrFilter
    SET FILTER TO &c
  ENDIF
  IF SEEK( This.SeekPrefix + UPPER( TRIM( This.WhatUserTyped ) ) ) .OR.;
        nSkip<>0 .AND. EMPTY(This.Value)
    PRIV c
    c = 'Trim(' + This.PkrReturnExpression + ')'  // Trimming is new as of Feb 19 1998
    IF nSkip <> 0
      * User pressed down- or up-arrow. 
      LOCAL nSaveRec
      nSaveRec = RECN()
      IF EMPTY(This.Value)
        * Field was empty when user hit Up or Down arrow, so fill it with
        * the first item on the pick-list:
        GO TOP
      ELSE        && Skip to next or previous record:
        IF FOUND()
          SKIP nSkip
        ELSE
          * The .PkrReturnExpression probably does NOT corespond to the 
          * index key (or to a LEFT() portion of it).
          * For example, a lookup table might be indexed on Code but
          * the picker could return a Description.
          * The method we use here to work around this flaw involves
          * a .LastRecNoSelected property which we maintain for this control.
          * But this is not perfect: for example, when an item is selected
          * from a dropdown list, we don't maintain this property correctly.
          IF TYPE('This.LastRecNoSelected')='N' .AND. This.LastRecNoSelected>0 .AND. This.LastRecNoSelected<RECC()
            GO This.LastRecNoSelected
            SKIP nSkip
          ELSE
            GO TOP    && Better than nothing.
          ENDIF
        ENDIF
      ENDIF
      IF EOF() .OR. BOF()
        Beep()
        IF nSaveRec > 0 .AND. nSaveRec<RECC()
          GO nSaveRec
        ENDIF
      ELSE
        This.WhatUserTyped = &c
      ENDIF
    ENDIF
    This.Value = &c
    This.LastRecNoSelected = RECN()
    IF .NOT.EMPTY(This.OnChange)
      This.OnChange()   && Trigger the OnChange method, as if the user typed in the new value.
    ENDIF
    IF TYPE('This.PkrCodeBlockOnSelect')='CB'
      This.PkrCodeBlockOnSelect()
    ENDIF
    * Highlight what the user has typed:  (chr 26 is Home)
    this.Keyboard( "{Home}" + REPL("{Shift+RightArrow}",LEN(This.WhatUserTyped)) )
*   this.Keyboard( "{Home}" + REPL("{RightArrow}",LEN(This.WhatUserTyped)) + "{Shift+Home}" )
  ELSE     && not found
    Beep()     && beep to indicate not found
    * Remove the last char from the search string:
    This.WhatUserTyped = LEFT(This.WhatUserTyped,LEN(This.WhatUserTyped)-1)
  ENDI
  SET ORDER TO TAG (cSaveTag)
  SELE (Form.SaveWorkArea)
  this.setfocus()
  return false
  *--------------- End Key() event subroutine --------------------


  Procedure DropDownList
  * Define, on-the-fly, a listbox populated with ReturnExpression values
  * that respect the targetfield's .Pkr?????? specifications:
  Form.SaveWorkArea = WorkArea()
  * Warning: If the current alias = the lookup alias, this won't work!
  IF SELECT(This.PkrAlias) = 0     && Is the lookup table is already open?
    SELE Select()
    USE (This.PkrAlias)
  ELSE
    SELE (This.PkrAlias)
  ENDIF
  LOCAL cSaveTag
  cSaveTag = Tag()
  IF TYPE('This.PkrTag')='C'
    SET ORDER TO TAG (This.PkrTag)
  ELSE
    SET ORDER TO Tag(1)
  ENDIF
  IF TYPE('This.PkrSetKeyTo')#'U'
    SET KEY TO This.PkrSetKeyTo
  *  This.SeekPrefix = This.PkrSetKeyTo
  *ELSE
  *  This.SeekPrefix = ''
  ENDIF
  Priv C
  IF TYPE('This.PkrFilter')='C'
    c = This.PkrFilter
    SET FILTER TO &c
  ENDIF
  * Fill the pick-array with a value for each record under this filter/key:
  PRIV aPickValues
  c = This.PkrReturnExpression
  aPickValues = NEW ARRAY()
  Form.MousePointer = 11     && Hourglass
  local lSmallLookupTable
  lSmallLookupTable = (recc()<2000)
  go top  // Fixes bizarre problem where rows were duplicated or omitted.
  local nWidest, cItem
  nWidest = 0
  SCAN while lSmallLookupTable or aPickValues.Size < 1000
    cItem = &c
    aPickValues.Add( cItem )
    if len( cItem ) > nWidest
      nWidest := len( cItem )
    endif
  ENDSCAN
  Form.MousePointer = 1      && Pointer
  SET ORDER TO TAG (cSaveTag)
  if aPickValues.Size = 1000
    tMsgBox('The pick-list has too many choices to show in a drop-down list. Click OK or press Enter to get a Picker dialog.')
    Keyb CHR(13)    // Invoke PICKER.WFM 
    select (Form.SaveWorkArea)
    return
  elseif aPickValues.Size = 0
    *IF MyMsgBox('There are no items '+;
    *   IIF(TYPE('This.PkrSetKeyTo')='U',' to select from.',;
    *   'for '+IIF(TYPE('This.PkrScopeCategory')#'U',;
    *   This.PkrScopeCategory+' = ','')+This.PkrSetKeyTo)+CHR(13)+CHR(10)+;
    *   'You can add new items to the pick-list'+IIF(TYPE('This.PkrScopeCategory')='U','.',', OR you should go back and select the correct '+This.PkrScopeCategory+'.'),'',;
    *   {'OK','Add a new item to the pick-list'}) = 2
    *  Keyb CHR(13) + "{Alt-N}"  && Invoke PICKER.WFM and fire the New button.
    *ELSE
    *  form.setFocus()
    *  *?    // "Class does not exist" error occurs.
    *ENDIF
    ** The above is commented out because of the "Class does not exist" error.
    ** Until we fix that, we'll do it more simply:
    tmsgBox('There are no items '+;
       IIF(TYPE('This.PkrSetKeyTo')='U',' to select from.',;
       'for '+IIF(TYPE('This.PkrScopeCategory')#'U',;
       This.PkrScopeCategory+' = ','')+This.PkrSetKeyTo)+CHR(13)+CHR(10)+;
       'You can add new items to the pick-list'+IIF(TYPE('This.PkrScopeCategory')='U','.',', OR you should go back and select the correct '+This.PkrScopeCategory+'.'),'No items',48)
    *
    select (Form.SaveWorkArea)
    return
  ELSE
    LOCAL nDropDownHeight, nDropDownTop
    nDropDownHeight = MIN(aPickValues.Size,9)   && Height is 9 or less.
    DO CASE
    CASE this.parent.Height-This.Top-1>nDropDownHeight   && If there is room below...
      nDropDownTop = This.Top+1                && Show it below this control
    CASE This.Top>nDropDownHeight                 && If there is room above...
      nDropDownTop = This.Top-nDropDownHeight  && Show it above
    OTHE
      * this.parent.Height = This.Top + 1 + nDropDownHeight + .2   && Increase parent's height
      nDropDownTop = This.Top+1                && Show it below this control
    ENDCASE
    *
    freeze(form)
    LOCAL lMultiple
    lMultiple = (TYPE('This.PkrMultiSelector')='L'.AND.This.PkrMultiSelector)
    This.Parent.DropDownList = new LISTBOX(This.Parent)
    with (This.Parent.DropDownList)
      ColorNormal = IIF(lMultiple,'N/RG+','N/G+')
      ColorHighLight = "HighLightText/HighLight"
      OnGotFocus = CLASS::DropDownList_OnGotFocus
      OnLostFocus = CLASS::DropDownList_OnLostFocus
      OnLeftDblClick = CLASS::DropDownList_OnLeftDblClick
      OnMiddleMouseDown = CLASS::DropDownList_OnMiddleMouseDown
      Left = This.Left
      Top = nDropDownTop
      Height = nDropDownHeight
      Width = MIN( nWidest+5, This.Parent.Width-This.parent.DropDownList.left-1 )
      PageNo = This.PageNo
      DataSource = "ARRAY aPickValues"
      Multiple = lMultiple
      Sorted = true
    endwith
    IF .NOT. lMultiple   && If SinglePicker...
      * Allow a single click to select an item:
      this.parent.DropDownList.OnLeftMouseUp = CLASS::DropDownList_OnLeftDblClick
    ENDIF
    _app.DropDownList = this.parent.DropDownList
*    ON KEY LABEL Enter _app.DropDownList.OnLeftDblClick()      && Select
*    ON KEY LABEL Esc   _app.DropDownList.OnMiddleMouseDown()   && Abort
    this.parent.DropDownList.SetFocus()
*   this.parent.CurrentPicker = This   && For easy reference in the select and abort methods.
    *
    * If there is not enough vertical room on the form to show the listbox
    * either above or below this control, then we will temporarily increase
    * the height of the form.
    // Saving All Heights...
    oObject = This.Parent.DropDownList  // The listbox
    Do While This.Form.ClassName <> oObject.ClassName   // do for each parent that is not the Form...
*     MsgBox(STR(oObject.Parent.Height)+ "  ,  "+STR(oObject.Top+oObject.Height))
      If oObject.Parent.Height < oObject.Top+oObject.Height   // If the parent of oObject needs to be resized to accommodate the new size of oObject...
        oObject.Parent.OriginalHeight = oObject.Parent.Height  // Save original height for later
          oObject.Parent.Height = oObject.Top + oObject.Height + 0.3  // resize the parent
      Else
        oObject.Parent.OriginalHeight = -1  // flag to indicate that no resizing was done for the parent of oObject and all of ITS parents.
        EXIT
      EndIf
      oObject = oObject.Parent  
    EndDo
    unfreeze()
    *
  ENDIF
  *
  SELE (Form.SaveWorkArea)
  return
  *---------------End of proc DropDownList---------------


  Procedure DropDownList_OnGotFocus
     ON KEY LABEL Esc   _app.DropDownList.OnMiddleMouseDown()
     ON KEY LABEL Enter _app.DropDownList.OnLeftDblClick()


  Procedure DropDownList_OnLostFocus
    ON KEY LABEL Esc
    ON KEY LABEL ENTER
    // Restore Heights of all containers/parents as needed:
    *    Note: There is a refreshing problem that surfaces here occasionally,
    *          when there are 2 or more "containers" on a form. If it occurs,
    *          you have to change the z-order of the containers on the form.
    *          Unfortunately, this means you might have to live with a tabbing
    *          order that you don't like.
    oObject = this  // The Listbox
    Do While This.Form.ClassName <> oObject.ClassName
     If TYPE("oObject.Parent.OriginalHeight") == 'N'
      If oObject.Parent.OriginalHeight > 0   // If it's not -1
        oObject.Parent.Height = oObject.Parent.OriginalHeight
      Else
        EXIT
      EndIf
      oObject = oObject.Parent
     Else
       EXIT
     EndIf
    EndDo
    local oCurrent
    oCurrent = This.Parent.CurrentPicker
    This.Release()  // get rid of the ListBox
    *
    form.Repaint()
*    local nSavePageNo
*    nSavePageNo = Form.PageNo
*    Form.PageNo = 99
*    Form.PageNo = nSavePageNo   // To solve the refresh problem.
    *
    oCurrent.SetFocus()

  Procedure DropDownList_OnLeftDblClick    && SelectFromDropDownList    && Triggered by doubleClick or Enter
    private oTargetField
    oTargetField = This.Parent.CurrentPicker
    IF This.Multiple = .T.
      LOCAL cRetVal, i
      *cRetVal = ''
      cRetVal = trim(oTargetField.Value)  // We'll add to what was already in the field.
      FOR i=1 TO This.Count()
        IF .NOT.EMPTY(This.Selected(i))
          cRetVal = cRetVal + IIF(EMPTY(cRetVal),'',', ') + TRIM(This.Selected(i))
        ENDIF
      NEXT
      oTargetField.Value = cRetVal
    ELSE
      oTargetField.Value = This.Value
    ENDIF
    IF .NOT.EMPTY(oTargetField.OnChange)
      oTargetField.OnChange()   && Trigger the OnChange method, as if the user typed in the new value.
    ENDIF
    IF TYPE('oTargetField.PkrCodeBlockOnSelect') $ 'CB/FP'
      oTargetField.PkrCodeBlockOnSelect()
    ENDIF
    This.LastRecNoSelected = 'JUNK'   && ensure consistent behaviour
    oTargetField.SetFocus()
  return


  Procedure DropDownList_OnMiddleMouseDown   && AbortDropDownList   && Triggered by ESC
    This.Parent.CurrentPicker.SetFocus()


ENDCLASS

*********************************************************************************
CLASS AllOrSpecificPicker(FormObj) OF EntryPicker(FormObj) Custom

   * An entryPicker with an ALL button next to it.
   * The entryPicker is given a value of "ALL" instead of being empty.
   * Used for defining filters, report criteria, etc.

   * 2 ways to test whether the control is currently set to ALL:
   *   ? this.ALL   // logical flag
   *   ? empty(this.value)

   *You must specifically switch on/off the checkbox by adding at the top of the 
   * <SingleInstanceOfAllOrSpecificPicker>_OnChange() method the following 
   * statement:
   *	 AllOrSpecificPicker::AOS_OnChange()
   *
   this.OnChange := CLASS::ASP_OnChange


   Procedure ASP_OnChange
     * Rember to call AllOrSpecificPicker::ASP_OnChange() if 
     * over-riding with your own OnChange.
     local cNewValue
     cNewValue = iif( type('this.assignValue')=='C', this.assignValue, this.Value )
     This.ALL = empty(cNewValue) 
     this.oAllButton.upBitMap := iif(empty(cNewvalue),'FILENAME Tick.bmp','RESOURCE #600')
   return

   Procedure Init
     EntryPicker::ExtractAlias()
     EntryPicker::DefinePickerButton()
     * Now, define an "ALL" button on the fly, next to the entrypicker:
     PRIV cBtnName, c, cn
     cBtnName = 'A'+TRIM(This.Name)
     IF TYPE('This.Parent.'+cBtnName) = 'O'
       * MsgBox('DEFINE has failed, because an object called '+cBtnName+' already exists')
       RETU   && Object of this name is already defined
     ENDIF
     c = 'This.Parent.'+cBtnName
     cn = c+' = new sButton(This.Parent)'      // Was a PushButton
     &cn
     this.oAllButton = &c //98.05.05 by Serge
     with (&c)
       FontBold = false
       FontSize := 8
       Top = This.Top + .1
       Left = This.Left+This.Width-(2*2.835) - .3 -5.86   // was - 4.5
       Height = This.Height - .15
       Width = 5.86   // was 4.5
       Text = "ALL"
       Group = true
       PageNo = This.PageNo
       OnClick = CLASS::AllButton_OnClick
       tabstop = false
       SpeedBar = true
       ColorNormal = 'B+/BtnFace'
       upBitMap := iif(empty(this.value),'FILENAME Tick.bmp','RESOURCE #600')
     Endwith
     priv oButton
     oButton = &c
     oButton.oOwner = this  // Create a reference to the Entryfield that owns the new button.
     oButton.onFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-(2*2.835)-0.3-5.86, this.oOwner.Top+0.1) }
     oButton.cOwner = This.Name   && Name of editfield that owns this button
                      && This.cOwner is a string;
                      && &("Form."+This.cOwner) is an object.
     *
     this.ALL = EMPTY(this.Value)   // Set a flag for testing the ALLness condition.
     *
     this.SetVisibility( this.visible )  // To set visibility of pkr buttons.
     *
   return

   Procedure AllButton_OnClick
     this.upBitMap := 'FILENAME Tick.bmp'   //  'RESOURCE #601'  // Tick
     this.oOwner.Value := ''
     this.oOwner.OnChange()
     this.oOwner.SetFocus()
   return

   Procedure Disable
     this.Enabled = false
     this.oAllButton.enabled = false
     this.oPlusButton.enabled = false
     this.oDropDownButton.enabled = false
   return

   Procedure Enable
     this.Enabled = true
     this.oAllButton.enabled = true
     this.oPlusButton.enabled = true
     this.oDropDownButton.enabled = true
   return

   Procedure SetVisibility( lVisible )
     // Sets visibility of the entryfield and its buttons.
     // This method should be used instead of directly setting the Visible property.
     super::SetVisibility( lVisible )
     if type('this.oAllButton') == 'O'  // If oAllbutton has been defined yet...
       this.oAllbutton.visible := lVisible
     endif
   return

ENDCLASS

*************************************************************************************
CLASS EditorPicker(FormObj) OF zEDITOR(FormObj) Custom
   *this.OnOpen         = CLASS::DefinePickerButton
   this.OnLeftDblClick = EntryPicker::PushTheButton            && !
   this.OnGotFocus     = {; zEditor::YellowBackground() ; EntryPicker::EntryPicker_OnGotFocus() }  && !
   this.OnLostFocus    = {; zEditor::WhiteBackground() ; EntryPicker::EntryPicker_OnLostFocus() }  && !
   this.Left = 13.332
*  this.FontBold = .T.
   this.Top = 10.9395
   this.Width = 24.5
   this.Value = This.name
   this.ColorHighlight = "N/RG+"      && !
   this.PkrMultiSelector = true       && !    Always in MULTI mode for this class.

   Procedure Disable
      this.Enabled = false
      this.oPlusButton.Enabled = false
      this.oDropDownButton.Enabled = false
      this.oZoomButton.Enabled = false

   Procedure Enable
      this.Enabled = true
      this.oPlusButton.Enabled = true
      this.oDropDownButton.Enabled = true
      this.oZoomButton.Enabled = true

   Procedure Init
   * Instead of the OnOpen event, which is slow cause it occurs AFTER the form is open.
   CLASS::DefinePickerButton()
   return


   Procedure DefinePickerButton
   IF TYPE('This.PkrMultiSelector')='L' .AND. This.PkrMultiSelector = .T.
     This.Key = zEntryField::Key
   ENDIF
   * The next 2 lines are essentially the OnOpen of zEditor.
   zEditor::DefineZoomButton()        && !
*  zEditor::DefineEditorPopup()    // Disabled - can cause instability!
   EntryPicker::ExtractAlias()
   This.Enabled = .T.    && No longer any such thing as a disabled picker.
   PRIV cBtnName, c, cn
   cBtnName = 'Pkr'+TRIM(This.Name)
   IF TYPE('This.Parent.'+cBtnName) = 'O'
     RETU   && Object of this name is already defined
   ENDIF
   c = 'This.Parent.'+cBtnName
   cn = c+' = new PUSHBUTTON(This.Parent)'
   &cn
   this.oPlusButton = &c  //For use in new disable/enable methods
   with (&c)
     Top = This.Top+This.Height-1              && !
     Left = This.Left+This.Width-(3*2.835)     && !
     Height = 0.88
     Width = 2.835
     Text = ""
     Group = true
     OnClick = EntryPicker::Pkr_OnClick        && !
     PageNo = This.PageNo
     UpBitmap = "FILENAME Plus.BMP"   //  Was: BlueDown.bmp - caused crash in Win 3.1 with VdB 5.x
     TabStop = false        && was:   .NOT.lEnabled
     SpeedBar = true
   EndWith
   priv oButton
   oButton = &c
   oButton.oOwner = this  // Create a reference to the Entryfield that owns the new button.
   oButton.onFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-(3*2.835), this.oOwner.Top+this.oOwner.Height-1) }
   oButton.cOwner = This.Name   && Name of editfield that owns this button
                    && This.cOwner is a string;
                    && &("Form."+This.cOwner) is an object.

   * Now establish the button for the DropDown list:
   cBtnName = 'PkrD'+TRIM(This.Name)
   c = 'This.Parent.'+cBtnName
   cn = c+' = new PUSHBUTTON(This.Parent)'
   &cn
   this.oDropDownButton = &c  //For use by new disable/enable methods
   with (&c)
     Top = This.Top+This.Height-1              && !
     Left = This.Left+This.Width-(2*2.835)     && !
     Height = 0.88
     Width = 2.835
     Text = ""
     Group = true
     OnClick = EntryPicker::DropDownButton_OnClick    && !
     PageNo = This.PageNo
     UpBitmap = "FILENAME pickdrop.BMP"
     TabStop = false
     SpeedBar = true
   Endwith
   priv oButton
   oButton = &c
   oButton.oOwner = this  // Create a reference to the Entryfield that owns the new button.
   oButton.onFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-(2*2.835), this.oOwner.Top+this.oOwner.Height-1) }
   oButton.cOwner = This.Name   && Name of editfield that owns this button
                    && This.cOwner is a string;
                    && &("Form."+This.cOwner) is an object.

   Procedure DropDownList
     EntryPicker::DropDownList()
   return

   Procedure SetVisibility( lVisible )
     // Sets visibility of the editor and its buttons.
     // This method should be used instead of directly setting the Visible property.
     this.visible := lVisible
     This.oPlusButton.visible := lVisible and ( type('_app.PermitPowerTools')=='U' or _app.PermitPowerTools )
     This.oDropDownButton.visible := lVisible
     This.oZoomButton.visible := lVisible
   return

    
ENDCLASS
***************************************************************************
CLASS FilePicker(FormObj) OF zENTRYFIELD(FormObj) Custom
   *this.OnOpen         = CLASS::DefinePickerButton
   this.OnLeftDblClick = CLASS::GETFILEBUTTON_OnClick
   this.Left = 13.332
*   this.FontBold = .T.
   this.Top = 10.9395
   this.Width = 70
   this.Value = This.name
   this.DialogTitle = 'Select a file'
   this.FileSpec = '*.*'
   * The last 2 properties can be "customized" by setting different values
   * to them in the Form_OnOpen routine of the form containing your FilePicker.


   Procedure Init
   * Instead of the OnOpen event, which is slow cause it occurs AFTER the form is open.
   CLASS::DefinePickerButton()
   return


   Procedure DefinePickerButton
*  zENTRYFIELD::DrawFrame()
*  Super::OnOpen() 
*  zEditor::DefineEditorPopup()    // Disabled - can cause instability!
   PRIV cBtnName, c
   cBtnName = 'Pkr'+TRIM(This.Name)
   IF TYPE('This.Parent.'+cBtnName) = 'O'
     RETU   && Object of this name is already defined
   ENDIF
   c = 'This.Parent.'+cBtnName
   PRIV cn
   cn = c+' = new PUSHBUTTON(This.Parent)'
   &cn
   this.oFolderButton = &c  //For use in new Disable/Enable methods
   with (&c)
     Group = true
     Text = ""
     OnClick = CLASS::GETFILEBUTTON_ONCLICK
     UpBitmap = "RESOURCE #136"
     Top = This.Top + .1
     Left = This.Left+This.Width-2.835 - .3
     Height = This.Height - .15
     Width = 2.835
     PageNo = This.PageNo
     TabStop = false
   endwith
   priv oButton
   oButton = &c
   oButton.oOwner = this  // Create a reference to the Entryfield that owns the new button.
   oButton.onFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-2.835-0.3, this.oOwner.Top+0.1) }
   oButton.cOwner = This.Name   && Name of editfield that owns this button
                    && This.cOwner is a string;
                    && &("Form."+This.cOwner) is an object.
   oButton.FileSpec = This.FileSpec
   oButton.DialogTitle = This.DialogTitle

   Proc Disable  // Disables the entryfield and its buttons
     This.Enabled = false
     This.oFolderButton.Enabled = false

   Proc Enable  // Disables the entryfield and its buttons
     This.Enabled = true
     This.oFolderButton.Enabled = true


   Procedure GETFILEBUTTON_OnClick
   PRIV c, cFile
   cFile = GetFile( this.FileSpec, this.DialogTitle )
   IF TYPE('This.cOwner') = 'C'            && IF the picker button was pressed...
     IF .NOT.EMPTY(cFile)
       c = "This.Parent." + This.cOwner + ".Value"
       &c = Trim(Proper(cFile))
     ENDIF
     c = "This.Parent." + This.cOwner + ".SetFocus()"
     &c
   ELSE                            && ELSE the user doubleClicked on the field....
     IF .NOT.EMPTY(cFile)
       This.Value = Trim(Proper(cFile))
     ENDIF
   ENDIF
   
ENDCLASS
***************************************************************************
CLASS FolderPicker(FormObj) OF zENTRYFIELD(FormObj) Custom
   *this.OnOpen         = CLASS::DefinePickerButton
   this.OnLeftDblClick = CLASS::GETDIRBUTTON_OnClick
   this.Left = 13.332
*  this.FontBold = .T.
   this.Top = 10.9395
   this.Width = 70
   this.Value = This.name

*  this.DialogTitle = 'Select a folder'   // GetDir() doesn't support this.

   this.DefaultFolder = ''
   * This property can be "customized" by setting a different value
   * to it in the Form_OnOpen routine of the form containing your FolderPicker.


   Procedure Init
   * Instead of the OnOpen event, which is slow cause it occurs AFTER the form is open.
   CLASS::DefinePickerButton()
   return


   Procedure DefinePickerButton
   PRIV cBtnName, c, cn
   cBtnName = 'Pkr'+TRIM(This.Name)
   IF TYPE('This.Parent.'+cBtnName) = 'O'
     RETU   && Object of this name is already defined
   ENDIF
   c = 'This.Parent.'+cBtnName
   PRIV cn
   cn = c+' = new PUSHBUTTON(This.Parent)'
   &cn
   this.oFolderButton = &c
   with (&c)
     Group = true
     Text = ""
     OnClick = CLASS::GETDIRBUTTON_ONCLICK
     UpBitmap = "RESOURCE #136"
     Top = This.Top + .1
     Left = This.Left+This.Width-2.835 - .3
     Height = This.Height - .15
     Width = 2.835
     PageNo = This.PageNo
     TabStop = false
   Endwith
   priv oButton
   oButton = &c
   oButton.oOwner = this  // Create a reference to the Entryfield that owns the new button.
   oButton.onFormResize = {; this.Move(this.oOwner.Left+This.oOwner.Width-2.835-0.3, this.oOwner.Top+0.1) }
   oButton.cOwner = This.Name   && Name of editfield that owns this button
                    && This.cOwner is a string;
                    && &("Form."+This.cOwner) is an object.
   oButton.DefaultFolder = This.DefaultFolder

   Proc Disable  // Disables the entryfield and its buttons
     This.Enabled = false
     This.oFolderButton.Enabled = false

   Proc Enable  // Disables the entryfield and its buttons
     This.Enabled = true
     This.oFolderButton.Enabled = true

   Procedure GETDIRBUTTON_OnClick
   PRIV c, cFolder
   cFolder = Proper(Trim(GetDIR( this.DefaultFolder )))
   IF TYPE('This.cOwner') = 'C'            && IF the picker button was pressed...
     IF .NOT.EMPTY(cFolder)
       c = "This.Parent." + This.cOwner + ".Value"
       &c = cFolder
     ENDIF
     c = "This.Parent." + This.cOwner + ".SetFocus()"
     &c
   ELSE                            && ELSE the user doubleClicked on the field....
     IF .NOT.EMPTY(cFolder)
       This.Value = cFolder
     ENDIF
   ENDIF
   
ENDCLASS
******************************************************************
FUNCTION TableBrowse(cAlias, cTitle, oEditSpec, cFields, cWfmName, cTag, PkrOnPickerWfmOpen)
* Simple table browser utilizing picker technology
If empty(cAlias)
  cAlias = GetFile(set('path')+'\*.DBF')
  If empty(cAlias)
    Return .t.
  Else
    cAlias = Left(cAlias,AT('.',cAlias)-1)
  Endif
Endif
LOCAL oSpecs
oSpecs = NEW Object()
oSpecs.PkrAlias = cAlias
oSpecs.PkrTitle = cTitle
oSpecs.PkrEditSpec = oEditSpec
oSpecs.PkrBrowseFields = cFields
oSpecs.PkrWfmName = cWfmName
oSpecs.PkrTag = cTag
oSpecs.PkrOnPickerWfmOpen = PkrOnPickerWfmOpen
DO Picker.WFM WITH oSpecs
RETU .T.
*********************************************************************
FUNCTION TB(cAlias,cTitle,oEditSpec)  // Shortcut for TableBrowse()
Return TableBrowse(cAlias,cTitle,oEditSpec)
*****************************************************************
Func PADR(cString,nLength,cPadChar)   && For clipper/Foxpro compatibility
return left( cString+REPL(IIF(PCOUNT()<3,' ',cPadChar),nLength), nLength )


