// Metric: Non Specific
/*
   FileName: formMouseOver.cc
   Author: Rich Muller - www.autotraker.com
	10/05/2003 - Original version
	11/10/2003 - Added the titleBar events and changed to using a paintbox
	11/11/2003 - Added the isOverControl function and isOverAControl function
   12/01/2003 - Added test to see if form.MDI property exists so we don't
   			  get an error when used in a subForm. Now calulating titleBarHeight
              instead of using a static height.

	This control adds 7 events to your form that call the below functions
   	form_mouseOver - which fires constantly while mouse is over form
      form_mouseIn - which fires once when mouse first moves over form
      form.mouseOut - which fires once when mouse first moves away from form

   	form_TitleBarOver - which fires constantly while mouse is over the titleBar
      form_TitleBarIn - which fires once when mouse first moves over the titleBar
      form.TitleBarOut - which fires once when mouse first moves away from the titleBar

		form_OverControl - which fires constantly returning the name of any control 
      	the mouse is over. When added to your form you also have to add a parameter.
         Such as function form_OverControl(cName)

	You have the option of adding any or all of the events to your form
	the CC does check to see if these functions exist and if so fires them.
   So you need not put all six functions in your form if you don't need
   them. See sample form for example.

	The isOverControl function will return a logical if your mouse is over a
   perticular control. 
   	? form.FORMMOUSEOVER1.isOverControl(form.PUSHBUTTON1)

	The isOverAControl function will return a control name if your mouse is 
   over any control. 
   	? form.FORMMOUSEOVER1.isOverAControl()

   The whereIsMouse function will set variables that indicate the X & Y 
   position of the mouse in pixels. 
   	form.FORMMOUSEOVER1.whereIsMouse()
      ? form.FORMMOUSEOVER1.X
      ? form.FORMMOUSEOVER1.Y

	The positionMouseOver function will position the mouse over the center of a control
   	form.FORMMOUSEOVER1.positionMouseOver(form.PUSHBUTTON1)

   The form's metric can be anything you want as we are using the API 
   to get positioning and size information.

   If you pass over the titleBar or control fast enough, the timer will not fire fast
   enough to fire the TitleBarIn, TitleBarOut and OverControl functions. So if you have
   need faster timing, lower the TIMER_UPDATE_SPEED define. 

   The original idea of the getLong function comes from Gary White or I may have
   snared it from Jim Sare.
   The original idea for the positionMouseOver function came from Romain Seriff.

	Example:
	Just drop the control on your form and add the functions or events you need to the form

   Caution: I didn't localize or protect anything in this control ;-)
   	so be careful that these functions, variables, or methods don't get
      overwritten or interfer with those that are already defined.

      The event form_OverControl(cName) tests if the mouse is over a control by
      flipping though the form objects. Depending on the Z order they are in, you
      may get some strange results. Placing controls in notebooks or containers is
      not working since I'm not recursing these controls. I'll change in in a future
      version when I have time.

*/

#Define TIMER_UPDATE_SPEED .1
#ifndef SM_CYCAPTION
	#define SM_CYCAPTION 4
#endif
#ifndef SM_CYSMCAPTION
	#define SM_CYSMCAPTION 51
#endif

class formMouseOver(parentObj, name) of PAINTBOX(parentObj, name) custom
   with (this)
      height = 0.25
      width = 0.75
		visible = false
   endwith
   this.version = 1.5

	****************************************
	function OnOpen
	****************************************
   // create a variables for the form
   this.smallTitle = false
   this.MDI = true
	if type("this.form.MDI")<>"U"
   	this.MDI = this.form.MDI
	endif
   if this.form.smallTitle and not this.MDI
      this.smallTitle = true
   endif
   if type("this.form.mouseOver")=="U"
      this.form.mouseOver=false
   endif
   if type("this.form.TitleBarOver")=="U"
		this.form.TitleBarOver=false
	endif
   //externs for API functions
   if type('GetCursorPos') # 'FP'
      extern clogical GetCursorPos(cptr) user32
   endif
   if type('ScreenToClient') # 'FP'
      extern clogical ScreenToClient(chandle, cptr) user32
   endif
   if type("SetCursorPos") # "FP"
      extern cLOGICAL SetCursorPos(cint, cint) user32
   endif
   if type("GetClientRect") # "FP"
      extern cLogical GetClientRect(cHandle, cptr) user32
   endif
   if type("GetWindowRect") # "FP"
      extern cLOGICAL GetWindowRect(cHandle, cptr) user32
   endif
   if type('GetSystemMetrics') # 'FP'
      extern CINT GetSystemMetrics(CINT) user32
   endif
   // default height of titlebar caption on form
   // it looks like this reports the proper height that
   // we need for determining if we are in the titlebar area.
	this.titleBarHeight = GetSystemMetrics(SM_CYCAPTION) 
   this.SmallTitleBarHeight = GetSystemMetrics(SM_CYSMCAPTION) 

   // get the mouse position for the first time so variables are set
   this.testMousePosition()
   // set up the timer
   this.setTimer()
   return

	****************************************
   function testMousePosition
	****************************************
      // check where the mouse is in the form and
      // see if it is over the form or not.

      if type("this.form")=="U"
      	// form already closed, in case timer 
         // was still functioning 
         return
      endif

      // update X & Y
		class::whereIsMouse()

      // get width and height of the form
      cRect = new string(space(8))
      GetClientRect(this.form.hwnd, cRect)  
      nPixWidth  = int(this.getLong(8,cRect))
      nPixHeight = int(this.getLong(12,cRect))

      // test mouse position in relationship to the form
      if this.X>0 and this.Y>0 and ;
         this.Y<nPixHeight and ;
         this.X<nPixWidth
         // mouse is over form
         class::form_mouseOver()
         if not this.form.mouseover
            class::form_mouseIn()
            this.form.mouseover=true
         endif
      else
      	// mouse is not over form
         if this.form.mouseover
            class::form_mouseOut()
            this.form.mouseover=false
         endif
      endif

      // test mouse position in relationship to the titlebar
      if this.smallTitle
      	titleBarHeight = - this.SmallTitleBarHeight
		else
      	titleBarHeight = - this.titleBarHeight
		endif
      if this.X>0 and this.Y<0 and ;
      	this.X<nPixWidth and ;
         this.Y>titleBarHeight
         // mouse is over titlebar
			class::form_TitleBarOver()
         if not this.form.TitleBarOver
            class::form_TitleBarIn()
            this.form.TitleBarOver=true
         endif
      else
         // mouse is not over titlebar
         if this.form.TitleBarOver
            class::form_TitleBarOut()
            this.form.TitleBarOver=false
         endif
      endif

      cName = class::isOverAControl()
      class::form_OverControl(cName)

		return

	****************************************
	function whereIsMouse
	****************************************
      // find the position of the mouse
      cRect = new string(space(4))
      GetCursorPos(cRect)
      ScreenToClient(this.form.hwndClient,cRect)
      this.X = int(this.getLong(0,cRect))
      this.Y = int(this.getLong(4,cRect))
      //? "X:"+this.X+"  Y:"+this.Y
		return

	****************************************
   function isOverAControl
	****************************************
		// check all the controls in this form and return
      // the name of the control that the mouse is over
      private oRef
      cName = ""
      oRef  =  this.form.first
      do
         if class::isOverControl(oRef)
         	cName = oRef.name 
         endif
         oRef := oRef.before
      until oRef == this.form.first
      oRef  := null
      return cName

	****************************************
   function isOverControl
	****************************************
      parameters oCntl
      local isOver
      if type("oCntl")<>"O"
      	return false
      endif

      // get position of mouse
      cRect = new string(space(4))
      GetCursorPos(cRect)
      ScreenToClient(oCntl.hwnd,cRect)
      X = int(this.getLong(0,cRect))
      Y = int(this.getLong(4,cRect))

      // get width and height of the control
      cRect = new string(space(8))
      GetClientRect(oCntl.hwnd,cRect)  
      nPixWidth  = int(this.getLong(8,cRect))
      nPixHeight = int(this.getLong(12,cRect))

      // test mouse position in relationship to the control
      if X > 0 and X < nPixWidth and ;
      	Y > 0  and Y < nPixHeight
         //? "mouse is over control"
	      isOver = true
      else
         //? "mouse is not over control"
         isOver = false
      endif
	   return isOver

	****************************************
   function setTimer
	****************************************
   	// set up the timer object for checking mouse position
      this.timer = new timer()
      this.timer.parent = this
      this.timer.onTimer = CLASS::CheckMousePosition
      this.timer.interval = TIMER_UPDATE_SPEED
      this.timer.enabled  = true
      return

	****************************************
   function onClose
	****************************************
   	// disable the timer and other variables
      try
         if type("this.timer")<>"U"
            this.timer.enabled = false
         endif
		catch(exception e)
      endtry
      return

	****************************************
   function CheckMousePosition
	****************************************
   	// timer event calling the controls check mouse position
      this.parent.testMousePosition()
      return

	****************************************
   function form_mouseOver
	****************************************
      // fires constantly while mouse is over form
      // stops firing when mouse moves from over form
      if (type("this.form.form_mouseOver") $ "CB|FP") and (not empty(this.form.form_mouseOver))
         this.form.form_mouseOver()
      else
	      //? "CC MouseOver Fired " + time(0)
      endif
      return

	****************************************
   function form_mouseIn
	****************************************
      // fires once when mouse first moves over form
      if (type("this.form.form_mouseIn") $ "CB|FP") and (not empty(this.form.form_mouseIn))
         this.form.form_mouseIn()
   	else
	      //? "CC MouseIn Fired " + time(0)
      endif
      return

	****************************************
   function form_mouseOut
	****************************************
      // fires once when mouse first moves away from form
      if (type("this.form.form_mouseOut") $ "CB|FP") and (not empty(this.form.form_mouseOut))
         this.form.form_mouseOut()
		else
	      //? "CC MouseOut Fired " + time(0)
      endif
      return

	****************************************
   function form_TitleBarOver
	****************************************
      // fires constantly while mouse is over titleBar
      // stops firing when mouse moves from over titleBar
      if (type("this.form.form_TitleBarOver") $ "CB|FP") and (not empty(this.form.form_TitleBarOver))
         this.form.form_TitleBarOver()
      else
	      //? "CC TitleBarOver Fired " + time(0)
      endif
      return

	****************************************
   function form_TitleBarIn
	****************************************
      // fires once when mouse first moves over titleBar
      if (type("this.form.form_TitleBarIn") $ "CB|FP") and (not empty(this.form.form_TitleBarIn))
         this.form.form_TitleBarIn()
   	else
	      //? "CC TitleBarIn Fired " + time(0)
      endif
      return

	****************************************
   function form_TitleBarOut
	****************************************
      // fires once when mouse first moves away from titleBar
      if (type("this.form.form_TitleBarOut") $ "CB|FP") and (not empty(this.form.form_TitleBarOut))
         this.form.form_TitleBarOut()
		else
	      //? "CC TitleBarOut Fired " + time(0)
      endif
      return

	****************************************
   function form_OverControl
	****************************************
   	parameters cName
      // fires once when mouse first moves away from titleBar
      if (type("this.form.form_OverControl") $ "CB|FP") and (not empty(this.form.form_OverControl))
         this.form.form_OverControl(cName)
		else
	      //? "CC OverControl "+cName+" Fired " + time(0)
      endif
      return cName

	****************************************
   function getLong(nIndex,cStr)
	****************************************
      local n
      n = cStr.getByte(nIndex) + ;
         bitlshift(cStr.getByte(nIndex + 1),  8) + ;
         bitlshift(cStr.getByte(nIndex + 2), 16) + ;
         bitlshift(cStr.getByte(nIndex + 3), 24)
      if bitset(n, 31)
         n := - bitnot(n)
      endif
      return n

	****************************************
   function positionMouseOver
	****************************************
   	parameters oCtnl
      local cRect,nPixWidth,nPixHeight,nCol,nRow
      cRect=space(8)
      // get width and height of the control
      GetClientRect(oCtnl.hwnd, cRect)  
      nPixWidth  = int(this.getLong(8,cRect))
      nPixHeight = int(this.getLong(12,cRect))
      // Get the position onscreen of the control
      GetWindowRect(oCtnl.hwnd,cRect)
		nCol = int(this.getLong(0,cRect))
		nRow = int(this.getLong(4,cRect))

		// set the mouse position over the center of the control
	   SetCursorPos(ncol+(nPixWidth/2),;
                nrow+(nPixHeight/2))   
      return



endclass




