/******************************************************************************

	Hash Class -- Calculate a hash value from a given input string
	
	Programmer 	: Marc Van den Berghen
	
	Version		: 	0.0 (24-01-2021) 	Question in NG (Alex Safian)
												Based on the Windows CNG-API
												(Cryptography Next Generation)
						0.1 (24-01-2021)  First release, to be tested by NG
						1.0 (25-01-2021)	code simplified, added gethashx
						2.0 (26-01-2021)	thanks Ronnie! Added cmpr method.
						2.1 (28-01-2021)  Random salt with random length.
												Added time independant string comparison
												based on info provided by Alex Safian
												use of the Cryptographically Secure 
												Pseudo-Random Number Generator (CSPRNG) of
												windows
					  
	Usage: 	** instantiate a hash object
	
				hobject = new hash()
				
				** test if all went well, if not, don't use the hashobject
				
				if not hobject.initok
					** Errorhandling (Msgbox,log...)
					hobject.release()
				endif
				
				** Get a hashvalue by passing a string to the gethash() method.
				** If you pass a second string, it will be used as salt
				** this method can be used to get a general hashvalue, but if you 
				** want to hash passwords, the Gethashx method is prefered, because
				** it will automatically use a random salt of random length 
				
				hashvalue = hobject.gethash("Newsgroup")  // general hashvalue 
				hashvalue1 = hobject.gethash("Newsgroup","X")  // salt = "X"
				hx = hobject.gethashx("Test")  // 64 byte result with random salt
				
				** if you want to check if a password matches with a hash, use cmpr.
				** to be used only on the 64 bytes of gethashx
				
				b_result = hobject.cmpr(c_in,c_hash)  // true if match
				
				** if you don't need the object anylonger, release it !!!
				
				hobj.release()
				
	IMPORTANT !!! IMPORTANT !!! IMPORTANT !!! IMPORTANT !!! HOPE YOU GOT IT
	***********************************************************************
	
	Be aware of a kind of meta vulnerability! The hash.cc is open source,
	published here and in the duflp. If you use it as is, to someone who is 
	aware of this code and who guesses, that you hashed your passwords with it,
	the hashing falls back to a simple SHA256 without salt! So i strongly 
	recommend, that you do one more simple transformation with the output of 
	the class, before you store it in a table (something like alternating every
	2 characters, adding a fixed value, call gethash n times with it or 
	something similar easily done within dbase) 
	This way, even if someone hacks your table and knows this sourcecode
	of hash.cc, he can not use a brute force attack to get to the passwords. 
	(Unless of course he hacked and got a hand on your source code too...)
	
	WARNING : THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
				 MAKE YOURSELF FAMILIAR WITH SECURITY CONSIDERATIONS BEFORE
				 USING THIS CODE IN VULNERABLE ENVIRONMENTS (WEB)	
	
******************************************************************************/

CLASS Hash

	protect alg_handle,init,StringCmpr

	this.e 				= new hashexception()	// genuine error handling
	this.alg_handle	= 0							// must be released at the end	
	this.initok			= false						// true if initialisation succeeded
	this.init()
	
	***************************************************************************
	function GetHash(c_text, csalt)	// returns the hash of c_text using csalt
	***************************************************************************	
		local salt,c_out,c_in,buffer,buffer1,hhash,n
		this.e.clear()                         			// clear pending errors
		try
			if not this.initok  ; this.e.check(1) ; endif
			if empty(csalt) ; csalt = "" ; endif

			c_in  = c_text+csalt								
		
			** prepare two buffer needed for the hash object and the hash value
			hhash   = 0 
			buffer  = replicate(chr(0),150)
			buffer1 = replicate(chr(0),16)
			
			** Now let the OS do all the hard work :-)
			this.e.check(BCryptCreateHash(this.alg_handle,hhash,buffer,294,0,0,0))
			this.e.check(BCryptHashData(hhash,c_in,len(c_in)*2,0))
			this.e.check(BCryptFinishHash(hhash,buffer1,32,0))
			
			** and get the hashvalue into a dbase string
			c_out = ""
			for n = 0 to 31
				c_out += itoh(buffer1.getbyte(n),2)
			endfor
		
		catch(exception e)
			this.e.message = "Hash not successfull"
			this.e.format(e,"Don't use the hashvalue")
			c_out = ""
		finally
			** in any case release the used resources
			if hhash <> 0 ; BCryptDestroyHash(hhash) ; endif
		endtry
		
	return c_out
	
	***************************************************************************
	function GetHashx(c_in)		// stronger version of the gethash method
	***************************************************************************
		
		** create a random length for the salt ( between 16 and 30 )
		** create a random salt of this length.
		** hide the random salt at a random position in a random 32 byte block
		** hash the password (c_in parameter) with the random salt.
		** return a 64 byte block (128 hex digits) , where the parts of the salt
		** block and hash block alternate.
		
		try 
			local rndbuffer, length, pos, salt, base, c_out, i, tmp
			c_out  = ""
			rndbuffer = " "
			this.e.check(BCryptGenRandom(0,rndbuffer,1,2))
			pos = rndbuffer.getbyte(0)%30 + 1					// random position
			this.e.check(BCryptGenRandom(0,rndbuffer,1,2))	// random length 
			length = rndbuffer.getbyte(0)%14 + 16				// make it 16-30
			rndbuffer = replicate(chr(0),length)
			this.e.check(BCryptGenRandom(0,rndbuffer,length,2))	
			salt = ""													// get the random salt
			for i = 0 to length-1
				salt += right(itoh(rndbuffer.getbyte(i),2),1)
			endfor
			base = this.gethash(salt)								// get 32 byte block
			if empty(base) ; this.e.check(1); endif			// throw error
			base = base.stuff(0,1,itoh(length-15))				// save length
			base = base.stuff(62,2,itoh(pos,2))					// save position
			base = base.stuff(pos,length,salt)					// hide salt
			tmp = this.gethash(c_in,salt)
			if empty(tmp) ; this.e.check(1) ; endif			// throw error
			for i = 1 to 64
				c_out += substr(tmp,i,1)+substr(base,i,1)		// alternate hash-salt  
			endfor
			
		catch(exception e)
			** if we get here, something went wrong and c_out is not valid
			this.e.message = "Something went wrong during hashing"
			this.e.format(e, "Don't use the result")
			c_out = ""
		endtry

	return c_out														
	
	****************************************************************************
	function Cmpr(c_in,c_hash)			// returns true on match
	****************************************************************************
	
		** this function must be used to compare the stored hashvalue with 
		** the actually entered password. c_hash must be 128 digits long
		** returns true if match
		try
			local base, salt, hash, pos, length ,c_out, match, i
			if len(c_hash) <> 128 ; return false ; endif
			base = "" ; hash = "" ; salt = ""
			for i = 0 to 63
				hash += substr(c_hash,i*2 + 1,1)			//extract hash and salt
				base += substr(c_hash,i*2 + 2,1)
			endfor
			length = htoi(substr(base,1,1))+15
			pos    = htoi(right(base,2))
			salt 	 = substr(base,pos+1,length)			// get salt out of base
			c_out  = this.GetHash(c_in,salt)				// gethash of c_in with salt
		catch(exception e)
			this.e.message = "Error during comparison"
			return false
		endtry
	return this.stringcmpr(hash,c_out)					// true on match
	
	****************************************************************************
	function StringCmpr(c1,c2)			// time independant string compare
	****************************************************************************
		** according to information provided by Alex Safian, there are attacks
		** that focus on the time used for string comparison. Because i don't
		** know the internal way, dbase does this, i want to be on the safe side 
		** and do the comparison time independant (that means it doesn't matter
		** if a difference is found at the first or at any other position in the 
		** strings)
	
		local tmp,i
		tmp = true
		for i = 1 to 64
			** check all 64 digits, even if the first don't match already
			tmp = tmp and (substr(c1,i,1) == substr(c2,i,1))
		endfor
	return tmp
	
	****************************************************************************
	function Release   // cleanup
	****************************************************************************
		If this.alg_handle <> 0
			BCryptCloseAlgorithmProvider(this.alg_handle,0)
		endif
	return
	
	****************************************************************************
	function Init		// open the algorithm provider
	****************************************************************************
		try
			** Externalize the needed APIs and get the algorithm handler
			** the externs are put into the try catch block, so if the class is
			** called from a OS that doesn't support them, we get warned
		
			extern clong BCryptOpenAlgorithmProvider(cptr culong,cptr,cptr,culong) Bcrypt.dll
			extern clong BCryptCloseAlgorithmProvider(chandle,culong) Bcrypt.dll
			extern clong BCryptCreateHash(chandle,cptr culong,cptr,culong,culong,culong,culong) Bcrypt
			extern clong BCryptDestroyHash(chandle) Bcrypt
			extern clong BCryptHashData(chandle,cptr,culong,culong) Bcrypt
			extern clong BCryptFinishHash(chandle,cptr,culong,culong) Bcrypt
			extern culong GetTickCount(cvoid) kernel32
			extern clong BCryptGenRandom(chandle,cptr,culong,culong) Bcrypt
			
			this.e.check(BCryptOpenAlgorithmProvider(this.alg_handle,"SHA256",null, 0x20))
			this.initok = true
		
		catch(exception e)
			this.e.message = "The hash object can not be used"
			this.e.format( e, "Error initialising")
			this.initok		= false
			this.alg_handle= 0
		endtry
Endclass

/******************************************************************************
Exception Handler
******************************************************************************/

Class HashException of Exception
	this.clear()
	
	function format(exc,ctext)
		if this.code = 0					// be aware of standard errors
			this.code = exc.code
			this.message = exc.message
		endif
		this.message = ctext + " : " + this.message
	return
	
	function check(nstatus)		// can be used to test result of apis
		if nstatus <> 0
			this.code = nstatus
			throw this		// there is an error
		endif
		this.clear()
	return true	// no error occured
	
	function clear()
		this.code 		= 0									// Success
		this.message	= "Success"
	return
	
EndClass

