/*
   -----------------------------------------------------------------------------
    Description:
 
    Programmers have always had a problem of taking periodic backups
    Especially with RDBMS like MariaDB & Firebird where copying of files does 
    not create a backup
    Both these RDBMS have a utility to dump the data to desired files.
    This program uses these utilities to create the backup files. 
    These files are named using the following convention
    year_2018.sql will be the backup file name when the backup is run for the 
         first time in 2018
    month_2018_04.sql will contain backup created at the start of April 2018
    weekly_2018_05.sql will be backup created at the start of 5th week of 2018. 
         eweekno(date()) of dateex.cc in duflp is used to get the current week number
    daily_20181231.sql will be the backup at the start of 31st:December 
         2018 dtos() is used to convert current date to string
    hourly_20181231_0814.sql will be hourly backup created at 08:14 hrs on 
         31st december 2018

    After setting the required parameters build an exe and run it using 
    windows scheduler at regular intervals to create files named in the 
    above pattern.
    
    After creating the files this program will check for number of versions 
    of the backup already on disk and delete the older ones according to 
    the option selected
    
    Programmers: 
 
       Akshat Kapoor 
       (With a lot of Help and suggestions from Mervyn Bick, He provided me 
       with the basic program which I expanded to keep a control on number 
       of versions to be retained)
 
    History: 
 
		17th May 2018 Sharing the ready to deploy code for the first time.

    Dependencies
    
      This program uses miscapi.prg and dateex.cc from duflp. 
      They should be available while compiling the exe
      
    Instantiation/use: 
    
      Set the parameters in the parameter section and compile into an exe 
      (Please select the no BDE option as no tables are being used hence 
      BDE is not required)
      Run that exe using a windows scheduler. 
      
      (Using a timer to execute can also be an option but that will 
      require your app to run continuously)

		 for Any further suggestions please email me  akshatkapoor76 at gmail dot com
   -----------------------------------------------------------------------------
*/
set procedure to miscapi.prg
set procedure to dateex.cc

//Uncomment and modify paths in the following lines for MySQL (MariaDB) provided by XAMPP
//backup_dir = "c:\dbasetutorial\muneemado\"
//backup_cmd = "c:\xampp\mysql\bin\mysqldump -u root --all-databases>"
//backup_ext = ".sql"

//Uncomment and modify paths in the following lines for Firebird
backup_cmd = ["c:\program Files (x86)\firebird\firebird_2_5\bin\gbak.exe" -b -user SYSDBA -passw masterkey d:\firebird_databases\employee.fdb ]
backup_dir = [d:\firebird_backups\]
backup_ext = [.fbk]

//Default directories are used above. Please check the directories on your computer
//And change the passwords etc.
number_mon_back = 60 //Backups older than date()- this.value*30 days will be deleted 
number_wee_back = 50 //Backups older than date()- this.value*7 days will be deleted
number_dai_back = 40 //Backups older than date()- this.value days will be deleted
number_hou_back = 30 //Backups older than date() - this.value days will be deleted

check_prev()
generate_bat() // this function generates the batch file named backup.bat which in turn generates the desired backup files.
cbackup = "backup.bat"
runhidden("&cBackup")
delete_old_backups()
//delete_old_backups2()
/*
Only use one of the above functions never both.
delete_old_backups() will maintain the number of backup files specified above. 
It will not delete backup files without maintaining a certain number backup files

delete_old_backups2() will delete files older than their expiry dates.
Calculations are shown above. 
Files will be deleted regardless of there are other backup files are there or not
*/
Return

//No settings to be customised below this line

Function generate_bat
#define ARRAY_DIR_NAME 1 // Manifest constants for columns returned by dir( )
#define ARRAY_DIR_SIZE 2
#define ARRAY_DIR_DATE 3
#define ARRAY_DIR_TIME 4
#define ARRAY_DIR_ATTR 5
#define OUT_FILE "BACKUP.BAT"
fOut = new File()
fOut.create(OUT_FILE)

aFiles = new Array() // Array will be resized as needed
nFiles = aFiles.dir(backup_dir+"year_"+year(date())+backup_ext)
if nFILES = 0
	cBackup = backup_cmd + backup_dir + "year_"+year(date())+backup_ext
   fout.writeln(cBackup)
endif

aFiles = new Array()
nFiles = aFiles.dir(backup_dir+"month_"+ year(date()) +"_"+ str(month(date()),2,0,"0") +backup_ext)
if nFiles = 0
	cBackup = backup_cmd + backup_dir + "month_" +year(date())+ "_"+str(month(date()),2,0,"0")+backup_ext
   fout.writeln(cBackup)
endif
	
//Check for Weekly Backup
aFiles = new Array() 
nFiles = aFiles.dir(backup_dir+"weekly_"+year(date())+"_"+str(dateex::eweekno(date()),2,0,"0")+backup_ext)
if nFiles = 0
	cBackup = backup_cmd + backup_dir + "weekly_" +year(date())+"_"+ str(dateex::eweekno(date()),2,0,"0")+backup_ext
   fout.writeln(cBackup)
endif

//	check for daily Backup
aFiles = new Array() 
nFiles = aFiles.dir(backup_dir+"daily_"+dtos(date())+backup_ext)
if nFiles = 0
	cBackup = backup_cmd + backup_dir + "daily_"+dtos(date())+backup_ext
   fout.writeln(cBackup)
endif

cBackup = backup_cmd + backup_dir + "hourly_"+dtos(date())+"_"+substr(time(),1,2)+substr(time(),4,2)+backup_ext
fout.writeln(cBackup)
fout.close()
return

Function delete_old_backups
//This version maintains the specified number of backup files and deletes the older ones on an FIFO basis
//number_mon_back = 60 
//number_wee_back = 50 
//number_dai_back = 40 
//number_hou_back = 30 
   //delete monthly backup files 
   aFiles = new Array() // Array will be resized as needed
   nFiles = aFiles.dir(backup_dir+"month_*"+backup_ext)
   
   if nFiles > number_mon_back
      for temp = 1 to (nFILES - number_mon_back)
         delete file &aFiles[ temp, ARRAY_DIR_NAME ]
      endfor
   endif
   
   //delete weekly backup files
   aFiles = new Array()
   nFiles = aFiles.dir(backup_dir+"weekly_"+backup_ext)
   if nFiles > number_wee_back
      ?"to check and delete older files"
      for temp = 1 to (nFILES - number_wee_back)
         delete file &aFiles[ temp, ARRAY_DIR_NAME ]
      endfor
   endif	

   //Delete Daily backup files
   aFiles = new Array() 
   nFiles = aFiles.dir(backup_dir+"daily_*"+backup_ext)
   if nFiles > number_dai_back
      ?"to check and delete older files"
      for temp = 1 to (nFILES - number_dai_back)
         delete file &aFiles[ temp, ARRAY_DIR_NAME ]
      endfor
   endif	

	//Delete Hourly backup files
   aFiles = new Array() 
	nFiles = aFiles.dir(backup_dir+"hourly_*"+backup_ext)
	if nFiles > number_hou_back
		?"to check and delete older files"
		for temp = 1 to (nFILES - number_hou_back)
			delete file &aFiles[ temp, ARRAY_DIR_NAME ]
		endfor
	endif	

return

Function delete_old_backups2
//This version of the function uses date of the backup files so generated and deletes files older than specified date
// Going to check for old monthly backup files
   aFiles = new Array() // Array will be resized as needed
   nFiles = aFiles.dir(backup_dir+"month_*"+backup_ext)
   temp = true
   temp2 = 1
   do while temp
      if nFILES = 0
         temp = false
      else
         curr_file_d = aFiles[ temp2, ARRAY_DIR_date ]
         if curr_file_d < date() - number_mon_back*30 AND temp2 < nFiles
            delete file &aFiles[ temp2, ARRAY_DIR_NAME ]
            temp2++
         else
            temp = false
         endif
      endif
   enddo

// Going to check for Old Weekly Backup files
   aFiles = new Array()
   nFiles = aFiles.dir(backup_dir+"weekly_*"+backup_ext)
   temp = true
   temp2 = 1
   do while temp
      if nFILES = 0
         temp = false
      else
         curr_file_d = aFiles[ temp2, ARRAY_DIR_date ]
         if curr_file_d < date() - number_wee_back*7 AND temp2 < nFiles
            delete file &aFiles[ temp2, ARRAY_DIR_NAME ]
            temp2++
         else
            temp = false
         endif
      endif
   enddo

//Going to check for old daily Backup files
   aFiles = new Array()
   nFiles = aFiles.dir(backup_dir+"daily_*"+backup_ext)
   temp = true
   temp2 = 1
   do while temp
      if nFILES = 0
         temp = false
      else
         curr_file_d = aFiles[ temp2, ARRAY_DIR_date ]
         if curr_file_d < date() - number_dai_back AND temp2 < nFiles
            delete file &aFiles[ temp2, ARRAY_DIR_NAME ]
            temp2++
         else
            temp = false
         endif
      endif
   enddo

// Going to check for old hourly backup files
	aFiles = new Array() 
	nFiles = aFiles.dir(backup_dir+"hourly_*"+backup_ext)
   local temp
   temp = true
   temp2 = 1
   do while temp
      if nFILES = 0
         temp = false
      else
         curr_file_d = aFiles[ temp2, ARRAY_DIR_date ] 
         if curr_file_d < date() - number_hou_back AND temp2 < nFiles
            delete file &aFiles[ temp2, ARRAY_DIR_NAME ]
            temp2++
         else
            temp = false
         endif
      endif
   enddo
Return

function check_prev
#define IN_FILE "backup.bat"
ferr = new file()
if ferr.exists(backup_dir+"error.log")
   ferr.open(backup_dir+"error.log","A")
else
   ferr.create(backup_dir+"error.log","A")
endif
merror = false
fIn = new File()
if fIn.exists(IN_FILE)
   fIn.open(IN_FILE)
   do while not fIn.eof()
      cmd = fIn.readln()
      if "year" $ cmd
         file_name = right(cmd,13)
         aFiles = new Array() // Array will be resized as needed
         nFiles = aFiles.dir(backup_dir+file_name)
         if nFILES = 0 or afiles[1,ARRAY_DIR_SIZE] <100
            msgbox("Error in yearly backup file generation ")
            ferr.writeln(dtoc(date())+" "+ time() + " Error in yearly backup file generation file named "+file_name)
            merror = true
         endif
      endif
      if "month" $ cmd
         file_name = right(cmd,17)
         aFiles = new Array() // Array will be resized as needed
         nFiles = aFiles.dir(backup_dir+file_name)
         if nFILES = 0 or afiles[1,ARRAY_DIR_SIZE] <100
            msgbox("Error in Monthly backup file generation ")
            ferr.writeln(dtoc(date())+" "+ time() + " Error in monthly backup file generation file named "+file_name)
            merror = true
         endif
      endif
      if "week" $ cmd
         file_name = right(cmd,18)
         aFiles = new Array() // Array will be resized as needed
         nFiles = aFiles.dir(backup_dir+file_name)
         if nFILES = 0 or afiles[1,ARRAY_DIR_SIZE] <100
            msgbox("Error in Weekly backup file generation ")
            ferr.writeln(dtoc(date())+" "+ time() + " Error in weekly backup file generation file named "+file_name)
            merror = true
         endif
      endif
      if "daily" $ cmd
         file_name = right(cmd,18)
         aFiles = new Array() // Array will be resized as needed
         nFiles = aFiles.dir(backup_dir+file_name)
         if nFILES = 0 or afiles[1,ARRAY_DIR_SIZE] <100
            msgbox("Error in Daily backup file generation ")
            ferr.writeln(dtoc(date())+" "+ time() + " Error in daily backup file generation file named "+file_name)
            merror = true
         endif
      endif
      if "hour" $ cmd
         file_name = right(cmd,24)
         aFiles = new Array() // Array will be resized as needed
         nFiles = aFiles.dir(backup_dir+file_name)
         if nFILES = 0 or afiles[1,ARRAY_DIR_SIZE] <100
            msgbox("Error in Hourly backup file generation ")
            ferr.writeln(dtoc(date())+" "+ time() + " Error in hourly backup file generation file named "+file_name)
            merror = true
         endif
      endif   
      
   enddo
   if merror 
      //do nothing error would already have been logged by now
   else
      ferr.writeln(dtoc(date())+" "+ time() + " Last Backup creation at "+fin.createdate(IN_FILE)+" " + fin.createtime(IN_FILE) +" was completed successfully")
   endif   
   fIn.close()
endif
ferr.close()
Return