/* -----------------------------------------------------------
   toggle.cc
   Author: Mervyn Bick, Maurizio Salzano and Tom Stancel
   Date.: 2018-10-15
   
   This custom control is based on contributions in the dBASE 
   newsgroup from Maurizio Salzano and Tom Stancel.
   
   My contribution was to take these ideas and turn them into
   a custom control.
   
   Left-clicking on a control toggles the display and the contents
   of the control's value property.

   The control expects to find a function named TOGGLE_onChange in the 
   parent form to which it can return its name and the contents of its 
   value property.

      function TOGGLE_onChange(cName,lValue)
         //place code here to deal with change of value
         //?cName,lValue
         return
         
   The control defaults to "Off" when it is placed.  To have a control open
   in the "On" position it is necessary to overwrite the toggle_onOpen event
   handler as shown below.

      this.TOGGLE2 = new TOGGLE(this)
      with (this.TOGGLE2)
         onOpen = class::TOGGLE2_ONOPEN // add this line
         height = 1.0
         left = 9.0
         top = 7.5
         width = 7.5
      endwith

      function TOGGLE2_onOpen()
         toggle::toggle_onOpen() // call the original event handler
         this.value = true
         this.datasource = 'filename toggleon.png'
         return
   
   Dependancy:  :duflp:base64bin.cc
*/

class TOGGLE(parentObj) of image(parentObj) custom
   with (this)
      onOpen =  class::toggle_onOpen
      onLeftMouseUp = class::toggle_onLeftMouseUp
      onDesignOpen = class::toggle_onDesignOpen
      height = 1.0
      width = 7.5
      borderStyle = 3  //None
      transparent := true
      alignment = 3	// Keep Aspect Stretch
      borderStyle = 3	// None  
   endwith

   function toggle_onLeftMouseUp
      if this.dataSource = "FILENAME toggleoff.png"
         this.dataSource = "FILENAME toggleon.png"
         this.value = true
      else
         this.dataSource = "FILENAME toggleoff.png"
         this.value = false
      endif      
      if type('this.parent.toggle_onChange') <> 'FP'
          class::toggle_onChange()
      else 
         this.parent.toggle_onChange(this.name,this.value)
      endif         
      return

   function toggle_onChange
      //Create a like-named function in form with required actions.
      //See note in header.
      msgbox('Create function toggle_onChange in form to handle change of value.')
   return
      
   function toggle_onOpen   
      if not file('toggleoff.png') 
         set procedure to :duflp:base64bin.cc
         oRef = new base64bin()
         a = class::image_off()
         oRef.decodebase64File(a,'toggleoff.png') 
         release object oRef
         close procedure :duflp:base64bin.cc 
      endif 
      if not file('toggleon.png')
         set procedure to :duflp:base64bin.cc
         oRef1 = new base64bin()
         b = class::image_on()
         oRef1.decodebase64File(b,'toggleon.png')
         release object oRef1         
         close procedure :duflp:base64bin.cc 
      endif 
      this.dataSource = "FILENAME toggleoff.png"
      return
      
   function toggle_onDesignOpen   
      this.borderStyle = 4   //Single line
   return
      
   function image_off   
      a = ''
      a += 'iVBORw0KGgoAAAANSUhEUgAAAO0AAABtCAYAAABN/eZXAAAACXBIWXMAAA7DAAAOwwHHb6hk'
      a += 'AAAKT2lDQ1BQaG90b3Nob3AgSUNDIHByb2ZpbGUAAHjanVNnVFPpFj333vRCS4iAlEtvUhUI'
      a += 'IFJCi4AUkSYqIQkQSoghodkVUcERRUUEG8igiAOOjoCMFVEsDIoK2AfkIaKOg6OIisr74Xuj'
      a += 'a9a89+bN/rXXPues852zzwfACAyWSDNRNYAMqUIeEeCDx8TG4eQuQIEKJHAAEAizZCFz/SMB'
      a += 'APh+PDwrIsAHvgABeNMLCADATZvAMByH/w/qQplcAYCEAcB0kThLCIAUAEB6jkKmAEBGAYCd'
      a += 'mCZTAKAEAGDLY2LjAFAtAGAnf+bTAICd+Jl7AQBblCEVAaCRACATZYhEAGg7AKzPVopFAFgw'
      a += 'ABRmS8Q5ANgtADBJV2ZIALC3AMDOEAuyAAgMADBRiIUpAAR7AGDIIyN4AISZABRG8lc88Suu'
      a += 'EOcqAAB4mbI8uSQ5RYFbCC1xB1dXLh4ozkkXKxQ2YQJhmkAuwnmZGTKBNA/g88wAAKCRFRHg'
      a += 'g/P9eM4Ors7ONo62Dl8t6r8G/yJiYuP+5c+rcEAAAOF0ftH+LC+zGoA7BoBt/qIl7gRoXgug'
      a += 'dfeLZrIPQLUAoOnaV/Nw+H48PEWhkLnZ2eXk5NhKxEJbYcpXff5nwl/AV/1s+X48/Pf14L7i'
      a += 'JIEyXYFHBPjgwsz0TKUcz5IJhGLc5o9H/LcL//wd0yLESWK5WCoU41EScY5EmozzMqUiiUKS'
      a += 'KcUl0v9k4t8s+wM+3zUAsGo+AXuRLahdYwP2SycQWHTA4vcAAPK7b8HUKAgDgGiD4c93/+8/'
      a += '/UegJQCAZkmScQAAXkQkLlTKsz/HCAAARKCBKrBBG/TBGCzABhzBBdzBC/xgNoRCJMTCQhBC'
      a += 'CmSAHHJgKayCQiiGzbAdKmAv1EAdNMBRaIaTcA4uwlW4Dj1wD/phCJ7BKLyBCQRByAgTYSHa'
      a += 'iAFiilgjjggXmYX4IcFIBBKLJCDJiBRRIkuRNUgxUopUIFVIHfI9cgI5h1xGupE7yAAygvyG'
      a += 'vEcxlIGyUT3UDLVDuag3GoRGogvQZHQxmo8WoJvQcrQaPYw2oefQq2gP2o8+Q8cwwOgYBzPE'
      a += 'bDAuxsNCsTgsCZNjy7EirAyrxhqwVqwDu4n1Y8+xdwQSgUXACTYEd0IgYR5BSFhMWE7YSKgg'
      a += 'HCQ0EdoJNwkDhFHCJyKTqEu0JroR+cQYYjIxh1hILCPWEo8TLxB7iEPENyQSiUMyJ7mQAkmx'
      a += 'pFTSEtJG0m5SI+ksqZs0SBojk8naZGuyBzmULCAryIXkneTD5DPkG+Qh8lsKnWJAcaT4U+Io'
      a += 'UspqShnlEOU05QZlmDJBVaOaUt2ooVQRNY9aQq2htlKvUYeoEzR1mjnNgxZJS6WtopXTGmgX'
      a += 'aPdpr+h0uhHdlR5Ol9BX0svpR+iX6AP0dwwNhhWDx4hnKBmbGAcYZxl3GK+YTKYZ04sZx1Qw'
      a += 'NzHrmOeZD5lvVVgqtip8FZHKCpVKlSaVGyovVKmqpqreqgtV81XLVI+pXlN9rkZVM1PjqQnU'
      a += 'lqtVqp1Q61MbU2epO6iHqmeob1Q/pH5Z/YkGWcNMw09DpFGgsV/jvMYgC2MZs3gsIWsNq4Z1'
      a += 'gTXEJrHN2Xx2KruY/R27iz2qqaE5QzNKM1ezUvOUZj8H45hx+Jx0TgnnKKeX836K3hTvKeIp'
      a += 'G6Y0TLkxZVxrqpaXllirSKtRq0frvTau7aedpr1Fu1n7gQ5Bx0onXCdHZ4/OBZ3nU9lT3acK'
      a += 'pxZNPTr1ri6qa6UbobtEd79up+6Ynr5egJ5Mb6feeb3n+hx9L/1U/W36p/VHDFgGswwkBtsM'
      a += 'zhg8xTVxbzwdL8fb8VFDXcNAQ6VhlWGX4YSRudE8o9VGjUYPjGnGXOMk423GbcajJgYmISZL'
      a += 'TepN7ppSTbmmKaY7TDtMx83MzaLN1pk1mz0x1zLnm+eb15vft2BaeFostqi2uGVJsuRaplnu'
      a += 'trxuhVo5WaVYVVpds0atna0l1rutu6cRp7lOk06rntZnw7Dxtsm2qbcZsOXYBtuutm22fWFn'
      a += 'Yhdnt8Wuw+6TvZN9un2N/T0HDYfZDqsdWh1+c7RyFDpWOt6azpzuP33F9JbpL2dYzxDP2DPj'
      a += 'thPLKcRpnVOb00dnF2e5c4PziIuJS4LLLpc+Lpsbxt3IveRKdPVxXeF60vWdm7Obwu2o26/u'
      a += 'Nu5p7ofcn8w0nymeWTNz0MPIQ+BR5dE/C5+VMGvfrH5PQ0+BZ7XnIy9jL5FXrdewt6V3qvdh'
      a += '7xc+9j5yn+M+4zw33jLeWV/MN8C3yLfLT8Nvnl+F30N/I/9k/3r/0QCngCUBZwOJgUGBWwL7'
      a += '+Hp8Ib+OPzrbZfay2e1BjKC5QRVBj4KtguXBrSFoyOyQrSH355jOkc5pDoVQfujW0Adh5mGL'
      a += 'w34MJ4WHhVeGP45wiFga0TGXNXfR3ENz30T6RJZE3ptnMU85ry1KNSo+qi5qPNo3ujS6P8Yu'
      a += 'ZlnM1VidWElsSxw5LiquNm5svt/87fOH4p3iC+N7F5gvyF1weaHOwvSFpxapLhIsOpZATIhO'
      a += 'OJTwQRAqqBaMJfITdyWOCnnCHcJnIi/RNtGI2ENcKh5O8kgqTXqS7JG8NXkkxTOlLOW5hCep'
      a += 'kLxMDUzdmzqeFpp2IG0yPTq9MYOSkZBxQqohTZO2Z+pn5mZ2y6xlhbL+xW6Lty8elQfJa7OQ'
      a += 'rAVZLQq2QqboVFoo1yoHsmdlV2a/zYnKOZarnivN7cyzytuQN5zvn//tEsIS4ZK2pYZLVy0d'
      a += 'WOa9rGo5sjxxedsK4xUFK4ZWBqw8uIq2Km3VT6vtV5eufr0mek1rgV7ByoLBtQFr6wtVCuWF'
      a += 'fevc1+1dT1gvWd+1YfqGnRs+FYmKrhTbF5cVf9go3HjlG4dvyr+Z3JS0qavEuWTPZtJm6ebe'
      a += 'LZ5bDpaql+aXDm4N2dq0Dd9WtO319kXbL5fNKNu7g7ZDuaO/PLi8ZafJzs07P1SkVPRU+lQ2'
      a += '7tLdtWHX+G7R7ht7vPY07NXbW7z3/T7JvttVAVVN1WbVZftJ+7P3P66Jqun4lvttXa1ObXHt'
      a += 'xwPSA/0HIw6217nU1R3SPVRSj9Yr60cOxx++/p3vdy0NNg1VjZzG4iNwRHnk6fcJ3/ceDTra'
      a += 'dox7rOEH0x92HWcdL2pCmvKaRptTmvtbYlu6T8w+0dbq3nr8R9sfD5w0PFl5SvNUyWna6YLT'
      a += 'k2fyz4ydlZ19fi753GDborZ752PO32oPb++6EHTh0kX/i+c7vDvOXPK4dPKy2+UTV7hXmq86'
      a += 'X23qdOo8/pPTT8e7nLuarrlca7nuer21e2b36RueN87d9L158Rb/1tWeOT3dvfN6b/fF9/Xf'
      a += 'Ft1+cif9zsu72Xcn7q28T7xf9EDtQdlD3YfVP1v+3Njv3H9qwHeg89HcR/cGhYPP/pH1jw9D'
      a += 'BY+Zj8uGDYbrnjg+OTniP3L96fynQ89kzyaeF/6i/suuFxYvfvjV69fO0ZjRoZfyl5O/bXyl'
      a += '/erA6xmv28bCxh6+yXgzMV70VvvtwXfcdx3vo98PT+R8IH8o/2j5sfVT0Kf7kxmTk/8EA5jz'
      a += '/GMzLdsAAAAgY0hSTQAAeiUAAICDAAD5/wAAgOkAAHUwAADqYAAAOpgAABdvkl/FRgAAFMVJ'
      a += 'REFUeNrsXf1PU9cffu5LW15aaktAWyoyRMo025TEGadgFvfj/rz9sGTZkv1iTJaYLZvuxSwG'
      a += 'pma+wjDqQCfy2iog1Ja3tvee7w967vf09Ja29BZa+DwJKZRLe3s5z32ez+d8zucojDEQCIT6'
      a += 'gUqXgEAg0hIIhCpC30sfZnFxkSWTScTjcQCAaZrIZDJYWloCACiKAtM0oWkaGGPWl6IoNBL2'
      a += 'EPj/VNM0HDx4EIqiQFVVuFwutLW1IRAIoLm5uW7/6YpdTDs8PFzTJ3369Gk2NTWFFy9eYGlp'
      a += 'CQsLC4jFYuCfRVEUKIqCaDQK0zShqqpFYv5P5cfJvyPsPfJOTEzk/c7j8aC9vR2hUAhtbW3o'
      a += '6urC9PR0zRN5cHCwfkh76NAh9uLFCzx//hxTU1Po7e0FYwzd3d1gjMHtdpNiEkomM0c2m8Wz'
      a += 'Z8+gqipmZmbQ3d2N7u5u9Pb24tmzZwqRtkwcPXqUPXz4EPfv30ckEkFXVxfcbrelkgRCNcg8'
      a += 'MTEBxhjW1tbQ39+PkydPYnR0VCHSbgHDMNj9+/dx4MAB9Pb2EkkJu0biTCaDP/74A9FoFGfP'
      a += 'nsXi4qJCpH2PU6dOsVu3bmFhYQGhUAgul4uISqgZbG5u4uXLl0ilUhgcHMTq6qqyW6Td9Smf'
      a += '/v5+lkgk2I8//ghd19HZ2UnxKaHm4PF4EI1G0d/fj+XlZdy9e5c1NjbuSmXSrpJWVVX2008/'
      a += 'oaGhgchKqAsoigK3243Tp09jZWUFDx8+ZEePHmV7nrRdXV3swYMHzDAMRCIReDweIiuh7sjr'
      a += '8Xhw6tQpPHnyBIuLiztG3B0vrlhcXGTj4+Po7+8nohL2DHndbjfm5ubY8ePHkUgkqjqwd0xp'
      a += 'P/zwQ/b27VvW0tKCaDRKhCXsOfKGw2GsrKzgzZs3rO5J6/f72aNHj+Dz+ax5VgJhLxJXURQE'
      a += 'AgEkk0l2/PhxVpekffPmDVtZWYGu66SuhH1DXq/Xi7GxMbS3t7O6Im0mk2GBQIDIStiXxNV1'
      a += 'Ha9evYLTU0NVI+3jx4+ZaZpEWMK+J+/KygpM02Q1S9pPPvmEMcbYsWPH4HK56L9G2PfweDzI'
      a += 'ZrNYXl52hLiOTvmcOXOG/f333wBACScCQYDb7eYixgBUZD9VJwl7584dyxIQCIR8q/y+1r8i'
      a += 'xXVMae/cubOrXSBUVUVjYyMaGhqsJVaapqGxsREAsLq6CuDdyg3TNLGxsYHNzU1QYzvCThMX'
      a += 'ALxeL0ulUsqukZa9H/k7SVhN0+Dz+azHw4cPy3eznJ/lR8YY4vE40uk0VldXkU6nkUwmaVQR'
      a += 'dgSpVAqhUIjFYjFlx0mraRozDGPHPmwwGISu6/B6vYhEIlvezcS2MsINxnouHA7nPDc3N4dE'
      a += 'IoG1tTWsr68XfA0CwQnEYjF8/PHHbGxsTNkx0kajUTY+Pr4j1jcQCMDv96OjoyNHLbcilPy8'
      a += 'ndqKz3d0dCAcDkNRFDx//hypVIrUl1BVjI2N7aw93gnCBoNBHDhwAJFIJCdmtrO95SQDCv09'
      a += 'J3JPTw9M0wRjDKOjo1hbW6MRRqgWysoo6xW+UdXg8/nQ2NiIvr4+GIYBXqghJ47487zrokjE'
      a += 'YrbW7u/kuNk0TYTDYSQSCSwvLyObzdIQIzjLWMbQ1NTE1tfXlaqRtrGxkfGYz2koioJgMIhA'
      a += 'IIBQKATDMHL6FHNCcqKpqmpL6FKUV3wdO+XlbVVDoRDC4TCmp6extLRElpng+Jgvh0/bIm21'
      a += 'COvxeNDS0oK+vr6chtOyaooEtbPMxYgqHi9bZvkYMfbt7OxEJBLB06dPrQboBMJO22R1my9c'
      a += 'FTscCATQ19dnS0CZWHwZlF2yqdhdrdjzvIG5/B5cmU+cOIG2tjbrOALBKZw4cYI5StqTJ09W'
      a += 'jbCtra04duxYntKVknRyYjpGvEHYqbWsvH19fQgEAtB1nUYawTE8fvwYjpJ2dHS0KoQ9ePAg'
      a += 'Ojs7LXKICiZv9VHq9M524gr5JiGTVX6P48ePIxgM0sIIgnP++J2bY46QtrW11XGV9fv9aGtr'
      a += 'QygUyiMF/3m3ihpkEstxMD+/aDSKQCBAVpng2Lgrtq9UySPN6cRLY2MjAoEAOjo6rOf4vGg9'
      a += 'XFiedQaAvr4+tLe3U9UUwTFspba7Ig+apuUUTNjFkYUIUGuk5jFub28vgsEgjTaCI9hKbUsl'
      a += 'raNMaW1tRU9Pj2U77aZtCpGzFtSs0FaZvNSSQHACDQ0NrCaUNhgMWvvG2sWL9bJPLF/ix284'
      a += 'qqri0KFD8Hg8lFEmOIKNjY3tKa2u646prK7r8Hg83LNbg18mbqEEVK1YY7ukFK/KikajpLaE'
      a += '6sa7xQ5wstY2GAyip6fHspOiPRZj21q2xrI9Fm86/PwCgQB8Ph+NLkLlBLVJSG1J2s7OTsek'
      a += 'rbm5GV6v1xrYYnmidJI1fyHlaR95kUIoFEJzczONOELFsAsXt2TIzMwMnCStvGi9kCWuh6kT'
      a += 'uxJHfoEZYzh27BjZZMLO22OnYsimpia0tLTkrNTZC5Bro+VMOI/fCYRKIOeVdsSLNjY2IhQK'
      a += '5SjRXoC82khV1Rz17e3thdfrpVFHqAhyXqkgaf1+P3NqYPv9fjDGoGla3djfUhR2q2WBnMSk'
      a += 'toQds8eJRMKRN/B6vVZfJ1Fp61ltxZuOuABftsiMMfj9fipvJFQMcSOvqtvjhoYGa9CK9rGe'
      a += 'B7JcESVnj0Vih0IhtLS00KgjVIQ3b97sXEzb3Nyco6z1siigVKW164QhHqeqKlVIERyNa6tK'
      a += 'WrE3cbldE+uRwLIa889Lc7aEqse0Bw4ccEQK3W63ZYlLGeh7lcidnZ2ktoTqktapHr+6rudN'
      a += '8+zHpAypLaHqpM1kMo68eFNTU15v4f0AOYbnXSUJhKqR1ily8UXuYo/indz3Z1cv7PuwgPdl'
      a += 'JtISKkUwGGQFSevUoOWJGE5UwzD2leJypeXbcBIIlYD3G68qafmKHb6iZ69URJVjk7nSEgiV'
      a += 'gotf1UjLs6X7MaaVIbeFJRC269qqRlrTNNHU1JSjNvspe2xXfCF2nSQQtuvcqqq04XC4rB3s'
      a += '9vJFFu+SBEKlYlA10m5ubuY1/N5vhLXbKYHIS6gUVSXtfo1h+U1KnqtNpVJ5CkwglEzW9yJQ'
      a += 'NdKura2RLZZcxurq6r68HgRnYHUyrdag5UuJ6n3trFOKaxgG3r59SyOPsG243e7CpK1ECXjM'
      a += 'lkgk8jZn3m8JA349eCXYysoKjTzCtrG0tKQUJG1DQ0NFA3Z4eBgLCwvWZLDdrup7Hdac2vvi'
      a += 'ivX1dUtpKaYlVBTb2j25vr6+banlxHz16hVM08zZOmM/2mL++WOxGBhjuHnzJpGW4DxpK35R'
      a += 'VUUymcSrV6/y7PF+UFu57Yxpmpiamsrrk0Ug7Dpp5cE6PT2NbDZru1/PXieqbJUnJyct0t6+'
      a += 'fZtGHqF2lPbmzZtWre34+HhOIka0jXtNaexKF3k8m0gkEI/H8wouCIRSIeaZ1K0s7nbBLeDU'
      a += '1BRWVlas9aRi36S91PvYzu6Kz4+NjeU0MydrTCgXGxsbSlHStre3b3sw8wUC2WwWT548QTab'
      a += 'zdmPtt6V1m7jLbs+WPwaPHr0yHIb+33emlBFexyPx7clhTxu4y1WRkdH85bm1bvSyrsKiDci'
      a += 'OeE2Pz+PWCy2J9wFYY/GtLI9jMVimJiYQDab3XOFFnZKK5IznU5jeHjYim15yEHkJZQDvsy1'
      a += 'JNJW0vaTD2ZVVTE0NGQ1W94rimPXpFyuhFpcXMS///6bt10IkZZQDtbW1pSSSZvNZsseXefP'
      a += 'n88Z1IqiYGpqCs+fP7emfwolcOpFge3KM+VHwzAwPDyclzUHgM8++4xGIqE27LGoOGKTclVV'
      a += 'ce3aNaTTaUuFZJKKG1nVC3H5o3jufHHA5OQkRkZGLLcid2QktSWUAruGgEVJK/vpcqwxV1pV'
      a += 'VRGLxXD37l0YhpH3ez7o6wX8XMWMuKjAGxsb+Pnnn63PqWnavmu5Q3AGbW1t5ZNW9tOlqpA4'
      a += 'SBljcLlcuH79Ol6/fm0R1zqJOp2/5B0nRbVNp9O4ffu2lTHmUz1ijE8glIrp6WmlqvaYq4hd'
      a += '0kXTNGQyGVy+fBnpdDpvF/V6im3l+JV/zkwmg9nZWfz2229wuVw5LoJPgdEcLaFUtLa2VhTT'
      a += 'KuUMaHHfVq5IXG3m5+dx/fp1bGxs5Bxfr3O4YkuZVCqFy5cv51ln0UnwRB2BUAx8/eyOKO35'
      a += '8+dhmiZ0Xc8jMGMMt27dwsjIiKW4ckxbq2oklmGKME0Ta2tr+OGHH7C8vGwlnPjcLCft4OAg'
      a += 'xbSEkuDz+QqHZeXwsZwkjdWESthlgD9ms1lcuXIFExMTeVa5lrsVii5AvLFks1n88ssvmJiY'
      a += 'yHMbdnEwgVAMyWRScYK0ZQ9w+VGsDDJNE5cuXcLU1BSy2WzO9hlyvCuqnBMqXOw1tir+l2Pv'
      a += 'TCaD33//Hffu3cshtpg5p4IKglMqux3Slqy2AwMDli3mKssYy9kuZHNzE9988w0mJiawsbGR'
      a += 'c5xc1+ukrSzldfh8q+wARNJmMhlcuXIFQ0NDOUqqaVoOUYm4BKdUdltKW4q9k7PGvLBA3PKS'
      a += 'k9kwDHz//fd49OhRjlW224y6WrFuodcVd/4Tp2zS6TTW1tZw+fJl3Lt3L0dRDcPIsfjia1Dm'
      a += 'mFAMdvOyFZPWNE2lVCLwKQ5d1/PUkisut5yXLl3CtWvXsLq6alvXW83kkkwouVhCtO3pdBpL'
      a += 'S0v49ttvMTIyAuD/W3jyGxMnME/EaZqG8+fPk9ISigriwsKC4jhpAcDr9ZaktufOnctRWlGB'
      a += 'edM3XdetQT80NITvvvsOCwsL1m70YhxZisUsR822Wg8rfg7uLtLpNJ48eYKvvvoKU1NTFild'
      a += 'Lpdlp8VzEMsbyR4TiiEcDpcW3tkN8uHh4ZLGfCmkYIzhr7/+QjqdzqnN5fGirGSMMXg8Hnzx'
      a += 'xRc4e/YsdF23SCESrZAKF+ruL1ttmbhiEYR8/oZhIJFI4OrVq/jnn38sJc1ms9B13Uquic5C'
      a += 'VVWL1BcuXNgR10CoX3i9XqRSqaIDZHBwENtee3fo0CHE4/GSEj6iVeZL9MRssVz2uLGxgatX'
      a += 'r2J0dBRffvklOjs7oWmaZT+3IqpYuG/XTaIQgeVjeWY4k8ng3r17+PXXX5FOp63zFGNyTlL+'
      a += 'ucRiCjGOJxAK2eJSCMuxbdLG43ElGAwyvv1HMQsq195ywhqGAbfbjUwmYyVyOMFnZ2fx9ddf'
      a += 'o6enBxcvXsSRI0eg63rejvJyM3SZsFu1g5FJz8m6ubmJBw8eYGhoCCsrKzlliOLfir2vxCyx'
      a += 'HN8SCJXmiZywxwAAt9vN+JK7YhaZx3yZTMbKEPNm3pzAcgM4MaaNRCI4deoUPvroI/h8vjzl'
      a += 'leuet4op5aWApmkik8kgHo9jZGQEo6OjSCaTVswt1g3bdavgBOWOgBN3YGCArDFhSw6Wc3BF'
      a += '9pijvb0dc3NzBRNAfLDywXvjxg0rOSW3VxXXm8rFFqqqYn5+HrOzs7h27Rqi0Sh6enrQ09MD'
      a += 'v98Pj8eTkzQSFZ0TmL8mv1FwVZ2cnMTk5CTGx8cxNzdnWVqe9eY22G4xhFhEwckq2mQiLKEQ'
      a += 'fD4fkslk2X9XMWlnZ2eVjo4ONjc3t2Vsy4kjEkBUL05cDp5R5uQStxcxTRNPnz7F48ePwRhD'
      a += 'MBjEwYMHEYlE4PV6EQ6HLZK6XC5sbm5airm+vo7l5WXE43HMz88jHo9b/as0TcuZiuLnIJ4L'
      a += 'J75dXyj+GpzE586do5FJsEVDQ0PRIoqq2WOOUCjEYrFY0fiWMYahoSErKcXJYVdMwckiW1n+'
      a += 'yDO3sqLKMSv/Pb9JFEpm8SkocfMs8T3F+WaRxDxTzL8fHBzc07spECqDruvbauXE7bFjKc1Y'
      a += 'LKZ0dnZufYd4P4AvXLiQU3wgJmz4MWI9smxD+bEimcUqK7Gogf/M17eKFlwkrKZpcLlc1kXl'
      a += 'Fp4T3OVy5WS++XGiLRYJS/OyhEIc2C5hrdDRyROanp5WijU5l6dGeDKJk0lULpGU3D7z+V27'
      a += 'mFWe7pEXGdjZWXHpILe9/D1EZeZZbvH8+Y1HVlgiK6EQYRljFQ8OxycPvV4vOjo6ip78wMBA'
      a += 'XhzI1dAuPhQVlZNIXIcrHiMTV7TUdjva2RGb3yDE1+fvJ95w5CZ2RFhCIUvsBGGrQtoXL14o'
      a += 'H3zwAUKhUNHE1MDAgGVhxakSOxLI851c5exav9gVN8i7Aoi2uND5iSuSRLstf8lxLIEgJ50q'
      a += 'tcRVJS0A3Lx5U+np6UEkEtmyppeX+InxrPi9HYGLkdGudY28cN1u0X2hrTjleJv/LE7niK6B'
      a += 'QJCdp7h5liOqXUX/rhw5coQZhoFkMolUKrVlnPs+o5ZTyyta1632zZGTSuL3duWJdoQWj5fX'
      a += 'work5QTm3/PVO0RYgoxQKIRYLOb4wNCredKapimxWAzd3d3M5/Oh0JQQb3Y2PDycV8XEf85m'
      a += 's1ZGV1Zeu0bnxXa0E7PScnsY/pxYXMHjEjEDzc+bCEuwSzgVmwKtSdKKca6gqkwsXRTBG58B'
      a += '7yqnxKSTy+XKK3O0I5vdah07wsrVS5zodoX/PL4Vbxi8wotAENHc3IzV1dWq3sUdK64oB62t'
      a += 'rWxpaank42/cuIFMJmM1heOwawFTynpau8ULIvnFhJiYBON2mNSVUEhdq/0+jtQebwdSP9ei'
      a += 'LPv8889zCCwWOYiqa2eJ7Ugsx63iVJOY4OLTOufOnSu6WJ6wf9HW1lZSx4m6ssfFblKHDx9m'
      a += 'MzMzJR0sEhgA/vzzz7yklDi/arcoXlxiZ0diUVk5YSnZRChkhRcWFnb0fWuBtJiZmVEAoKur'
      a += 'i718+bKsv7148WIeicUYVSSr3fcyaXmz9a2OJ+xv+P1+JBIJZXV1dVfeX6+li/Hy5UuZHWW3'
      a += 'L5RJXCrkHfCIqIRCZE0kErt6HrXeA0UppaVkJeDra6mxOMFW1d5VxSkAlEQiURMDo+YbF70P'
      a += '8JUzZ86go6PDcUJRSxiCjPcbOSsAFCfLD/eUPRbnZ+0wPDyMO3fu5Fy8Tz/9lL1+/RpLS0tY'
      a += 'WFhAQ0ND2e9L2WAC8K42WCw1XF9fr237aRgGU1VVkUlSSyhGartz7u3tZeIiepmg//33n1Xh'
      a += 'ZF2MLQhMOwTUN9xud1kdD2sVg4OD+N8ATz/p0vF3GcIAAAAASUVORK5CYII='
   return a

   function image_on
      b = ''
      b += 'iVBORw0KGgoAAAANSUhEUgAAAO0AAABtCAYAAABN/eZXAAAACXBIWXMAAA7DAAAOwwHHb6hk'
      b += 'AAAKT2lDQ1BQaG90b3Nob3AgSUNDIHByb2ZpbGUAAHjanVNnVFPpFj333vRCS4iAlEtvUhUI'
      b += 'IFJCi4AUkSYqIQkQSoghodkVUcERRUUEG8igiAOOjoCMFVEsDIoK2AfkIaKOg6OIisr74Xuj'
      b += 'a9a89+bN/rXXPues852zzwfACAyWSDNRNYAMqUIeEeCDx8TG4eQuQIEKJHAAEAizZCFz/SMB'
      b += 'APh+PDwrIsAHvgABeNMLCADATZvAMByH/w/qQplcAYCEAcB0kThLCIAUAEB6jkKmAEBGAYCd'
      b += 'mCZTAKAEAGDLY2LjAFAtAGAnf+bTAICd+Jl7AQBblCEVAaCRACATZYhEAGg7AKzPVopFAFgw'
      b += 'ABRmS8Q5ANgtADBJV2ZIALC3AMDOEAuyAAgMADBRiIUpAAR7AGDIIyN4AISZABRG8lc88Suu'
      b += 'EOcqAAB4mbI8uSQ5RYFbCC1xB1dXLh4ozkkXKxQ2YQJhmkAuwnmZGTKBNA/g88wAAKCRFRHg'
      b += 'g/P9eM4Ors7ONo62Dl8t6r8G/yJiYuP+5c+rcEAAAOF0ftH+LC+zGoA7BoBt/qIl7gRoXgug'
      b += 'dfeLZrIPQLUAoOnaV/Nw+H48PEWhkLnZ2eXk5NhKxEJbYcpXff5nwl/AV/1s+X48/Pf14L7i'
      b += 'JIEyXYFHBPjgwsz0TKUcz5IJhGLc5o9H/LcL//wd0yLESWK5WCoU41EScY5EmozzMqUiiUKS'
      b += 'KcUl0v9k4t8s+wM+3zUAsGo+AXuRLahdYwP2SycQWHTA4vcAAPK7b8HUKAgDgGiD4c93/+8/'
      b += '/UegJQCAZkmScQAAXkQkLlTKsz/HCAAARKCBKrBBG/TBGCzABhzBBdzBC/xgNoRCJMTCQhBC'
      b += 'CmSAHHJgKayCQiiGzbAdKmAv1EAdNMBRaIaTcA4uwlW4Dj1wD/phCJ7BKLyBCQRByAgTYSHa'
      b += 'iAFiilgjjggXmYX4IcFIBBKLJCDJiBRRIkuRNUgxUopUIFVIHfI9cgI5h1xGupE7yAAygvyG'
      b += 'vEcxlIGyUT3UDLVDuag3GoRGogvQZHQxmo8WoJvQcrQaPYw2oefQq2gP2o8+Q8cwwOgYBzPE'
      b += 'bDAuxsNCsTgsCZNjy7EirAyrxhqwVqwDu4n1Y8+xdwQSgUXACTYEd0IgYR5BSFhMWE7YSKgg'
      b += 'HCQ0EdoJNwkDhFHCJyKTqEu0JroR+cQYYjIxh1hILCPWEo8TLxB7iEPENyQSiUMyJ7mQAkmx'
      b += 'pFTSEtJG0m5SI+ksqZs0SBojk8naZGuyBzmULCAryIXkneTD5DPkG+Qh8lsKnWJAcaT4U+Io'
      b += 'UspqShnlEOU05QZlmDJBVaOaUt2ooVQRNY9aQq2htlKvUYeoEzR1mjnNgxZJS6WtopXTGmgX'
      b += 'aPdpr+h0uhHdlR5Ol9BX0svpR+iX6AP0dwwNhhWDx4hnKBmbGAcYZxl3GK+YTKYZ04sZx1Qw'
      b += 'NzHrmOeZD5lvVVgqtip8FZHKCpVKlSaVGyovVKmqpqreqgtV81XLVI+pXlN9rkZVM1PjqQnU'
      b += 'lqtVqp1Q61MbU2epO6iHqmeob1Q/pH5Z/YkGWcNMw09DpFGgsV/jvMYgC2MZs3gsIWsNq4Z1'
      b += 'gTXEJrHN2Xx2KruY/R27iz2qqaE5QzNKM1ezUvOUZj8H45hx+Jx0TgnnKKeX836K3hTvKeIp'
      b += 'G6Y0TLkxZVxrqpaXllirSKtRq0frvTau7aedpr1Fu1n7gQ5Bx0onXCdHZ4/OBZ3nU9lT3acK'
      b += 'pxZNPTr1ri6qa6UbobtEd79up+6Ynr5egJ5Mb6feeb3n+hx9L/1U/W36p/VHDFgGswwkBtsM'
      b += 'zhg8xTVxbzwdL8fb8VFDXcNAQ6VhlWGX4YSRudE8o9VGjUYPjGnGXOMk423GbcajJgYmISZL'
      b += 'TepN7ppSTbmmKaY7TDtMx83MzaLN1pk1mz0x1zLnm+eb15vft2BaeFostqi2uGVJsuRaplnu'
      b += 'trxuhVo5WaVYVVpds0atna0l1rutu6cRp7lOk06rntZnw7Dxtsm2qbcZsOXYBtuutm22fWFn'
      b += 'Yhdnt8Wuw+6TvZN9un2N/T0HDYfZDqsdWh1+c7RyFDpWOt6azpzuP33F9JbpL2dYzxDP2DPj'
      b += 'thPLKcRpnVOb00dnF2e5c4PziIuJS4LLLpc+Lpsbxt3IveRKdPVxXeF60vWdm7Obwu2o26/u'
      b += 'Nu5p7ofcn8w0nymeWTNz0MPIQ+BR5dE/C5+VMGvfrH5PQ0+BZ7XnIy9jL5FXrdewt6V3qvdh'
      b += '7xc+9j5yn+M+4zw33jLeWV/MN8C3yLfLT8Nvnl+F30N/I/9k/3r/0QCngCUBZwOJgUGBWwL7'
      b += '+Hp8Ib+OPzrbZfay2e1BjKC5QRVBj4KtguXBrSFoyOyQrSH355jOkc5pDoVQfujW0Adh5mGL'
      b += 'w34MJ4WHhVeGP45wiFga0TGXNXfR3ENz30T6RJZE3ptnMU85ry1KNSo+qi5qPNo3ujS6P8Yu'
      b += 'ZlnM1VidWElsSxw5LiquNm5svt/87fOH4p3iC+N7F5gvyF1weaHOwvSFpxapLhIsOpZATIhO'
      b += 'OJTwQRAqqBaMJfITdyWOCnnCHcJnIi/RNtGI2ENcKh5O8kgqTXqS7JG8NXkkxTOlLOW5hCep'
      b += 'kLxMDUzdmzqeFpp2IG0yPTq9MYOSkZBxQqohTZO2Z+pn5mZ2y6xlhbL+xW6Lty8elQfJa7OQ'
      b += 'rAVZLQq2QqboVFoo1yoHsmdlV2a/zYnKOZarnivN7cyzytuQN5zvn//tEsIS4ZK2pYZLVy0d'
      b += 'WOa9rGo5sjxxedsK4xUFK4ZWBqw8uIq2Km3VT6vtV5eufr0mek1rgV7ByoLBtQFr6wtVCuWF'
      b += 'fevc1+1dT1gvWd+1YfqGnRs+FYmKrhTbF5cVf9go3HjlG4dvyr+Z3JS0qavEuWTPZtJm6ebe'
      b += 'LZ5bDpaql+aXDm4N2dq0Dd9WtO319kXbL5fNKNu7g7ZDuaO/PLi8ZafJzs07P1SkVPRU+lQ2'
      b += '7tLdtWHX+G7R7ht7vPY07NXbW7z3/T7JvttVAVVN1WbVZftJ+7P3P66Jqun4lvttXa1ObXHt'
      b += 'xwPSA/0HIw6217nU1R3SPVRSj9Yr60cOxx++/p3vdy0NNg1VjZzG4iNwRHnk6fcJ3/ceDTra'
      b += 'dox7rOEH0x92HWcdL2pCmvKaRptTmvtbYlu6T8w+0dbq3nr8R9sfD5w0PFl5SvNUyWna6YLT'
      b += 'k2fyz4ydlZ19fi753GDborZ752PO32oPb++6EHTh0kX/i+c7vDvOXPK4dPKy2+UTV7hXmq86'
      b += 'X23qdOo8/pPTT8e7nLuarrlca7nuer21e2b36RueN87d9L158Rb/1tWeOT3dvfN6b/fF9/Xf'
      b += 'Ft1+cif9zsu72Xcn7q28T7xf9EDtQdlD3YfVP1v+3Njv3H9qwHeg89HcR/cGhYPP/pH1jw9D'
      b += 'BY+Zj8uGDYbrnjg+OTniP3L96fynQ89kzyaeF/6i/suuFxYvfvjV69fO0ZjRoZfyl5O/bXyl'
      b += '/erA6xmv28bCxh6+yXgzMV70VvvtwXfcdx3vo98PT+R8IH8o/2j5sfVT0Kf7kxmTk/8EA5jz'
      b += '/GMzLdsAAAAgY0hSTQAAeiUAAICDAAD5/wAAgOkAAHUwAADqYAAAOpgAABdvkl/FRgAAL15J'
      b += 'REFUeNrsvXm05Vd13/nZ+5zffVPNpSqVVJIQEhpADAIRGTAIy0DAsDxhBy/bxGE5jr1W4tXG'
      b += 'zmrbWWm303G3HSfpZrndnY5DGtMeMB1sOthMhoABY0sYgUBoRkNpLpVqHt5wf+fs3X+c87v3'
      b += 'vlevJk3U8Ntr1bqv7nv17nu3zvfs6bu/W9yd3nrr7cwx7d+C3nrrQdtbb709hxbP5B9+/9K8'
      b += '7zyym8f2P8VhXyLkTNbMg/t3kiwjbv29dJaZuyMioz9Dy6hGxJzgxrYNm5mODdEDILxg/UVs'
      b += 'nl7H9nVbpAft82x3PnWf3/nk/dy38CQP7HmMRw49wZE0j7qQJ/47BBBzXMuTfcZ+9piIjIDb'
      b += '1WIiMvqcoZhRAe2IZ1p13GE6B3/hpu1cvHYb56/dyqu2XMlV2y5lQ5g948Asp2sh6v4Dj/nX'
      b += 'HruTW/bew7eeuI/54WEwJ4QpWocgGVenUUWbiKki5qgIHhTV7j6y/rSfZV62A6m7I+aYACqY'
      b += 'GeKOpYSgZARSi7ljBp4N0QJ4U2GQ4aotL+LvXXAlr9p8Fa+88CXSg/YUbe/ifv/8g1/lszu+'
      b += 'wp37doCCUsE4mKJpAgRhMBWJYUBASeL1FwHDUS3/mSagvZs9680EAlI9b3lUVbByWbsExA0z'
      b += 'wyxjKYE5aWHIfEqAoTkjqqyN63nr5a/l77/wu7hy08XSg/Y4dtOjt/nHvv1Fbn7idlo1gsNA'
      b += 'GpqZaUITGQwiWR2PimYnePGfuaarK8E5AqwYeJ/TntUhs4PLCUBdg63Reck+8tbDZLRLQ4bD'
      b += 'IUtLC0QVcOGFc9t561V/j5+86h3Sg3bCPrPjJv/jez/DjqceA4wQAjI7xfT0FIM4wAVEGOUv'
      b += 'XWhk9RZVVfo+c2+TYfNqefDxzsjo8xXYS/MLLC0uklICYG2Y44ev/h7eecX3smlmnZyzoP3M'
      b += 'fV/2P773s+zY+yQAg5kBg7kpYox4dFyU4IaLICajG7WEO/0h7e3UilfHOuNdRNZ9jTJ2AnnY'
      b += 'srS0xKGFRLTETDPgBy9/PT/x0u9jw+A7C97nFbR373/Y3/flP+KOgzvQLEyvGTAzN4MPIuqC'
      b += 'u0EIaC7AdREkt9XjLgdvD+Denk7+O5lOrQytxx57/KQzZOlQy9KReYbmrB2s52de8X386JVv'
      b += 'krMetL/9tQ/5J+/8K4aNs256hsFcQwgNFqaYStCS0NiQ2yU0RjzncgPW8FgcUDmlfKa33lwF'
      b += 'MT9mtLbyuZWeWVFcnJyGpMXMwqHDZIwr1l3KL333j3PNhsvlrAPtFx66xf/dLR/m8OI+wvSA'
      b += 'tWvWIYNY3yBwMplMZEDyhHogYmQJmDjRBcPJlFtQHTBf9ib31tupFqo6sBpjsoaZLTtPBcCA'
      b += 'JyQorQMGCwfnaRfmCQbff8WN/NL1PyFnDWjf97UP+5/f/VdkgTUb1zM11SACWQ3NQlbBJTNl'
      b += 'gSQZl4BaxtFRvrGydTMZ4vSetrdTAe8yDyzj8+TuBGRVT2siBDESQiNgKeMh0i4usXBwAW8T'
      b += 'L9rwQv6n17+H7eu2yRkL2icOPem/9uUPct+eb+Nrp9iwdv3oNvOcQQxTIViob2DXYzMUBRyX'
      b += 'muMex5v2vdjeTlg5zjZKs1YDclZGgD0+FoSMMZBAsoyqEhwOHVlk8fAhBiHyP7zmPbzx4uvl'
      b += 'jAPt3z5xu//6X/9nFtMB1q7fwMzMDEOVwpf0TBYlWCALqLTgusx7uhcK4uTt11tvJwPaSV7y'
      b += 'iUF4ct9XMMwV13F0VxyNIGRSCMh8y6GDB0mt8UNXv5l/ft275IwB7acf+Bv/Nzd/EG0Cc+vX'
      b += 'M4gBm8gbujfieLdjb+dm2OoyjpzK+ahMCMmI6fjMiJHr5wYZsoy/3qWmTG6jouXYIUzmquPi'
      b += 'prsXCqQYCogpIKBCxlH3o3Lj8nN2eXBGNZBzZuHAftrFxA0XvZp/fcPPyWkP2g/d9Rn/va//'
      b += 'F5rZKdasW49FRc0R74pGQk/f7+3oE2gIATNDVUfFIKfWOGorsBsAURcUsCDk3BIljjzqpKf1'
      b += 'bFhw1BXBxuAs0CxgFEM9kOv3n6yhSGVb+MSUWBdOj1MyKReAJaIHsgSOLBxi/uAhrjvvJfzO'
      b += 'W/65nLag/Z1bP+wfvedzxGbAms1bUGtBSkHJvdxsk9U4qfmqqOPWe9xz2TIZdSVIqMCiejgp'
      b += '3k078EJyJ1byklRH4O4EKZ44T4SwWWvnoV4Gk6A2Sg1FXMnaEgiIQ4sRDCw47kJAxpHhUZ62'
      b += 'vH7wFg9TtNlL14NEHgqH9h7k8nVb+Ddv+UXOn9oopxVo/9M3PuofuvNTxLlp1qzdRPBE2xT+'
      b += 'cHaZYJyMwTqZ3Pfe99w2ExmFoIghzghYhiIixDaRVCEaYkIwSCEiyRAJmCXAicWn4iIEF5Iq'
      b += 'QZyWEi6LA7mAvFwO5cIogCwePIlX4ApkHzmcjsOctTx2AI4eaW2IR8CMoAPMjGFOHN69jyvW'
      b += 'X8x/ePOvMD2YktMCtH9y72f9d2/5M9atn2V2dgZXw03w7AQBVEdBxjif6Khi3hP6e6vUQasd'
      b += 'heJti5ctZ8YAgiIpjxhyWQJqRvTIBWu2MEjKtjWbGYQpgkOSlscW9jC0loOL8xzOCyyR0I6c'
      b += 'o6VDEUSr12VE3lHLJKn5sx8dGnfAnSyaAsTsWPXoTQ3plzxzYM9ert18Bb9543tZEwbyHQXt'
      b += '5x68yf/1zX/IzJqG6TVrRm+yC2iopH5vgDzyqiU86Q9qbxPhsRbvql7z2RhrSFtDWwfLLdPW'
      b += '8KJ1F3LN+it58dqL2Ta3hXUza2mkeNviEALeeeucqjhC5PBwnp3zu7nvyKN8a+/9PHDgUfbb'
      b += 'AqaMClYigrgsC7G1htnjiKA4oJX8gFA9tUkmGLSxht4mhHbIwT37eM32V/BbN/z8dw60X9t5'
      b += 'l//qF/4jOu3MbZhjCExboA1ewpNcZxrJqIc+BO5t1YrxqM4hkHOmiQW42Y2QA9GUazdezhsv'
      b += 'fCUv3XgZg2aWxp04aCogayvRuiKTowg4OI4go4oy7rTmeMrk3HLf/BP83ZP38sUnb2G/LYC0'
      b += 'hFBbkV56sEuaCabj6TJZHhqX54qk0ajXmw0JAXcjaHFeC21mcfdB3nnlG/jvXv1ued5Bu2dx'
      b += 'l/+Tv/h3HGgWmdswV26ZmshnzwxUyFILUO5HNbX7ds+5DlhZ4WkzIkXjCXOSZbY0G3nr9lfz'
      b += 'pvNfwdpmI3FKCCoVFjVk7WiI4qM8eOUZ84nKSfdPRh9nI1lmaJlvPHknn3j0K9x1+KHy7TWQ'
      b += 'FBpRck4lx5bVBw3UigpGB2gLUvWqSmifVIjmzLcLLOwZ8q9e+4/43hc+MwLGKYP2Fz7/O377'
      b += '3nvYsGkDrcr4FzmqyNRbb9S6RSitFl1CLSJeGUqeQCJYUZbYquv44UvfwA1bX8Hs9HokFh5S'
      b += 'OVtyFMXw6fT8V/57MJI5adhy795H+LMdX+S2+ftwdfDi0YMYTgs6RUiZFBwnEJxVo8gO1CWi'
      b += 'CBASbkJ7eMhwcZ4PvOXXuHjDBfK8gPb9t/2Ff/iuT7Fu03pSo4Ts9eKrLRzR+th70t5YVuCh'
      b += 'TtokcaJCQghu0LZEWcOPXPA63nzpq9kwtZGAoKEQKQoow3IP+jRJOsf6dzW6JrvRtgt8bec9'
      b += 'fOj+z/GIP0XUgHkkkYnmBCI5tIXwcQxq5Pj50h0J2Ula8vfh7nkuWnMhH3jbv3juQXvrrnv8'
      b += 'vZ97H+s3boBpSq5qBbQjRoobqmEE3F5VojeT2taThFtkCqd1aAiklLh67WX89BVv5gXrL2Iq'
      b += 'DHDJIAHcSSJEz6XA1PVYV+m7HsubikiJURWMjDIJeMEpbZ6Sy5W2kcuAtAhH0iE+tuNL/Onj'
      b += 'X6QJgqqyKNCY4CJFg2piVHSyJVTmwcvngjqSlbaBkFuSTHFw917eecnr+YXrf1yeU9D+1Md+'
      b += 'zXfKAlNrG6J0gmplWqdQznzC0ypO6ts5vY1ZRFamY9wzgxxYyi3vvPB7+fFLXkczO43GAbgj'
      b += 'WSqJ30ET7vGoUPhUPK1TKkeCYF5ZV6OCVSUAiaGmoEKyTNQC6uGC8a0Dd/G+u/+M+bQIA4Mc'
      b += 'KiAdV1+mrLKyJSQitALBhoQ8hWhJH1NKHNy3yP/2pp/nleddecrAPSlUffj2z/qOtJfZjUpj'
      b += 'JZ6IZsQ0Vd+05YA1ywWw0ue457plheCGSoOYE7OQNfPLL/mH/ORVNzC1dm4MWAOPCZUSr2Zr'
      b += 'xrTEFfKpk0A+/qWhtZDkpT2LVWqkY4SaeyqmxeFGDRUWxmBWeMWmq/m31/4sl09fSE6xzKB5'
      b += 'KZppVYrrQuJJ8kXn8adTxpmlbQxDSDEjTcPUjPJ/fOUjT+s9PSFoHz3ylH/w7r9gy5q12LAh'
      b += 'NVqwqIJrOxGSjAE7IlD0nvact5gdp8EwkifW6Tp+48U/w2vOv5KmmR2FwuLgwcEj1MpvmDg+'
      b += 'xxu/O66nnRgYcFdAR/JF5W9SPC0yMVPWpXdKM6Wcv3YL//JVP811a1/IgiUaIhKKosWyopMK'
      b += '0SfaTMAwKEESsYy1QVJcjem5NexYeoyP3/tFf9ZB+7tf/TB5ANYoBKdpldw4lovqxPJKXpdv'
      b += '9G2d3rpcb0CWRdScC9jEv3rpu7l88yU0caq0aiqH17UQEpYrNNmqVd9TCs9HYfKY2LOsTmRe'
      b += '6zPjT7grReatBVM0RNbORN774nfxPVteybzNl5E9MVAdF5/My/DBRKtTvfRqLViVUFLEhKgt'
      b += 'U3Nz/O43/iu7l/b7swbaW598wG968husm1tLW71qJuEuhVpGOOpN7YtPvS3PKYc0WZn1Af/i'
      b += '2p/kovXbmIkDfCJ1km6kTmwsI+TLj+czdQLH+veuUsCqExXmru4rTXlenKhTrFuzjn962dt4'
      b += 'w7praFPJZ0uYzQiok/O8o9C55sgEH4XQQwk0TUOazvzxbZ949jztB2///1i7cRNJjIEPSk+2'
      b += '9mY7saze+kLTKOfsRuCQEnbm8tjQ8N+/7N1ctPZC4iCAgHiuVdux1ytfvcJFPtc//2rgljF4'
      b += 'vYbQWOHKT8+t4+df8mO8bPZSsEogKoO4hDyuJJvYqB+tlRKJK1kLcLVOv62dW8cnH/obdh55'
      b += 'yp8xaG/bdZ/fvvdBmhAnxpkmb8HeeoMUILlB6ELK0jlocWhKSPiLV76LazZfzqAJjPgI0ozI'
      b += '9qf1pWSOe655thBFmJ1p+KVrfoQLp87DqhhhFsWaemFVLrWfBFA8Oj4zw4fu+Mtn7mn/5O5P'
      b += 'M7VuBo2hkqlLzF7CAe/B2xsATXICinguyoVaevgBoHXesfU1vPK8lxBjqci6Fa90uhydY6Vz'
      b += 'Mh4CxwWSZMQhSenZbpzZwnuv+n5CbmglM5UVSTYOuRmrZhz/9WFmZoa/fOTmZwbaRw497jft'
      b += 'uYfBoBkB1etmMp0IYnJfHO5z1qAldHTFYyEzeFVzuGL2En7iRX+fwXQolEUTRLXrko7UIb6j'
      b += 'nvRYuW63TrObm61V5pgFIeADuHTDi3jPC97EbOuYZ1otsoTdrqCTER1sLCMEZCbwB3d80p82'
      b += 'aP/ygVtZMwh4V86e8LSGj8Aa+jZsbzbWbgppFPsysAH/7Kq3MxhMU5K/WD2QIdnOmMEvs5LT'
      b += 'GoJ6wEP50YMNmYrKWy59DZfMbkekqItmt1NyZkYAaZmbnuPTD5+ct13123/usZvR6TkIjmdb'
      b += '5mnLD8dI3qO3cxyzQAoZPOCakdBASrx12+vYvnYboSldhuxWGoRe2jtZlusvnW4hclcFVi1C'
      b += 'DVoqZQW+DqIRU2cmTPHzV/0AZC2D8Spo9krfPDkEKkrC2TM8wN37HvRTBu1tT93tB9I8IRqS'
      b += 'QMLRb2xWxmr/vZ3TpkAwJbgXJcM05LxmKz922esZDAY1VINA+dNN6wTz06YmctwQGSMLtY5j'
      b += 'HcoKB9rAonLR+u28fdt3saQ+oi+KnRxXwcwxbxFRmtlpPn3vTafuaT95/1eJM4UjyjFWSfZg'
      b += '7W184ktrYykIroZZ4scu+W5mp2eWezIphShVqwUrPyO2QwiR0FEf6aJ6QzVWXSlBo/POy1/P'
      b += 'utygVoYFpJItTurCqMSSEBq+tPObHLHWTwm0N++6DZpQGsvWJ629nSilFZBMNMjuXNBs5bXb'
      b += 'XoZqU/SexHDPldZaWAzFAelpL0U/9pS6oo/cwaYWnCSyMa7l7dteSyaPPr+yy2LLFC/q85LB'
      b += 'G0QMEeOwLvKtJ+4+eU97796HPZGAumlMeypibyfIB7VWU91ocuAdF7+K6TgYHe5JLWKhequz'
      b += 'hDXXkUocxwcNb7/0WmaZIySv4ucnxo9KBKwoRgJzc+u4ddddJw/ar+y8gzA7RdPdFD1mezuB'
      b += 'BaTIjkpgRga8Yet1MCgTXiOw1nlYEz/pXO+MuLC6y8cyCsxNbeDVG69gGI3GlCT5mGll93HO'
      b += 'GdHC2XcvOuC37r3v5EF7+74HSmjcplJs6qmKvZ04PgYgWeLadVcyPT1NIJC7dR8+HqsrhVg5'
      b += 'a3jq5UJyVCPZoYlTvOH8l6PDOmsrZRb4WLm7C6hWKZ4soxrBo4tPsm940E8I2pyGfufe+yEb'
      b += 'aUoJJkXUubfeTsKapLxx+0todBq86F5P7n4daQRnWyVnPDO9bJmrFaDMACvw4o3bOT+uJyfG'
      b += 'IuzH/T4l5w9S2koSHKam+NaTD5zY096z/3F8qvBUNKVye/Y8xd5OdOikRGTTMuCa9Vfgmupk'
      b += 'amEhdwd8RNAJ+qxstTs9vGydkSUi4ojCdDPLdVuuAoWwSn658ikRLZrP3XtliaZp+Naue08M'
      b += '2gf2PV7iahVMS8m6Zzz1dsLD64U/+9KNl9EMBCUeNTozAqksT7nO9NzW3Un4uLjm0KBcs/Ey'
      b += 'crYJofPV37fO0yKGSeVuS4BsPLD41IlB++ChnWhoaDI0xrJB3t56O97BFVeuWX9J4dB2w+Cr'
      b += 'ARzOuo5EdEGsXkDu5Bh42YYXMGNHcxy6gf/lz3V19k4oTlBxHjuy+8SgfTLtqvSyVJ/uEdvb'
      b += 'ibysIOpgLS/ZdBmNTHUulFDl087q31+k+FkdT/cEUaabAZfObSd4GTQYaUYJSOU+dA6x6D8r'
      b += 'LmUTh3iRlz2ohzi4OO/HBe2uI/tKOZ5QC1l9PtvbSXhaE4IJ56/ZxISQyTI1/7MduCX+tTI0'
      b += '75mgDdvWbC51IZvUAZdVUoLS05VK67QAqkr0wO6Ffcf3tLvaA4XVohwzDu+tt9Vs6+wWBpU1'
      b += '5J7rUfRz596XBBTqb7ebaPvMeVguKWae0AMv1CpbFjpXTccy0ugZsbIL64kje44N2sPDeW+D'
      b += '183Zsmxpbm+9HT+nhW2DdagM6hb3IiejhHNCQtcsgTTF00oAjBAaLpjaXN4cSi8251SASwIP'
      b += 'Ja2oea7Vx8ntBCLKnnb/cUC7dIS6eWGklN5bbyd3aI0tg40lg1Udedpz48IqK0uMFlwxS0X8'
      b += '3J3N0+uxXMLnssSrygsTK99YRzmt1seiEV3AmxQOt4ePDdr5NCRa+UKtLr3ckn1e29vxTV2Z'
      b += 'kapV7N1Kj3Pj5HSMKKHByajG6lmFuTgYkUpUA2YZ1VJscluu1mh1iEJ9ojUGLOT22KBdslTe'
      b += 'bCv9WXcnu/caUL2dlKednZ4uWwE6ivyIpnj2p1fuUmRzJJR8tPhNBrHBrOSuZpkQ4jinnQB9'
      b += 'p9bo5MrNHk8QtTkfG7TJCo3K3QlShaN7sfHeTtLbNCGiWplOdVZW4Jw4Q+V31pKXdlGGQKgc'
      b += '6yIxW5Zld1v0ljPC6ntkZYC+W9ylXgpYxwQtQcjWlsVaZgQragQ9uaK34+VzOZc52dwadFVj'
      b += 'P/cE66sOOUULCyQ70WPlJ4Oa4cqq+5vFizSNhzj6+yiKOQb+FCBW/mOut0BWesD2dlzAdtM9'
      b += '4qUmMl4vyWgU75yJNmB5o0WlbNkbgayGwSxfGNaxDtV9RO8crRNBmA7NsUE7TUBC4Tx2GwSy'
      b += '9rrGva30CpSLfaI1YZ5YrFVTSF06ds6lV5NQMZz5vFAUKhESEJKPhBGXhdYrawS1pmRmDIjH'
      b += 'Bu36mTWFuRHLoui+7dPbqoCtaxwnt8IJgYNLtTVhcYUQ+blziJbJ0JhzYHgEEWGYU1nCFQQ1'
      b += 'L9sY6vt5dPgrI2E4VJgbTB8btHNhGg8RSY6FwpXsGVG9TdrkwmSpBRN1QIzdCwcg5bIEulZP'
      b += 'S/h3jh0irzPEruw6uIecnCjKUJ2QjITTaFgWBq+WeiiCibB2as2xQTvVTMusBzzqRJzdbxDo'
      b += 'beKgrOJpMyUX2zm/txILInhZn+GcOyGyV+9afGXJW3fO7y9OtxZ2u00M49xflhecKntsJBjg'
      b += 'znmDtccGLcC2uY0j/Z5uhEjpY+TelgO3A6x7WfWYMfbOH2BvnsfJpe2hzurDeWehWRV2m1is'
      b += 'm6zlocNPIupo7cWG7HjUUVqR6TSS9aic1nCiB7bPnnd80G6ZWrNMgLwvQvV2zHOKj7yBBCVr'
      b += '4v7dT6Ctlt6i+xkgkPos57TVQ+KZJJmH9j0BONmsbByYeM8m6wSj93GCeywiDCSwbc1WOS5o'
      b += 'L5m+CM9ldUPW2gSenLXqrbeJwxakwteLd71r77cZYigl6XXXs74Q5e6YFiFz8470H3h47052'
      b += 'pQMEpM61e90FJMvIEyVqcUSchODitQAsXDC7/tgRT/fB9jWbylpLK2Vql+LCe+vteN4lFSoQ'
      b += 't+26D3IR3Pa8UtT77CRclL3NlfRfw9pFhtzz1IPlc3Wv8ySnYjKXHanDZIj4aEE3wMVrtp4Y'
      b += 'tJev3U5rmUYUz3bUC/TW22o2cCWrcd/Bx9m9uKeM54UyW8tZpAe1mpd1L7mnAOSEqhJS4uad'
      b += 'd9BIGaaAsWxsF0J378Wk2F2Z+glkMhnjgultJwbtC9ZeIJubNXUmsi5AyH0hqrfjH1pQGnGS'
      b += 'ZL6042tkKxqMVjnIKz3t2aV5LPVeMjxEshtPze/j9t33kUTJXaW4E2ivjzahZpG78BoHMoHI'
      b += 'tAgvXXPRiUEL8PLNl5NCyWfLtH3f8+ltdcCOcjoxpM1IUL708K0Mc0Kwo/TbJr3M5OOZfml1'
      b += 'qzDJRs4tX95xJ0skRFrQMErrO6C6j7cJmFCE3UlFcFYKPznHyIvOf9HJgfbF615QGuhRjuJJ'
      b += '9tbbah5S3Wmj0rjw7f2P8cjuh5dVSbuPjxUenvGeFsHNICiWhM8++hWiRJwA3o5+x8DRl5U6'
      b += 'SE5EDyTppFSFV665hNla6jshaF+18UUEItpaT6zo7ZiHdfLjLIJKHOWyH77rv2GLw8r40WUA'
      b += 'PRs87MoLTF3KKHqb+Orjd/Dw3kfKHqM8VmEsRSmW/e6jxzrdM3ABDwRzrth42XFfdxk0z589'
      b += 'TzbOrCF0S3F7R9vbMTzM+ONQVqKqQoC/efwOHjm0C09HE+NXAvisuMDqCLG1xp/e8VmsKTpR'
      b += 'OdRpuQmATq6+7P5kHKu5bVlYLbxq3SmAFuCGC6/FCIhmXI/eu9KHzH14DOCitSVotXWhuBiu'
      b += 'i3zk9s9DXS1TVkGWIzv578t5Ov0Lncvhxnicp1uYh2Mp8bWdd3P7nvsZuJNFUctHLV8fbRUQ'
      b += 'So9IFc1FMlVxxI2tc+dx5cZL5ZRAe+OWa8ubb3EZuWI1zmRv57apl+FvbbSse8yBgTR86uFb'
      b += 'uG33PbBUwepeBc9ykWbpYOCnfw4mRhmC8DFroVxShViiBkfSkP9860dRVTJCDgEJOlKAGT3K'
      b += 'SqZhnV8XMA+g8D3nXXPi933lE9tnt3LlhktA8lGUq257dT8B1IfI40fHMDQ4EgNLKNJkfu8r'
      b += 'f8qRdASpzCiZyO08U9ZLnwnnqIqsiQhieUQzLOIxAfPEp+++iQcPPIQ1SorCwJYX3CY526hM'
      b += '7PHpPjDEy+jejVtfeeqgFZAbt15bdG+6F0WWjWb1s7bnXji8ErBd6OdRizNVwXxICEXa7O6D'
      b += 'j/OJu7/AcDisOWwmISiBKo18RvDtvLjamm+G0T2jLnhrPHrgCf6f2z6Oxmk0O407pokBOlo5'
      b += 'axOA7VQsJj1umZxyrtpwKdvmtsgpgxbgjVtfzuygIdRRIlcjmJDro6uNFgf1dm541ZU2irYM'
      b += 'GiJeNy66Z4Ip0zHw/m9+jG/veYi2bQEldBKhGIac9oVOdx+pS+aax3aFJRFhIS3y21/4Q1pf'
      b += 'INDgWvNZhGHVMCZo4TtoIVe0WJkMqh5b8UKwEOdN268/ubRktSdnmxl529bX4t1CYFNcM8EC'
      b += 'rhmpj72dm563O3AighBIaqgrkVjG8tQZipOC8dtf+gMOzx9CUuddSqhcNln4GXBhKa6O5ira'
      b += '5mWUZri0wB99/ZPcc+AhXAKZtoa+CiZoKPrHnYyqVPWKplaIu/cye5EN2Dy1mTedf608bdAC'
      b += 'fP+lr2UgVVhKxkAdAdb7CaBz3fOW2c883vaugSBlp406RGl4fHEXv/U3f8Kh4SGwrt5c+iTd'
      b += 'wPjp2pHoKt9itRdLBoF2uMRf3f9VPvLtvyKFMlfsmmikAFKasrdHDEII4yKU2WhRe3nPKjfZ'
      b += 'lB+95IaTLwAe6xObBhvkzduvrT+9goKY1h9G6bcPnJvh8Qhg1VuoKhEQcQJ1bjTEEaPO1fn6'
      b += 'E7fz/pv+Kwu0uGtJrSSPCzync0fCMy6C51Lbadslvv7oXbzvb/9fYIGBB1L0sl3AteTrUqiK'
      b += 'KSxXpezm1SdZYhaULdPreev26+UZgxbgBy9+E1PqKIXxQhWkTqHTCjr+6/TkjLOrELUSYF1b'
      b += 'w7p8NvhYtFzH4XOOmY8/9CX+y80fZymlUrga+2vG/drSDz2qJ3qSRbKn9wvWhQhdYW1EFq4q'
      b += 'LoBLiwQlpcTdO3fwv/z1B0hhCSSQKDpQQhixnoyAI8S6KNqXrY4tO32taJwjnviBi248pR/5'
      b += 'uKA9f3qtvOPiG8neFkIzCcOZzoJmPWFe29eqzk4Ar6QyduCUusWte041IhKIGVydP7r7s/xf'
      b += 'X/4QbV7ELXSOrJxez2BFjsUtgTOqvp5qkexUzKS0nrrucal9V7BhZA8oDfPtAt946B5+9bO/'
      b += 'w3yer0U3YaCBTIkoNAriVvNZRWpRakRM6iR4PBAorMNLpi7gBy65Xp410AK86wVvZNNgEy4+'
      b += 'mvlbUkcl4abL3sCzjVva28mDuaxuNEKo+1fVMWlJ0hayAQ0iwsd3fJn/9fN/yEJ7gLZdKt6a'
      b += 'TJJCLshexc7FSi93OcSe9Z9dqzid1o3sGCSUFkeSEnDmU+Jr936D//FL/yeLngkhlGXaQWi1'
      b += 'iP0Ty8WjMYxkUC12m2dzUT/WCDgqhlpmGIT3XPW2p/Ezn8Cmw5T87Iu/n+iRrGDBkOQgAZmg'
      b += 'OZqVOL577O3symmPUlyoF7SIILHBUSQ0uCgaGkBRiTQCjShRIcsSLvDZR27mVz/5fp44uJvU'
      b += 'LuEEgjtlWMZGMqTLdaYqN/fZBu7k+hsxUIgYDY7rkMX5w/zx3/0pv37z+xl6wDSRicQYGQZo'
      b += 'rPCGxQ0GZRUIsYTKmr167IDo1KiS7ARaDbx587Vct/lqedZBC/CazVdy7XmX0SSnQYsyQe3H'
      b += 'jb7RBGDLsqHezhYvepQgmUhd/1EoEu4J1Rr+dVVWzZgYmoVFSbQKQWcIVkB95/67+MU///fc'
      b += 'tON2cruII5RiqhJWXUatTwu4J4z6dPJCKj1UXFlojcf27+ZXP/cf+PDtnyMSaeMCqjMEazFP'
      b += 'KEJqIFJSAW1B6ioPEaFkAF7eKzIiSlAwy8yg/NRlb356F+nJhrJPLe719379f+fw4pCmJtuL'
      b += 'ITPI41GjSeD2IfI5kt+6jNZYiNVlXGbkNmFmWMoFZimXr9FITkNUplE3zBNv2X49//j6H2Hj'
      b += '2g3EWOiB3fSMjF5rvGl9Ne2plZEAtaLtvnp1etmInEoNxZWchqQ85JN3/TW/f+tnmE8LhJhp'
      b += '1YkZTBsIZU42SERjIAchooSpplSaB4Kq0jjkUFo7mh1vHEsNGub5uSvezdu2XyvPKWgB/u6p'
      b += 'u/03b/+DsuXLjShhtAls2QRDb+dWZbljziUrQM0ZspXaUm6RRCk8ZcN8EZcG8VyqrSZAYkam'
      b += '+cev+BHectV3M5hSYhyMwuOVwDzR8MoY5JUc4SfSqkp4a+ScuW33Q3zw5j/jjn2PFApjGMua'
      b += 'ajJ8IIgpEss0osVIEyJSi086KJ7WVZCgNbcdoqkpV5AnXnf+dfzy1T8EYfDcgxbgP937Cf/0'
      b += 'YzfhboQq/Thu7XSbrHsve04VoWrbxMwKWM3w5MW75hZy+ZyZ4RZKhbWj8jnkur81mnHh9CZ+'
      b += '+GVv4c1XfBezcXoEgu58dUvPj+U9J0f+Og2r1cDanfucndwm7tp1Px/65qe45ak7aaMTEaIL'
      b += 'OZcN7wY0OkXLkKihKMnESBCFCBKFUCmLogFCyZXVDKu97CSZi+fO5zev+znWhbmn7d7k6YSx'
      b += 'v/x3/9HvPfJIXag7JEtDQDBvUZlCcyaHjBBX3ck5fnU7I8azejsJj2tlFyt57G27cNmz4Tlj'
      b += 'mZFCo1f6nifFo0FwJCVEGobJOH+wnu+5/JW8+dLXcOGWC5jWAUG0VKlHnYqwanGsS9WWdzOs'
      b += 'ajAJng2zxN7Fg/ztg7fyuftv4a69D2GhEP5dlOxS2jIiuCU0lqEZIZTvHYuIouoM1hgDrcMT'
      b += 'WqrgqqW4phJYklzCZYF///J/yqUbtj+jePRpgXbPcL//yi2/x87hXgYeaTGilmpZVqpk9RiQ'
      b += 'RWtOMbHCX5aySFf7vPesKlaV0dnSQrGcycmRnDADSw6W8G5sLRvWFZwydZN6U2RJBVqr+krJ'
      b += 'uWLzxVx/4TW8bOsVXHPBZTShhs4qBcihFj9VwAQJhnjA6igdqegvLS4tsXNxP7c9ci/fePwe'
      b += 'bnn0LpbCEAngEgjWkhFcHaRBEhBAMUwF0Vx7zw6xKV2SBqIEUqNMSUBUyzBFSBgNkUQiEjzz'
      b += 'iy/+Sd54/stA5fkHLcAD84/7r3/tA+zLR4g2IMqQVDemlTEkrV62zFCqO+JC1oy6joDc29lj'
      b += 'ZlbQl0tro+S24DlVEAtYgiwkyWgWAgFnCfFIkpbozUi5MEkk5OI1WxJqzrRGLttwIZdvuJgX'
      b += 'bLqQC2Y3MzMzQwiB3GamZhrapYyRECI7dj/K7qVDPLD7Ie479ARPHtoN6oRQwGle1rtGaqon'
      b += 'UtpW1hJEyVEqbRc8RprgmJYebQFtAapqrCFxJkSHHJHoWAZFefcVb+OHtr+B8CzsS5FnUuV9'
      b += 'cM8O/+U7fp9si7QeCIBJ0bswT7UKWG9T1xoOB8Z1Qeg5zGe2TW5583oxF+9ZCjtSw+WcBElL'
      b += 'JCvgjhW42QPBFJMFpO63DUiJ3nLh8qaUCKJlx2vNgbHCSAqeStibDY+gOYBmxGMpMAkEibRS'
      b += 'hMUBgiptHpIjxCylyiwBJ6M2QEh1R1GJFlXKyF2IjmjpPXvTFLmYKMQQiicOQiQzZEDEyFEI'
      b += 'SfnBi2/gH11+I6LxWSnTyjNtzXxlz7f8397xEXJOpVDgThKI6ssYUx1QpewL6/PZsz23rcD1'
      b += 'WpyynEhJISfEndYTIYXicduWoDMMvaXxQMqZEOvYnxcQQ6E3ulAHya2OyUUEI4sSEbIYkrS0'
      b += 'ZSjhdnJjoELCCfWS6RpHSiAbeKw/vyQalOQJCUVySdQJUpQTJQBxiqgGsVA3XcvAgCq0agy0'
      b += 'eFrXzI3nXccvvPQfwLOo0yHPRj/1K7vu8ffd+WGOyBIxl5/Pu7UQ5VIsUpPuZQFRN+onvac9'
      b += 'qwA7OlRVyNzG+a1lEGvJJljKSIJMLmNvriRfQiziYgQfV6U7/nGhVGjZqi4lz3QZnykZtZ8E'
      b += 'NIPHAiYrfN9WhFgnj4KUyq5lRl7VM6COEmg1E7SwuoJaaf3EhqCADohBCsiDQHACDao1xzah'
      b += 'DRDMecuFr+Nnr3wLUaee1UaoPFskiNsPPOi/9a0/ZGk4pA0llynRsY5W+BU4Z6j7X7T3tGdl'
      b += 'mLwSuMXjZiyXnNYz5JyLVlRp5kIu25DdhewlHO4qzOX8WK1ZeREDn3wdGVMuMpngDUhCXMfk'
      b += 'CS/yL4qR6yB/I91rFnWWBiUrqIdSzRbBJaBRUHE0BDwKogXAHmIdSSxsrtYNbSJi8A8u+15+'
      b += '4gU3FMojcnqCFmDH4Sf9N277v9mzeKCIMOeuwT0hZiXeh8bnkNftKsqecunfupdqcWVImQsh'
      b += 'JbIb2QW1XMfbipt1Xy61GpYNpBzNjgpSduR2vOhOGaL7SvEyUpjMCCKj76yuWCiDDyK1naSx'
      b += 'VIybQXntEPBQcmJVxVRGNEjxUFpGGd5z1ffxjgtfA0T0OVjUK8823fCp+f3+P9/9IR46+DCm'
      b += 'AUmlAeTuEAyxQBBI5cVLg7320DoxOfXxwqKeYXWGAzj7skJVB14xx1J9rharpPsak2X93JWX'
      b += 'gbKcYLFy1cjKM72SjNF9bFD6vp0D0VIRtjrE3s0G66CE5T6IuMrIs+aoBGIpm8mA6Rj4hat/'
      b += 'lO8+76WFbaLPzXS/PBccYbPkv3/fJ/jzR7+KaMIIaDZElKRD8IbojEJkkxLiBKvqAFhho+jk'
      b += '4rUJ/b7eU59hxSlGFd6uHdRVlTEn51LAKvxlIbmVnHUCwDKZt1bAHevsrsqYqtKly6RMO5mc'
      b += '0Whh0XAWQlVODBDB1JkKAQ9N8cIKbXSmXWk9IJ550YYL+ZVrforzpmdJ3tC4I0HPHNCOC1R3'
      b += '+O/e/VEO2CKNB0wWEWuKpjI6CpkVyra+1pA6j1mYKCt/51LE6gF75piN1jxOUAvrkEkhXBTg'
      b += 'JmtRp1SOXWlz6ecXuZeJLXUeRgMKk+HtJGB1hUd1GY22E0TqKB1VVUJGgy4xxlKd7rxsAFNB'
      b += 'QxmQGQQliRMitUJW2jxvv/gGfvqFbyVIi+jgOY8N5bmexnl0cZ9/4O6Pcev+2xGfplUjeiR5'
      b += 'W6iP9T8klncLz7loT5oj2kzQIDvAhvFjX3k+AypUVg64jpeVuzukqtjvRs5G9ECbE2qQvVIf'
      b += 'a9HSzMYFKS/hrOEjKuPxz7ChhJGHLb1eG6n+azDcxuT+UHuuQbzKn5YpHQ+Fohi8xaoix9aZ'
      b += 'TfzM5d/H9Vuufl6TOHm+Rui+vOtb/sH7Ps1T7X7UWpwIlNDCC/uMJgttqEoCCGO+dxcaSw/Y'
      b += 'Mza3tSoEDJqlKFRUSiNex/bMSO4Eg2S151uB3g2QqypiUoEsYwmXY72uLPe4wamgrHRFVzQG'
      b += 'zBJN09CK04iSo5YqtZZ52ICS1QlaClk/fMHreeelNzLXzDzvVRd5Pudej9iSf/SBL/DRx2/C'
      b += '87A2xQNTBonCW46iLFpiypXS7/UVOW0fIp+ZnnaiuFTz2y5PTeYEh5QLVSJ5GuW+oKPcV7pJ'
      b += 'ouqtjdU3O04WqLq2YldAGj/WllFTpo4klJbQQANLjTBlQioDPARRhjERfMDLNlzOb7zyp7+j'
      b += '5VH5TgyrP7Wwxz/y8Bf5wmPfZAiYD2sYIrRW+mWpVvDGIgbOmbNMoreVRaEiMda1fEq4LCvk'
      b += 'RNUpRagOnCalj0tJfcdUyfH3dUvHXF1SCkxxWfFJKaEwjENkqX0ZVS1ANUhNGc0rNEjlpesu'
      b += '44cueyPXbbriO97PkO+kwsSB9qB/9KHP86knbqNtF8t/SgyEYcZDWT+yXBS9B+wZ63BXtGLK'
      b += 'xxPg8uVqEmLLW0V0gB21go5evdp9n+VFKJZpli0rTnXrOjBCVLIXmmMWaKok6nUbX8y7Ln0d'
      b += 'V226/LRpPsrpIAuzLx3yzz96C5/f9U12Ht5VidphOVCNOu2RJ0Sfy+eLun1l0OSOydIvxf6O'
      b += 'etiTPeKjSKqswuzAN2I82YQ+1Sp6VZ2S/6j3KuM2oUiZKsOEKIHaSKKJSis1d45Co2Uxdjk3'
      b += 'whpdw4du/JenLUNATjctpzsO7vCbdt3Fl5+8jSNLB2ll+XCzR0WSFZq4lkJE9/8eCKO8ySjM'
      b += 'q8kdu709Fwfo2ZEYkjp/3Xngrmvgq3jgo8Jv1/Hwe6dgXDWmGheGjdD4OA+2JhBMIEMMZTfP'
      b += 'dJjhFZsv44ZtL+N1579CTu/3/DQeRH/i8C6/dd99fPPgg9yx/wEODA/TiBKQsq+4WidjmRAG'
      b += 'tUVgoadSPR+mz/D4jFvx40Kje6KjJwa3ZaDtLvBuvUb37y2XHuyILovUC74KvGGEEEhWtM1A'
      b += 'uXrzRbx84+Vcue4irt70QuZkRs6Mi/IMUo945NBOv+fQ4zw+3MOe+X08cuQpHj70JLkWOEoP'
      b += 'vzCvJosVvZ0GYfCxvHT3fySTS926UNmB1SVlur+XKdhxJGZ42VZXv6bJcN7MRi5as5VtU5u4'
      b += 'ZN35XDSzlSvWbGN6MHNGHg45W6RO9yzs873DI2iZwB/tVemlXJ/jA/RMAM+k4FqYINLYaEJH'
      b += 'ZXUO8ihlkvFziqBS9k5NEbhk44Vn3Y1tZv7/DwAE1bJiWLNgDAAAAABJRU5ErkJggg=='
   return b

endclass
